@@ -198,7 +198,7 @@ class ConfigItem(object):
         element (e.g. 'astropy' if this is 'astropy.config.configuration')
         will be used to determine the name of the configuration file, while
         the remaining items determine the section. If None, the package will be
-        inferred from the package within whiich this object's initializer is
+        inferred from the package within which this object's initializer is
         called.
 
     aliases : str, or list of str, optional
@@ -598,7 +598,7 @@ def is_unedited_config_file(content, template_content=None):
     # The jquery_url setting, present in 0.3.2 and later only, is
     # effectively auto-generated by the build system, so we need to
     # ignore it in the md5sum calculation for 0.3.2.
-    content = re.sub(b'\njquery_url\s*=\s*[^\n]+', b'', content)
+    content = re.sub(br'\njquery_url\s*=\s*[^\n]+', b'', content)
 
     # First determine if the config file has any effective content
     buffer = io.BytesIO(content)
@@ -693,7 +693,7 @@ def update_default_config(pkg, default_cfg_dir_or_fn, version=None):
 
     # Don't install template files for dev versions, or we'll end up
     # spamming `~/.astropy/config`.
-    if not 'dev' in version and cfgfn is not None:
+    if 'dev' not in version and cfgfn is not None:
         template_path = path.join(
             get_config_dir(), '{0}.{1}.cfg'.format(pkg, version))
         needs_template = not path.exists(template_path)
