@@ -2,6 +2,8 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import math
+
 import numpy as np
 
 from .core import Kernel1D, Kernel2D, Kernel
@@ -9,16 +11,16 @@ from .utils import KernelSizeError
 from ..modeling import models
 from ..modeling.core import Fittable1DModel, Fittable2DModel
 
-__all__ = sorted(['Gaussian1DKernel', 'Gaussian2DKernel', 'CustomKernel',
-                  'Box1DKernel', 'Box2DKernel', 'Tophat2DKernel',
-                  'Trapezoid1DKernel', 'MexicanHat1DKernel',
-                  'MexicanHat2DKernel', 'AiryDisk2DKernel',
-                  'Moffat2DKernel', 'Model1DKernel', 'Model2DKernel',
-                  'TrapezoidDisk2DKernel', 'Ring2DKernel'])
+
+__all__ = ['Gaussian1DKernel', 'Gaussian2DKernel', 'CustomKernel',
+           'Box1DKernel', 'Box2DKernel', 'Tophat2DKernel',
+           'Trapezoid1DKernel', 'MexicanHat1DKernel', 'MexicanHat2DKernel',
+           'AiryDisk2DKernel', 'Moffat2DKernel', 'Model1DKernel',
+           'Model2DKernel', 'TrapezoidDisk2DKernel', 'Ring2DKernel']
 
 
 def _round_up_to_odd_integer(value):
-    i = int(np.ceil(value))
+    i = int(math.ceil(value))  # TODO: int() call is only needed for six.PY2
     if i % 2 == 0:
         return i + 1
     else:
@@ -533,7 +535,7 @@ class MexicanHat1DKernel(Kernel1D):
     1D Mexican hat filter kernel.
 
     The Mexican Hat, or inverted Gaussian-Laplace filter, is a
-    bandpass filter. It smoothes the data and removes slowly varying
+    bandpass filter. It smooths the data and removes slowly varying
     or constant structures (e.g. Background). It is useful for peak or
     multi-scale detection.
 
@@ -603,7 +605,7 @@ class MexicanHat2DKernel(Kernel2D):
     2D Mexican hat filter kernel.
 
     The Mexican Hat, or inverted Gaussian-Laplace filter, is a
-    bandpass filter. It smoothes the data and removes slowly varying
+    bandpass filter. It smooths the data and removes slowly varying
     or constant structures (e.g. Background). It is useful for peak or
     multi-scale detection.
 
@@ -791,10 +793,11 @@ class Moffat2DKernel(Kernel2D):
     _is_bool = False
 
     def __init__(self, gamma, alpha, **kwargs):
-        self._model = models.Moffat2D((gamma - 1.0) / (np.pi * alpha * alpha),
-                                      0, 0, gamma, alpha)
-        fwhm = 2.0 * alpha * (2.0 ** (1.0 / gamma) - 1.0) ** 0.5
-        self._default_size = _round_up_to_odd_integer(4.0 * fwhm)
+        # Compute amplitude, from
+        # https://en.wikipedia.org/wiki/Moffat_distribution
+        amplitude = (alpha - 1.0) / (np.pi * gamma * gamma)
+        self._model = models.Moffat2D(amplitude, 0, 0, gamma, alpha)
+        self._default_size = _round_up_to_odd_integer(4.0 * self._model.fwhm)
         super(Moffat2DKernel, self).__init__(**kwargs)
         self.normalize()
         self._truncation = None
@@ -1004,13 +1007,13 @@ class CustomKernel(Kernel):
             raise TypeError("Must be list or array.")
 
         # Check if array is odd in all axes
-        odd = np.all([axes_size % 2 != 0 for axes_size in self.shape])
+        odd = all(axes_size % 2 != 0 for axes_size in self.shape)
         if not odd:
             raise KernelSizeError("Kernel size must be odd in all axes.")
 
         # Check if array is bool
         ones = self._array == 1.
         zeros = self._array == 0
-        self._is_bool = np.all(np.logical_or(ones, zeros))
+        self._is_bool = bool(np.all(np.logical_or(ones, zeros)))
 
         self._truncation = 0.0
