@@ -649,7 +649,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray):
         Return a replica of the frame, optionally with new frame attributes.
 
         The replica is a new frame object that has the same data as this frame
-        object and with frame attributes overriden if they are provided as extra
+        object and with frame attributes overridden if they are provided as extra
         keyword arguments to this method. If ``copy`` is set to `True` then a
         copy of the internal arrays will be made.  Otherwise the replica will
         use a reference to the original arrays when possible to save memory. The
@@ -678,7 +678,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray):
         Return a replica without data, optionally with new frame attributes.
 
         The replica is a new frame object without data but with the same frame
-        attributes as this object, except where overriden by extra keyword
+        attributes as this object, except where overridden by extra keyword
         arguments to this method.  The ``copy`` keyword determines if the frame
         attributes are truly copied vs being references (which saves memory for
         cases where frame attributes are large).
@@ -1072,8 +1072,18 @@ class BaseCoordinateFrame(ShapedLikeNDArray):
             data = self.represent_as(rep_cls, dif_cls, in_frame_units=True)
 
             data_repr = repr(data)
-            for nmpref, nmrepr in self.representation_component_names.items():
-                data_repr = data_repr.replace(nmrepr, nmpref)
+            # Generate the list of component names out of the repr string
+            part1, _, remainder = data_repr.partition('(')
+            if remainder != '':
+                comp_str, _, part2 = remainder.partition(')')
+                comp_names = comp_str.split(', ')
+                # Swap in frame-specific component names
+                invnames = dict([(nmrepr, nmpref) for nmpref, nmrepr
+                                 in self.representation_component_names.items()])
+                for i, name in enumerate(comp_names):
+                    comp_names[i] = invnames.get(name, name)
+                # Reassemble the repr string
+                data_repr = part1 + '(' + ', '.join(comp_names) + ')' + part2
 
         else:
             data = self.data
@@ -1251,16 +1261,18 @@ class BaseCoordinateFrame(ShapedLikeNDArray):
         return self.__getattribute__(attr)  # Raise AttributeError.
 
     def __setattr__(self, attr, value):
-        repr_attr_names = set()
-        if hasattr(self, 'representation_info'):
-            for representation_attr in self.representation_info.values():
-                repr_attr_names.update(representation_attr['names'])
+        # Don't slow down access of private attributes!
+        if not attr.startswith('_'):
+            if hasattr(self, 'representation_info'):
+                repr_attr_names = set()
+                for representation_attr in self.representation_info.values():
+                    repr_attr_names.update(representation_attr['names'])
 
-        if attr in repr_attr_names:
-            raise AttributeError(
-                'Cannot set any frame attribute {0}'.format(attr))
-        else:
-            super(BaseCoordinateFrame, self).__setattr__(attr, value)
+                if attr in repr_attr_names:
+                    raise AttributeError(
+                        'Cannot set any frame attribute {0}'.format(attr))
+
+        super(BaseCoordinateFrame, self).__setattr__(attr, value)
 
     def separation(self, other):
         """
@@ -1270,7 +1282,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray):
 
             If the ``other`` coordinate object is in a different frame, it is
             first transformed to the frame of this object. This can lead to
-            unintutive behavior if not accounted for. Particularly of note is
+            unintuitive behavior if not accounted for. Particularly of note is
             that ``self.separation(other)`` and ``other.separation(self)`` may
             not give the same answer in this case.
 
@@ -1289,7 +1301,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray):
         The separation is calculated using the Vincenty formula, which
         is stable at all locations, including poles and antipodes [1]_.
 
-        .. [1] http://en.wikipedia.org/wiki/Great-circle_distance
+        .. [1] https://en.wikipedia.org/wiki/Great-circle_distance
 
         """
         from .angle_utilities import angular_separation
