@@ -166,9 +166,9 @@ class SkyCoord(ShapedLikeNDArray):
         only one unit is supplied then it applies to both ``LON`` and
         ``LAT``.
     obstime : valid `~astropy.time.Time` initializer, optional
-        Time of observation
+        Time(s) of observation.
     equinox : valid `~astropy.time.Time` initializer, optional
-        Coordinate frame equinox
+        Coordinate frame equinox.
     representation : str or Representation class
         Specifies the representation, e.g. 'spherical', 'cartesian', or
         'cylindrical'.  This affects the positional args and other keyword args
@@ -1007,11 +1007,12 @@ class SkyCoord(ShapedLikeNDArray):
         Returns
         -------
         idxsearcharound : integer array
-            Indices into ``self`` that matches to the corresponding element of
-            ``idxself``. Shape matches ``idxself``.
+            Indices into ``searcharoundcoords`` that match the
+            corresponding elements of ``idxself``. Shape matches
+            ``idxself``.
         idxself : integer array
-            Indices into ``searcharoundcoords`` that matches to the
-            corresponding element of ``idxsearcharound``. Shape matches
+            Indices into ``self`` that match the
+            corresponding elements of ``idxsearcharound``. Shape matches
             ``idxsearcharound``.
         sep2d : `~astropy.coordinates.Angle`
             The on-sky separation between the coordinates. Shape matches
@@ -1065,11 +1066,12 @@ class SkyCoord(ShapedLikeNDArray):
         Returns
         -------
         idxsearcharound : integer array
-            Indices into ``self`` that matches to the corresponding element of
-            ``idxself``. Shape matches ``idxself``.
+            Indices into ``searcharoundcoords`` that match the
+            corresponding elements of ``idxself``. Shape matches
+            ``idxself``.
         idxself : integer array
-            Indices into ``searcharoundcoords`` that matches to the
-            corresponding element of ``idxsearcharound``. Shape matches
+            Indices into ``self`` that match the
+            corresponding elements of ``idxsearcharound``. Shape matches
             ``idxsearcharound``.
         sep2d : `~astropy.coordinates.Angle`
             The on-sky separation between the coordinates. Shape matches
