@@ -14,7 +14,8 @@ from numpy.testing import assert_allclose, assert_array_equal
 
 from ..angles import Longitude, Latitude, Angle
 from ... import units as u
-from ..errors import IllegalSecondError, IllegalMinuteError, IllegalHourError
+from ..errors import (IllegalSecondError, IllegalMinuteError, IllegalHourError,
+                      IllegalSecondWarning, IllegalMinuteWarning)
 
 
 def test_create_angles():
@@ -464,48 +465,56 @@ def test_negative_zero_hm():
 
 def test_negative_sixty_hm():
     # Test for HM parser
-    a = Angle('-00:60', u.hour)
+    with pytest.warns(IllegalMinuteWarning):
+        a = Angle('-00:60', u.hour)
     assert_allclose(a.hour, -1.)
 
 
 def test_plus_sixty_hm():
     # Test for HM parser
-    a = Angle('00:60', u.hour)
+    with pytest.warns(IllegalMinuteWarning):
+        a = Angle('00:60', u.hour)
     assert_allclose(a.hour, 1.)
 
 
 def test_negative_fifty_nine_sixty_dms():
     # Test for DMS parser
-    a = Angle('-00:59:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('-00:59:60', u.deg)
     assert_allclose(a.degree, -1.)
 
 
 def test_plus_fifty_nine_sixty_dms():
     # Test for DMS parser
-    a = Angle('+00:59:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('+00:59:60', u.deg)
     assert_allclose(a.degree, 1.)
 
 
 def test_negative_sixty_dms():
     # Test for DMS parser
-    a = Angle('-00:00:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('-00:00:60', u.deg)
     assert_allclose(a.degree, -1. / 60.)
 
 
 def test_plus_sixty_dms():
     # Test for DMS parser
-    a = Angle('+00:00:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('+00:00:60', u.deg)
     assert_allclose(a.degree, 1. / 60.)
 
 
 def test_angle_to_is_angle():
-    a = Angle('00:00:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('00:00:60', u.deg)
     assert isinstance(a, Angle)
     assert isinstance(a.to(u.rad), Angle)
 
 
 def test_angle_to_quantity():
-    a = Angle('00:00:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('00:00:60', u.deg)
     q = u.Quantity(a)
     assert isinstance(q, u.Quantity)
     assert q.unit is u.deg
@@ -524,7 +533,8 @@ def test_quantity_to_angle():
 
 
 def test_angle_string():
-    a = Angle('00:00:60', u.deg)
+    with pytest.warns(IllegalSecondWarning):
+        a = Angle('00:00:60', u.deg)
     assert str(a) == '0d01m00s'
     a = Angle('-00:00:10', u.hour)
     assert str(a) == '-0h00m10s'
@@ -849,6 +859,7 @@ def test_wrap_at_without_new():
     l = np.concatenate([l1, l2])
     assert l._wrap_angle is not None
 
+
 def test__str__():
     """
     Check the __str__ method used in printing the Angle
@@ -869,6 +880,7 @@ def test__str__():
     bigarrangle = Angle(np.ones(10000), u.deg)
     assert '...' in bigarrangle.__str__()
 
+
 def test_repr_latex():
     """
     Check the _repr_latex_ method, used primarily by IPython notebooks
