@@ -4,6 +4,7 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import pytest
 import numpy as np
 
 from ... import units as u
@@ -11,8 +12,9 @@ from ..distances import Distance
 from ..builtin_frames import ICRS, FK5, Galactic, AltAz, SkyOffsetFrame
 from .. import SkyCoord, EarthLocation
 from ...time import Time
-from ...tests.helper import (pytest, quantity_allclose as allclose,
-                             assert_quantity_allclose as assert_allclose)
+from ...tests.helper import assert_quantity_allclose as assert_allclose
+
+from ...extern.six.moves import range
 
 
 @pytest.mark.parametrize("inradec,expectedlatlon, tolsep", [
@@ -45,7 +47,7 @@ def test_skyoffset_functional_ra():
                       dec=input_dec*u.deg,
                       distance=1.*u.kpc)
 
-    for ra in np.linspace(0,360,24):
+    for ra in np.linspace(0, 360, 24):
         # expected rotation
         expected = ICRS(ra=np.linspace(0-ra, 360-ra, 12)[1:-1]*u.deg,
                         dec=np.linspace(-90, 90, 12)[1:-1]*u.deg,
@@ -65,7 +67,7 @@ def test_skyoffset_functional_ra():
         assert_allclose(actual_xyz, expected_xyz, atol=1E-5*u.kpc)
         assert_allclose(icrs_coord.ra, roundtrip.ra, atol=1E-5*u.deg)
         assert_allclose(icrs_coord.dec, roundtrip.dec, atol=1E-5*u.deg)
-        assert_allclose(icrs_coord.distance, roundtrip.distance, atol = 1E-5*u.kpc)
+        assert_allclose(icrs_coord.distance, roundtrip.distance, atol=1E-5*u.kpc)
 
 
 def test_skyoffset_functional_dec():
@@ -80,8 +82,8 @@ def test_skyoffset_functional_dec():
     icrs_coord = ICRS(ra=input_ra*u.deg,
                       dec=input_dec*u.deg,
                       distance=1.*u.kpc)
-    #Dec rotations
-    #Done in xyz space because dec must be [-90,90]
+    # Dec rotations
+    # Done in xyz space because dec must be [-90,90]
 
     for dec in np.linspace(-90, 90, 13):
         # expected rotation
@@ -120,8 +122,8 @@ def test_skyoffset_functional_ra_dec():
     input_dec = np.linspace(-90, 90, 12)[1:-1]
     input_ra_rad = np.deg2rad(input_ra)
     input_dec_rad = np.deg2rad(input_dec)
-    icrs_coord = ICRS(ra = input_ra*u.deg,
-                      dec = input_dec*u.deg,
+    icrs_coord = ICRS(ra=input_ra*u.deg,
+                      dec=input_dec*u.deg,
                       distance=1.*u.kpc)
 
     for ra in np.linspace(0, 360, 10):
@@ -156,6 +158,7 @@ def test_skyoffset_functional_ra_dec():
             assert_allclose(icrs_coord.dec, roundtrip.dec, atol=1E-5*u.deg)
             assert_allclose(icrs_coord.distance, roundtrip.distance, atol=1E-5*u.kpc)
 
+
 def test_skycoord_skyoffset_frame():
     m31 = SkyCoord(10.6847083, 41.26875, frame='icrs', unit=u.deg)
     m33 = SkyCoord(23.4621, 30.6599417, frame='icrs', unit=u.deg)
@@ -209,6 +212,7 @@ def test_m31_coord_transforms(fromsys, tosys, fromcoo, tocoo):
                     [1.0*u.deg, 1.0*u.deg], atol=convert_precision)
 
 
+@pytest.mark.remote_data
 def test_altaz_attribute_transforms():
     """Test transforms between AltAz frames with different attributes."""
     el1 = EarthLocation(0*u.deg, 0*u.deg, 0*u.m)
@@ -234,6 +238,7 @@ def test_altaz_attribute_transforms():
     assert_allclose([coo3.lon.wrap_at(180*u.deg), coo3.lat],
                     [1*u.deg, 1*u.deg], atol=convert_precision)
 
+
 @pytest.mark.parametrize("rotation, expectedlatlon", [
     (0*u.deg, [0, 1]*u.deg),
     (180*u.deg, [0, -1]*u.deg),
@@ -284,7 +289,25 @@ def test_skyoffset_origindata():
     with pytest.raises(ValueError):
         SkyOffsetFrame(origin=origin)
 
+
 def test_skyoffset_lonwrap():
     origin = ICRS(45*u.deg, 45*u.deg)
     sc = SkyCoord(190*u.deg, -45*u.deg, frame=SkyOffsetFrame(origin=origin))
-    assert sc.lon.to(u.deg).value < 180
+    assert sc.lon < 180 * u.deg
+
+
+def test_skyoffset_velerr():
+    # TODO: remove this when the SkyOffsetFrame's support velocities
+    origin = ICRS(45*u.deg, 45*u.deg)
+    originwvel = ICRS(45*u.deg, 45*u.deg, radial_velocity=1*u.km/u.s)
+
+    SkyOffsetFrame(origin=origin)
+    with pytest.raises(NotImplementedError):
+        SkyOffsetFrame(origin=originwvel)
+    SkyOffsetFrame(origin.data, origin=origin)
+    with pytest.raises(NotImplementedError):
+        SkyOffsetFrame(originwvel.data, origin=origin)
+    with pytest.raises(NotImplementedError):
+        SkyOffsetFrame(origin.data, origin=originwvel)
+    with pytest.raises(NotImplementedError):
+        SkyOffsetFrame(originwvel.data, origin=originwvel)
