@@ -8,10 +8,8 @@ from __future__ import (absolute_import, division, print_function,
 import warnings
 import numpy as np
 
-from .core import default_cosmology as _default_cosmology
 from .core import CosmologyError
 from ..units import Quantity
-from ..utils import deprecated
 
 __all__ = ['z_at_value']
 
@@ -25,7 +23,7 @@ def z_at_value(func, fval, zmin=1e-8, zmax=1000, ztol=1e-8, maxfun=500):
     methods (for example Planck13.distmod) is equal to a known value.
 
     .. warning::
-      Make sure you understand the behaviour of the function that you
+      Make sure you understand the behavior of the function that you
       are trying to invert! Depending on the cosmology, there may not
       be a unique solution. For example, in the standard Lambda CDM
       cosmology, there are two redshifts which give an angular
@@ -126,8 +124,7 @@ there is no solution, or that there is more than one solution between
 zmin and zmax satisfying fval = func(z).""")
 
     if isinstance(fval_zmin, Quantity):
-        unit = fval_zmin.unit
-        val = fval.to(unit).value
+        val = fval.to_value(fval_zmin.unit)
         f = lambda z: abs(func(z).value - val)
     else:
         f = lambda z: abs(func(z) - fval)
