@@ -14,31 +14,36 @@ from __future__ import absolute_import, division, print_function
 import re
 
 from . import core
+from ...extern.six.moves import zip
+
 
 class BasicHeader(core.BaseHeader):
-    '''Basic table Header Reader
+    """
+    Basic table Header Reader
 
     Set a few defaults for common ascii table formats
     (start at line 0, comments begin with ``#`` and possibly white space)
-    '''
+    """
     start_line = 0
     comment = r'\s*#'
     write_comment = '# '
 
 
 class BasicData(core.BaseData):
-    '''Basic table Data Reader
+    """
+    Basic table Data Reader
 
     Set a few defaults for common ascii table formats
     (start at line 1, comments begin with ``#`` and possibly white space)
-    '''
+    """
     start_line = 1
     comment = r'\s*#'
     write_comment = '# '
 
 
 class Basic(core.BaseReader):
-    """Read a character-delimited table with a single header line at the top
+    r"""
+    Read a character-delimited table with a single header line at the top
     followed by data lines to the end of the table.  Lines beginning with # as
     the first non-whitespace character are comments.  This reader is highly
     configurable.
@@ -71,22 +76,27 @@ class Basic(core.BaseReader):
 
 
 class NoHeaderHeader(BasicHeader):
-    '''Reader for table header without a header
+    """
+    Reader for table header without a header
 
     Set the start of header line number to `None`, which tells the basic
     reader there is no header line.
-    '''
+    """
     start_line = None
 
+
 class NoHeaderData(BasicData):
-    '''Reader for table data without a header
+    """
+    Reader for table data without a header
 
     Data starts at first uncommented line since there is no header line.
-    '''
+    """
     start_line = 0
 
+
 class NoHeader(Basic):
-    """Read a table with no header line.  Columns are autonamed using
+    """
+    Read a table with no header line.  Columns are autonamed using
     header.auto_format which defaults to "col%d".  Otherwise this reader
     the same as the :class:`Basic` class from which it is derived.  Example::
 
@@ -101,12 +111,16 @@ class NoHeader(Basic):
 
 
 class CommentedHeaderHeader(BasicHeader):
-    """Header class for which the column definition line starts with the
+    """
+    Header class for which the column definition line starts with the
     comment character.  See the :class:`CommentedHeader` class  for an example.
     """
+
     def process_lines(self, lines):
-        """Return only lines that start with the comment regexp.  For these
-        lines strip out the matching characters."""
+        """
+        Return only lines that start with the comment regexp.  For these
+        lines strip out the matching characters.
+        """
         re_comment = re.compile(self.comment)
         for line in lines:
             match = re_comment.match(line)
@@ -118,11 +132,17 @@ class CommentedHeaderHeader(BasicHeader):
 
 
 class CommentedHeader(Basic):
-    """Read a file where the column names are given in a line that begins with
+    """
+    Read a file where the column names are given in a line that begins with
     the header comment character. ``header_start`` can be used to specify the
     line index of column names, and it can be a negative index (for example -1
     for the last commented line).  The default delimiter is the <space>
-    character.::
+    character.
+
+    This matches the format produced by ``np.savetxt()``, with ``delimiter=','``,
+    and ``header='<comma-delimited-column-names-list>'``.
+
+    Example::
 
       # col1 col2 col3
       # Comment line
@@ -135,7 +155,6 @@ class CommentedHeader(Basic):
     header_class = CommentedHeaderHeader
     data_class = NoHeaderData
 
-
     def read(self, table):
         """
         Read input data (file-like object, filename, list of strings, or
@@ -143,10 +162,13 @@ class CommentedHeader(Basic):
         """
         out = super(CommentedHeader, self).read(table)
 
-        # Strip off first comment since this is the header line for
-        # commented_header format.
+        # Strip off the comment line set as the header line for
+        # commented_header format (first by default).
         if 'comments' in out.meta:
-            out.meta['comments'] = out.meta['comments'][1:]
+            idx = self.header.start_line
+            if idx < 0:
+                idx = len(out.meta['comments']) + idx
+            out.meta['comments'] = out.meta['comments'][:idx] + out.meta['comments'][idx+1:]
             if not out.meta['comments']:
                 del out.meta['comments']
 
@@ -161,28 +183,36 @@ class CommentedHeader(Basic):
 
 
 class TabHeaderSplitter(core.DefaultSplitter):
-    '''Split lines on tab and do not remove whitespace'''
+    """Split lines on tab and do not remove whitespace"""
     delimiter = '\t'
     process_line = None
 
 
 class TabDataSplitter(TabHeaderSplitter):
-    '''Don't strip data value whitespace since that is significant in TSV tables'''
+    """
+    Don't strip data value whitespace since that is significant in TSV tables
+    """
     process_val = None
     skipinitialspace = False
 
+
 class TabHeader(BasicHeader):
-    '''Reader for header of tables with tab separated header'''
+    """
+    Reader for header of tables with tab separated header
+    """
     splitter_class = TabHeaderSplitter
 
 
 class TabData(BasicData):
-    '''Reader for data of tables with tab separated data '''
+    """
+    Reader for data of tables with tab separated data
+    """
     splitter_class = TabDataSplitter
 
 
 class Tab(Basic):
-    """Read a tab-separated file.  Unlike the :class:`Basic` reader, whitespace is
+    """
+    Read a tab-separated file.  Unlike the :class:`Basic` reader, whitespace is
     not stripped from the beginning and end of either lines or individual column
     values.
 
@@ -199,19 +229,25 @@ class Tab(Basic):
 
 
 class CsvSplitter(core.DefaultSplitter):
-    '''Split on comma for CSV (comma-separated-value) tables'''
+    """
+    Split on comma for CSV (comma-separated-value) tables
+    """
     delimiter = ','
 
 
 class CsvHeader(BasicHeader):
-    '''Header that uses the :class:`astropy.io.ascii.basic.CsvSplitter`'''
+    """
+    Header that uses the :class:`astropy.io.ascii.basic.CsvSplitter`
+    """
     splitter_class = CsvSplitter
     comment = None
     write_comment = None
 
 
 class CsvData(BasicData):
-    '''Data that uses the :class:`astropy.io.ascii.basic.CsvSplitter`'''
+    """
+    Data that uses the :class:`astropy.io.ascii.basic.CsvSplitter`
+    """
     splitter_class = CsvSplitter
     fill_values = [(core.masked, '')]
     comment = None
@@ -219,7 +255,8 @@ class CsvData(BasicData):
 
 
 class Csv(Basic):
-    """Read a CSV (comma-separated-values) file.
+    """
+    Read a CSV (comma-separated-values) file.
 
     Example::
 
@@ -251,7 +288,7 @@ class Csv(Basic):
     data_class = CsvData
 
     def inconsistent_handler(self, str_vals, ncols):
-        '''
+        """
         Adjust row if it is too short.
 
         If a data row is shorter than the header, add empty values to make it the
@@ -269,7 +306,7 @@ class Csv(Basic):
         -------
         str_vals : list
             List of strings to be parsed into data entries in the output table.
-        '''
+        """
         if len(str_vals) < ncols:
             str_vals.extend((ncols - len(str_vals)) * [''])
 
@@ -277,25 +314,34 @@ class Csv(Basic):
 
 
 class RdbHeader(TabHeader):
-    '''Header for RDB tables'''
+    """
+    Header for RDB tables
+    """
     col_type_map = {'n': core.NumType,
                     's': core.StrType}
 
-
     def get_type_map_key(self, col):
         return col.raw_type[-1]
 
-
     def get_cols(self, lines):
-        """Initialize the header Column objects from the table ``lines``.
+        """
+        Initialize the header Column objects from the table ``lines``.
 
         This is a specialized get_cols for the RDB type:
         Line 0: RDB col names
         Line 1: RDB col definitions
         Line 2+: RDB data rows
 
-        :param lines: list of table lines
-        :returns: None
+
+        Parameters
+        ----------
+        lines : list
+            List of table lines
+
+        Returns
+        -------
+        None
+
         """
         header_lines = self.process_lines(lines)   # this is a generator
         header_vals_list = [hl for _, hl in zip(range(2), self.splitter(header_lines))]
@@ -307,14 +353,13 @@ class RdbHeader(TabHeader):
             raise ValueError('RDB header mismatch between number of column names and column types')
 
         if any(not re.match(r'\d*(N|S)$', x, re.IGNORECASE) for x in raw_types):
-            raise ValueError('RDB types definitions do not all match [num](N|S): %s' % raw_types)
+            raise ValueError('RDB types definitions do not all match [num](N|S): {}'.format(raw_types))
 
         self._set_cols_from_names()
         for col, raw_type in zip(self.cols, raw_types):
             col.raw_type = raw_type
             col.type = self.get_col_type(col)
 
-
     def write(self, lines):
         lines.append(self.splitter.join(self.colnames))
         rdb_types = []
@@ -327,12 +372,15 @@ class RdbHeader(TabHeader):
 
 
 class RdbData(TabData):
-    '''Data reader for RDB data. Starts reading at line 2.'''
+    """
+    Data reader for RDB data. Starts reading at line 2.
+    """
     start_line = 2
 
 
 class Rdb(Tab):
-    """Read a tab-separated file with an extra line after the column definition
+    """
+    Read a tab-separated file with an extra line after the column definition
     line.  The RDB format meets this definition.  Example::
 
       col1 <tab> col2 <tab> col3
