@@ -8,37 +8,18 @@ fixedwidth.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-##
-## Redistribution and use in source and binary forms, with or without
-## modification, are permitted provided that the following conditions are met:
-##     * Redistributions of source code must retain the above copyright
-##       notice, this list of conditions and the following disclaimer.
-##     * Redistributions in binary form must reproduce the above copyright
-##       notice, this list of conditions and the following disclaimer in the
-##       documentation and/or other materials provided with the distribution.
-##     * Neither the name of the Smithsonian Astrophysical Observatory nor the
-##       names of its contributors may be used to endorse or promote products
-##       derived from this software without specific prior written permission.
-##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-## DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-## DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-## (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-## LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-## ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+from __future__ import absolute_import, division, print_function
+
+from ...extern.six.moves import zip, range
 
-import re
-import itertools
 from . import core
-from .core import io, next, izip, InconsistentTableError
+from .core import InconsistentTableError, DefaultSplitter
+from . import basic
 
 
 class FixedWidthSplitter(core.BaseSplitter):
-    """Split line based on fixed start and end positions for each ``col`` in
+    """
+    Split line based on fixed start and end positions for each ``col`` in
     ``self.cols``.
 
     This class requires that the Header class will have defined ``col.start``
@@ -50,9 +31,11 @@ class FixedWidthSplitter(core.BaseSplitter):
     style so line[start:end] is the desired substring for a column.  This splitter
     class does not have a hook for ``process_lines`` since that is generally not
     useful for fixed-width input.
+
     """
     delimiter_pad = ''
     bookend = False
+    delimiter = '|'
 
     def __call__(self, lines):
         for line in lines:
@@ -76,46 +59,41 @@ class FixedWidthSplitter(core.BaseSplitter):
         return bookend_left + padded_delim.join(vals) + bookend_right
 
 
-class FixedWidthHeader(core.BaseHeader):
-    """Fixed width table header reader.
+class FixedWidthHeaderSplitter(DefaultSplitter):
+    '''Splitter class that splits on ``|``.'''
+    delimiter = '|'
 
-    The key settable class attributes are:
 
-    :param auto_format: format string for auto-generating column names
-    :param start_line: None, int, or a function of ``lines`` that returns None or int
-    :param comment: regular expression for comment lines
-    :param splitter_class: Splitter class for splitting data lines into columns
-    :param names: list of names corresponding to each data column
-    :param include_names: list of names to include in output (default=None selects all names)
-    :param exclude_names: list of names to exlude from output (applied after ``include_names``)
-    :param position_line: row index of line that specifies position (default = 1)
-    :param position_char: character used to write the position line (default = "-")
-    :param col_starts: list of start positions for each column (0-based counting)
-    :param col_ends: list of end positions (inclusive) for each column
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
+class FixedWidthHeader(basic.BasicHeader):
     """
-
+    Fixed width table header reader.
+    """
+    splitter_class = FixedWidthHeaderSplitter
+    """ Splitter class for splitting data lines into columns """
     position_line = None   # secondary header line position
+    """ row index of line that specifies position (default = 1) """
+    set_of_position_line_characters = set(r'`~!#$%^&*-_+=\|":' + "'")
 
     def get_line(self, lines, index):
         for i, line in enumerate(self.process_lines(lines)):
             if i == index:
                 break
-        else: # No header line matching
+        else:  # No header line matching
             raise InconsistentTableError('No header line found in table')
         return line
 
     def get_cols(self, lines):
-        """Initialize the header Column objects from the table ``lines``.
+        """
+        Initialize the header Column objects from the table ``lines``.
 
         Based on the previously set Header attributes find or create the column names.
-        Sets ``self.cols`` with the list of Columns.  This list only includes the actual
-        requested columns after filtering by the include_names and exclude_names
-        attributes.  See ``self.names`` for the full list.
+        Sets ``self.cols`` with the list of Columns.
+
+        Parameters
+        ----------
+        lines : list
+            List of table lines
 
-        :param lines: list of table lines
-        :returns: None
         """
 
         # See "else" clause below for explanation of start_line and position_line
@@ -128,13 +106,18 @@ class FixedWidthHeader(core.BaseHeader):
         if start_line is None:
             if position_line is not None:
                 raise ValueError("Cannot set position_line without also setting header_start")
-            data_lines = self.data.process_lines(lines)
+
+            # data.data_lines attribute already set via self.data.get_data_lines(lines)
+            # in BaseReader.read().  This includes slicing for data_start / data_end.
+            data_lines = self.data.data_lines
+
             if not data_lines:
-                raise InconsistentTableError('No data lines found so cannot autogenerate column names')
+                raise InconsistentTableError(
+                    'No data lines found so cannot autogenerate column names')
             vals, starts, ends = self.get_fixedwidth_params(data_lines[0])
 
-            if self.names is None:
-                self.names = [self.auto_format % i for i in range(1, len(vals) + 1)]
+            self.names = [self.auto_format.format(i)
+                          for i in range(1, len(vals) + 1)]
 
         else:
             # This bit of code handles two cases:
@@ -152,51 +135,68 @@ class FixedWidthHeader(core.BaseHeader):
                 # slice col_ends but expects inclusive col_ends on input (for
                 # more intuitive user interface).
                 line = self.get_line(lines, position_line)
+                if len(set(line) - set([self.splitter.delimiter, ' '])) != 1:
+                    raise InconsistentTableError('Position line should only contain delimiters and one other character, e.g. "--- ------- ---".')
+                    # The line above lies. It accepts white space as well.
+                    # We don't want to encourage using three different
+                    # characters, because that can cause ambiguities, but white
+                    # spaces are so common everywhere that practicality beats
+                    # purity here.
+                charset = self.set_of_position_line_characters.union(set([self.splitter.delimiter, ' ']))
+                if not set(line).issubset(charset):
+                    raise InconsistentTableError('Characters in position line must be part of {0}'.format(charset))
                 vals, self.col_starts, col_ends = self.get_fixedwidth_params(line)
-                self.col_ends = [x - 1 for x in col_ends]
+                self.col_ends = [x - 1 if x is not None else None for x in col_ends]
 
             # Get the header column names and column positions
             line = self.get_line(lines, start_line)
             vals, starts, ends = self.get_fixedwidth_params(line)
 
-            # Possibly override the column names with user-supplied values
-            if self.names is None:
-                self.names = vals
+            self.names = vals
 
-        # Filter self.names using include_names and exclude_names, then create
-        # the actual Column objects.
         self._set_cols_from_names()
-        self.n_data_cols = len(self.cols)
 
-        # Set column start and end positions.  Also re-index the cols because
-        # the FixedWidthSplitter does NOT return the ignored cols (as is the
-        # case for typical delimiter-based splitters)
+        # Set column start and end positions.
         for i, col in enumerate(self.cols):
-            col.start = starts[col.index]
-            col.end = ends[col.index]
-            col.index = i
+            col.start = starts[i]
+            col.end = ends[i]
 
     def get_fixedwidth_params(self, line):
-        """Split ``line`` on the delimiter and determine column values and
+        """
+        Split ``line`` on the delimiter and determine column values and
         column start and end positions.  This might include null columns with
         zero length (e.g. for ``header row = "| col1 || col2 | col3 |"`` or
         ``header2_row = "----- ------- -----"``).  The null columns are
         stripped out.  Returns the values between delimiters and the
         corresponding start and end positions.
 
-        :param line: input line
-        :returns: (vals, starts, ends)
+        Parameters
+        ----------
+        line : str
+            Input line
+
+        Returns
+        -------
+        vals : list
+            List of values.
+        starts : list
+            List of starting indices.
+        ends : list
+            List of ending indices.
+
         """
 
-        # If column positions are already specified then just use those, otherwise
-        # figure out positions between delimiters.
+        # If column positions are already specified then just use those.
+        # If neither column starts or ends are given, figure out positions
+        # between delimiters. Otherwise, either the starts or the ends have
+        # been given, so figure out whichever wasn't given.
         if self.col_starts is not None and self.col_ends is not None:
             starts = list(self.col_starts)  # could be any iterable, e.g. np.array
-            ends = [x + 1 for x in self.col_ends] # user supplies inclusive endpoint
+            ends = [x + 1 if x is not None else None for x in self.col_ends]  # user supplies inclusive endpoint
             if len(starts) != len(ends):
                 raise ValueError('Fixed width col_starts and col_ends must have the same length')
             vals = [line[start:end].strip() for start, end in zip(starts, ends)]
-        else:
+        elif self.col_starts is None and self.col_ends is None:
             # There might be a cleaner way to do this but it works...
             vals = line.split(self.splitter.delimiter)
             starts = [0]
@@ -211,6 +211,15 @@ class FixedWidthHeader(core.BaseHeader):
             vals = [x.strip() for x in vals if x]
             if len(vals) != len(starts) or len(vals) != len(ends):
                 raise InconsistentTableError('Error parsing fixed width header')
+        else:
+            # exactly one of col_starts or col_ends is given...
+            if self.col_starts is not None:
+                starts = list(self.col_starts)
+                ends = starts[1:] + [None]  # Assume each col ends where the next starts
+            else:  # self.col_ends is not None
+                ends = [x + 1 for x in self.col_ends]
+                starts = [0] + ends[:-1]  # Assume each col starts where the last ended
+            vals = [line[start:end].strip() for start, end in zip(starts, ends)]
 
         return vals, starts, ends
 
@@ -220,40 +229,35 @@ class FixedWidthHeader(core.BaseHeader):
         pass
 
 
-class FixedWidthData(core.BaseData):
-    """Base table data reader.
-
-    :param start_line: None, int, or a function of ``lines`` that returns None or int
-    :param end_line: None, int, or a function of ``lines`` that returns None or int
-    :param comment: Regular expression for comment lines
-    :param splitter_class: Splitter class for splitting data lines into columns
+class FixedWidthData(basic.BasicData):
+    """
+    Base table data reader.
     """
-
     splitter_class = FixedWidthSplitter
+    """ Splitter class for splitting data lines into columns """
 
     def write(self, lines):
-        with self._set_col_formats(self.cols, self.formats):
-            vals_list = []
-            # Col iterator does the formatting defined above so each val is a string
-            # and vals is a tuple of strings for all columns of each row
-            col_str_iters = [col.iter_str_vals() for col in self.cols]
-            for vals in izip(*col_str_iters):
-                vals_list.append(vals)
+        vals_list = []
+        col_str_iters = self.str_vals()
+        for vals in zip(*col_str_iters):
+            vals_list.append(vals)
 
         for i, col in enumerate(self.cols):
             col.width = max([len(vals[i]) for vals in vals_list])
             if self.header.start_line is not None:
-                col.width = max(col.width, len(col.name))
+                col.width = max(col.width, len(col.info.name))
 
         widths = [col.width for col in self.cols]
 
         if self.header.start_line is not None:
-            lines.append(self.splitter.join([col.name for col in self.cols], widths))
+            lines.append(self.splitter.join([col.info.name for col in self.cols],
+                                            widths))
 
         if self.header.position_line is not None:
             char = self.header.position_char
             if len(char) != 1:
-                raise ValueError('Position_char="%s" must be a single character' % char)
+                raise ValueError('Position_char="{}" must be a single '
+                                 'character'.format(char))
             vals = [char * col.width for col in self.cols]
             lines.append(self.splitter.join(vals, widths))
 
@@ -263,8 +267,9 @@ class FixedWidthData(core.BaseData):
         return lines
 
 
-class FixedWidth(core.BaseReader):
-    """Read or write a fixed width table with a single header line that defines column
+class FixedWidth(basic.Basic):
+    """
+    Read or write a fixed width table with a single header line that defines column
     names and positions.  Examples::
 
       # Bar delimiter in header and data
@@ -287,35 +292,34 @@ class FixedWidth(core.BaseReader):
 
     See the :ref:`fixed_width_gallery` for specific usage examples.
 
-    :param col_starts: list of start positions for each column (0-based counting)
-    :param col_ends: list of end positions (inclusive) for each column
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
     """
+    _format_name = 'fixed_width'
+    _description = 'Fixed width'
+
+    header_class = FixedWidthHeader
+    data_class = FixedWidthData
+
     def __init__(self, col_starts=None, col_ends=None, delimiter_pad=' ', bookend=True):
-        core.BaseReader.__init__(self)
-
-        self.header = FixedWidthHeader()
-        self.data = FixedWidthData()
-        self.data.header = self.header
-        self.header.data = self.data
-
-        self.header.splitter.delimiter = '|'
-        self.data.splitter.delimiter = '|'
+        super(FixedWidth, self).__init__()
         self.data.splitter.delimiter_pad = delimiter_pad
         self.data.splitter.bookend = bookend
-        self.header.start_line = 0
-        self.data.start_line = 1
-        self.header.comment = r'\s*#'
-        self.header.write_comment = '# '
-        self.data.comment = r'\s*#'
-        self.data.write_comment = '# '
         self.header.col_starts = col_starts
         self.header.col_ends = col_ends
 
 
+class FixedWidthNoHeaderHeader(FixedWidthHeader):
+    '''Header reader for fixed with tables with no header line'''
+    start_line = None
+
+
+class FixedWidthNoHeaderData(FixedWidthData):
+    '''Data reader for fixed width tables with no header line'''
+    start_line = 0
+
+
 class FixedWidthNoHeader(FixedWidth):
-    """Read or write a fixed width table which has no header line.  Column
+    """
+    Read or write a fixed width table which has no header line.  Column
     names are either input (``names`` keyword) or auto-generated.  Column
     positions are determined either by input (``col_starts`` and ``col_stops``
     keywords) or by splitting the first data line.  In the latter case a
@@ -338,20 +342,40 @@ class FixedWidthNoHeader(FixedWidth):
 
     See the :ref:`fixed_width_gallery` for specific usage examples.
 
-    :param col_starts: list of start positions for each column (0-based counting)
-    :param col_ends: list of end positions (inclusive) for each column
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
     """
+    _format_name = 'fixed_width_no_header'
+    _description = 'Fixed width with no header'
+    header_class = FixedWidthNoHeaderHeader
+    data_class = FixedWidthNoHeaderData
+
     def __init__(self, col_starts=None, col_ends=None, delimiter_pad=' ', bookend=True):
-        FixedWidth.__init__(self, col_starts, col_ends,
+        super(FixedWidthNoHeader, self).__init__(col_starts, col_ends,
                             delimiter_pad=delimiter_pad, bookend=bookend)
-        self.header.start_line = None
-        self.data.start_line = 0
+
+
+class FixedWidthTwoLineHeader(FixedWidthHeader):
+    '''Header reader for fixed width tables splitting on whitespace.
+
+    For fixed width tables with several header lines, there is typically
+    a white-space delimited format line, so splitting on white space is
+    needed.
+    '''
+    splitter_class = DefaultSplitter
+
+
+class FixedWidthTwoLineDataSplitter(FixedWidthSplitter):
+    '''Splitter for fixed width tables splitting on ``' '``.'''
+    delimiter = ' '
+
+
+class FixedWidthTwoLineData(FixedWidthData):
+    '''Data reader for fixed with tables with two header lines.'''
+    splitter_class = FixedWidthTwoLineDataSplitter
 
 
 class FixedWidthTwoLine(FixedWidth):
-    """Read or write a fixed width table which has two header lines.  The first
+    """
+    Read or write a fixed width table which has two header lines.  The first
     header line defines the column names and the second implicitly defines the
     column positions.  Examples::
 
@@ -373,17 +397,14 @@ class FixedWidthTwoLine(FixedWidth):
 
     See the :ref:`fixed_width_gallery` for specific usage examples.
 
-    :param position_line: row index of line that specifies position (default = 1)
-    :param position_char: character used to write the position line (default = "-")
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
     """
+    _format_name = 'fixed_width_two_line'
+    _description = 'Fixed width with second header line'
+    data_class = FixedWidthTwoLineData
+    header_class = FixedWidthTwoLineHeader
+
     def __init__(self, position_line=1, position_char='-', delimiter_pad=None, bookend=False):
-        FixedWidth.__init__(self, delimiter_pad=delimiter_pad, bookend=bookend)
+        super(FixedWidthTwoLine, self).__init__(delimiter_pad=delimiter_pad, bookend=bookend)
         self.header.position_line = position_line
         self.header.position_char = position_char
         self.data.start_line = position_line + 1
-        self.header.splitter.delimiter = ' '
-        self.data.splitter.delimiter = ' '
-
-
