@@ -33,7 +33,7 @@ from . import fixedwidth
 from ...table import Table
 from ...utils.data import get_readable_fileobj
 from ...extern import six
-from ...utils.exceptions import AstropyWarning
+from ...utils.exceptions import AstropyWarning, AstropyDeprecationWarning
 
 _read_trace = []
 
@@ -65,7 +65,7 @@ def _probably_html(table, maxchars=100000):
                 if size > maxchars:
                     break
             table = os.linesep.join(table[:i+1])
-        except:
+        except Exception:
             pass
 
     if isinstance(table, six.string_types):
@@ -115,7 +115,7 @@ def get_reader(Reader=None, Inputter=None, Outputter=None, **kwargs):
     Parameters
     ----------
     Reader : `~astropy.io.ascii.BaseReader`
-        Reader class (DEPRECATED) (default= :class:`Basic`)
+        Reader class (DEPRECATED). Default is :class:`Basic`.
     Inputter : `~astropy.io.ascii.BaseInputter`
         Inputter class
     Outputter : `~astropy.io.ascii.BaseOutputter`
@@ -136,23 +136,23 @@ def get_reader(Reader=None, Inputter=None, Outputter=None, **kwargs):
         Line index for the end of data not counting comment or blank lines.
         This value can be negative to count from the end.
     converters : dict
-        Dictionary of converters
+        Dictionary of converters.
     data_Splitter : `~astropy.io.ascii.BaseSplitter`
-        Splitter class to split data columns
+        Splitter class to split data columns.
     header_Splitter : `~astropy.io.ascii.BaseSplitter`
-        Splitter class to split header columns
+        Splitter class to split header columns.
     names : list
-        List of names corresponding to each data column
-    include_names : list
-        List of names to include in output (default= ``None`` selects all names)
+        List of names corresponding to each data column.
+    include_names : list, optional
+        List of names to include in output.
     exclude_names : list
-        List of names to exclude from output (applied after ``include_names``)
+        List of names to exclude from output (applied after ``include_names``).
     fill_values : dict
-        specification of fill values for bad or missing table values
+        Specification of fill values for bad or missing table values.
     fill_include_names : list
-        List of names to include in fill_values (default= ``None`` selects all names)
+        List of names to include in fill_values.
     fill_exclude_names : list
-        List of names to exclude from fill_values (applied after ``fill_include_names``)
+        List of names to exclude from fill_values (applied after ``fill_include_names``).
 
     Returns
     -------
@@ -192,7 +192,7 @@ def read(table, guess=None, **kwargs):
         Input table as a file name, file-like object, list of strings,
         single newline-separated string or pathlib.Path object .
     guess : bool
-        Try to guess the table format (default= ``True``)
+        Try to guess the table format. Defaults to None.
     format : str, `~astropy.io.ascii.BaseReader`
         Input table format
     Inputter : `~astropy.io.ascii.BaseInputter`
@@ -223,20 +223,31 @@ def read(table, guess=None, **kwargs):
     names : list
         List of names corresponding to each data column
     include_names : list
-        List of names to include in output (default= ``None`` selects all names)
+        List of names to include in output.
     exclude_names : list
         List of names to exclude from output (applied after ``include_names``)
     fill_values : dict
         specification of fill values for bad or missing table values
     fill_include_names : list
-        List of names to include in fill_values (default= ``None`` selects all names)
+        List of names to include in fill_values.
     fill_exclude_names : list
         List of names to exclude from fill_values (applied after ``fill_include_names``)
-    fast_reader : bool
-        Whether to use the C engine, can also be a dict with options which default to ``False``
-        (default= ``True``)
+    fast_reader : bool or dict
+        Whether to use the C engine, can also be a dict with options which
+        defaults to `False`; parameters for options dict:
+
+        use_fast_converter: bool
+            enable faster but slightly imprecise floating point conversion method
+        parallel: bool or int
+            multiprocessing conversion using ``cpu_count()`` or ``'number'`` processes
+        exponent_style: str
+            One-character string defining the exponent or ``'Fortran'`` to auto-detect
+            Fortran-style scientific notation like ``'3.14159D+00'`` (``'E'``, ``'D'``, ``'Q'``),
+            all case-insensitive; default ``'E'``, all other imply ``use_fast_converter``
     Reader : `~astropy.io.ascii.BaseReader`
-        Reader class (DEPRECATED) (default= :class:`Basic`).
+        Reader class (DEPRECATED)
+    encoding: str
+        Allow to specify encoding to read the file (default= ``None``).
 
     Returns
     -------
@@ -251,7 +262,7 @@ def read(table, guess=None, **kwargs):
     # If an Outputter is supplied in kwargs that will take precedence.
     new_kwargs = {}
     fast_reader_param = kwargs.get('fast_reader', True)
-    if 'Outputter' in kwargs: # user specified Outputter, not supported for fast reading
+    if 'Outputter' in kwargs:  # user specified Outputter, not supported for fast reading
         fast_reader_param = False
     format = kwargs.get('format')
     new_kwargs.update(kwargs)
@@ -284,12 +295,13 @@ def read(table, guess=None, **kwargs):
         # which case the original `table` as the data filename must be left
         # intact.
         if 'readme' not in new_kwargs:
+            encoding = kwargs.get('encoding')
             try:
-                with get_readable_fileobj(table) as fileobj:
+                with get_readable_fileobj(table, encoding=encoding) as fileobj:
                     table = fileobj.read()
             except ValueError:  # unreadable or invalid binary file
                 raise
-            except:
+            except Exception:
                 pass
             else:
                 # Ensure that `table` has at least one \r or \n in it
@@ -363,8 +375,18 @@ def _guess(table, read_kwargs, format, fast_reader):
     format : str
         Table format
     fast_reader : bool
+    fast_reader : bool or dict
         Whether to use the C engine, can also be a dict with options which
-        default to ``False`` (default= ``True``)
+        defaults to `False`; parameters for options dict:
+
+        use_fast_converter: bool
+            enable faster but slightly imprecise floating point conversion method
+        parallel: bool or int
+            multiprocessing conversion using ``cpu_count()`` or ``'number'`` processes
+        exponent_style: str
+            Character to use for exponent or ``'Fortran'`` to auto-detect any
+            Fortran-style scientific notation like ``'3.14159D+00'`` (``'E'``, ``'D'``, ``'Q'``),
+            all case-insensitive; default ``'E'``, all other imply ``use_fast_converter``
 
     Returns
     -------
@@ -481,7 +503,7 @@ def _guess(table, read_kwargs, format, fast_reader):
                 reader_repr = repr(kwargs.get('Reader', basic.Basic))
                 keys_vals = ['Reader:' + re.search(r"\.(\w+)'>", reader_repr).group(1)]
                 kwargs_sorted = ((key, kwargs[key]) for key in sorted_keys)
-                keys_vals.extend(['%s: %s' % (key, repr(val)) for key, val in kwargs_sorted])
+                keys_vals.extend(['{}: {!r}'.format(key, val) for key, val in kwargs_sorted])
                 lines.append(' '.join(keys_vals))
 
             msg = ['',
@@ -489,12 +511,14 @@ def _guess(table, read_kwargs, format, fast_reader):
                    '** ERROR: Unable to guess table format with the guesses listed above. **',
                    '**                                                                    **',
                    '** To figure out why the table did not read, use guess=False and      **',
-                   '** appropriate arguments to read().  In particular specify the format **',
-                   '** and any known attributes like the delimiter.                       **',
+                   '** fast_reader=False, along with any appropriate arguments to read(). **',
+                   '** In particular specify the format and any known attributes like the **',
+                   '** delimiter.                                                         **',
                    '************************************************************************']
             lines.extend(msg)
             raise core.InconsistentTableError('\n'.join(lines))
 
+
 def _get_guess_kwargs_list(read_kwargs):
     """
     Get the full list of reader keyword argument dicts that are the basis
@@ -564,13 +588,14 @@ def _get_guess_kwargs_list(read_kwargs):
     for Reader in (fastbasic.FastCommentedHeader, basic.CommentedHeader,
                    fastbasic.FastBasic, basic.Basic,
                    fastbasic.FastNoHeader, basic.NoHeader):
-        for delimiter in ("|", ",", " ", "\s"):
+        for delimiter in ("|", ",", " ", r"\s"):
             for quotechar in ('"', "'"):
                 guess_kwargs_list.append(dict(
                     Reader=Reader, delimiter=delimiter, quotechar=quotechar))
 
     return guess_kwargs_list
 
+
 extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
                      'names', 'include_names', 'exclude_names', 'strip_whitespace')
 
@@ -583,7 +608,7 @@ def get_writer(Writer=None, fast_writer=True, **kwargs):
     Parameters
     ----------
     Writer : ``Writer``
-        Writer class (DEPRECATED) (default= :class:`Basic`)
+        Writer class (DEPRECATED). Defaults to :class:`Basic`.
     delimiter : str
         Column delimiter string
     comment : str
@@ -593,15 +618,15 @@ def get_writer(Writer=None, fast_writer=True, **kwargs):
     formats : dict
         Dictionary of format specifiers or formatting functions
     strip_whitespace : bool
-        Strip surrounding whitespace from column values (default= ``True``)
+        Strip surrounding whitespace from column values.
     names : list
         List of names corresponding to each data column
     include_names : list
-        List of names to include in output (default= ``None`` selects all names)
+        List of names to include in output.
     exclude_names : list
         List of names to exclude from output (applied after ``include_names``)
     fast_writer : bool
-        Whether to use the fast Cython writer (default= ``True``)
+        Whether to use the fast Cython writer.
 
     Returns
     -------
@@ -629,7 +654,7 @@ def get_writer(Writer=None, fast_writer=True, **kwargs):
     return writer
 
 
-def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwargs):
+def write(table, output=None, format=None, Writer=None, fast_writer=True, **kwargs):
     """Write the input ``table`` to ``filename``.  Most of the default behavior
     for various parameters is determined by the Writer class.
 
@@ -639,9 +664,9 @@ def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwa
         Input table as a Reader object, Numpy struct array, file name,
         file-like object, list of strings, or single newline-separated string.
     output : str, file_like
-        Output [filename, file-like object] (default = ``sys.stdout``)
+        Output [filename, file-like object]. Defaults to``sys.stdout``.
     format : str
-        Output table format (default= ``basic``)
+        Output table format. Defaults to 'basic'.
     delimiter : str
         Column delimiter string
     comment : str
@@ -651,22 +676,43 @@ def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwa
     formats : dict
         Dictionary of format specifiers or formatting functions
     strip_whitespace : bool
-        Strip surrounding whitespace from column values (default= ``True``)
+        Strip surrounding whitespace from column values.
     names : list
         List of names corresponding to each data column
     include_names : list
-        List of names to include in output (default= ``None`` selects all names)
+        List of names to include in output.
     exclude_names : list
         List of names to exclude from output (applied after ``include_names``)
     fast_writer : bool
-        Whether to use the fast Cython writer (default= ``True``)
+        Whether to use the fast Cython writer.
+    overwrite : bool
+        If ``overwrite=None`` (default) and the file exists, then a
+        warning will be issued. In a future release this will instead
+        generate an exception. If ``overwrite=False`` and the file
+        exists, then an exception is raised.
+        This parameter is ignored when the ``output`` arg is not a string
+        (e.g., a file object).
     Writer : ``Writer``
-        Writer class (DEPRECATED) (default= :class:`Basic`)
+        Writer class (DEPRECATED).
+
     """
+    overwrite = kwargs.pop('overwrite', None)
+    if isinstance(output, six.string_types):
+        if os.path.lexists(output):
+            if overwrite is None:
+                warnings.warn(
+                    "{} already exists. "
+                    "Automatically overwriting ASCII files is deprecated. "
+                    "Use the argument 'overwrite=True' in the future.".format(
+                        output), AstropyDeprecationWarning)
+            elif not overwrite:
+                raise IOError("{} already exists".format(output))
+
     if output is None:
         output = sys.stdout
 
-    table = Table(table, names=kwargs.get('names'))
+    table_cls = table.__class__ if isinstance(table, Table) else Table
+    table = table_cls(table, names=kwargs.get('names'))
 
     table0 = table[:0].copy()
     core._apply_include_exclude_names(table0, kwargs.get('names'),
@@ -700,6 +746,7 @@ def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwa
         output.write(outstr)
         output.write(os.linesep)
 
+
 def get_read_trace():
     """
     Return a traceback of the attempted read formats for the last call to
