@@ -8,6 +8,7 @@ import warnings
 import weakref
 
 from functools import reduce
+from collections import OrderedDict
 
 import numpy as np
 from numpy import char as chararray
@@ -18,10 +19,10 @@ from .util import (pairwise, _is_int, _convert_array, encode_ascii, cmp,
                    NotifierMixin)
 from .verify import VerifyError, VerifyWarning
 
-from ...extern.six import string_types, iteritems
-from ...utils import lazyproperty, isiterable, indent, OrderedDict
-from ...utils.compat import ignored
-
+from ...extern.six import string_types, iteritems, PY2
+from ...extern.six.moves import range, zip
+from ...utils import lazyproperty, isiterable, indent
+from ...utils.compat import suppress
 
 __all__ = ['Column', 'ColDefs', 'Delayed']
 
@@ -41,60 +42,58 @@ FITS2NUMPY = {'L': 'i1', 'B': 'u1', 'I': 'i2', 'J': 'i4', 'K': 'i8', 'E': 'f4',
               'D': 'f8', 'C': 'c8', 'M': 'c16', 'A': 'a'}
 
 # the inverse dictionary of the above
-NUMPY2FITS = dict([(val, key) for key, val in iteritems(FITS2NUMPY)])
-# Normally booleans are represented as ints in pyfits, but if passed in a numpy
+NUMPY2FITS = {val: key for key, val in iteritems(FITS2NUMPY)}
+# Normally booleans are represented as ints in Astropy, but if passed in a numpy
 # boolean array, that should be supported
 NUMPY2FITS['b1'] = 'L'
 # Add unsigned types, which will be stored as signed ints with a TZERO card.
 NUMPY2FITS['u2'] = 'I'
 NUMPY2FITS['u4'] = 'J'
 NUMPY2FITS['u8'] = 'K'
+# Add half precision floating point numbers which will be up-converted to
+# single precision.
+NUMPY2FITS['f2'] = 'E'
 
 # This is the order in which values are converted to FITS types
 # Note that only double precision floating point/complex are supported
 FORMATORDER = ['L', 'B', 'I', 'J', 'K', 'D', 'M', 'A']
 
+# Convert single precision floating point/complex to double precision.
+FITSUPCONVERTERS = {'E': 'D', 'C': 'M'}
+
 # mapping from ASCII table TFORM data type to numpy data type
 # A: Character
 # I: Integer (32-bit)
 # J: Integer (64-bit; non-standard)
-# F: Float (32-bit; fixed decimal notation)
-# E: Float (32-bit; exponential notation)
+# F: Float (64-bit; fixed decimal notation)
+# E: Float (64-bit; exponential notation)
 # D: Float (64-bit; exponential notation, always 64-bit by convention)
-ASCII2NUMPY = {'A': 'a', 'I': 'i4', 'J': 'i8', 'F': 'f4', 'E': 'f4',
-               'D': 'f8'}
+ASCII2NUMPY = {'A': 'a', 'I': 'i4', 'J': 'i8', 'F': 'f8', 'E': 'f8', 'D': 'f8'}
 
 # Maps FITS ASCII column format codes to the appropriate Python string
 # formatting codes for that type.
-ASCII2STR = {'A': 's', 'I': 'd', 'J': 'd', 'F': 'f', 'E': 'E', 'D': 'E'}
+ASCII2STR = {'A': '', 'I': 'd', 'J': 'd', 'F': 'f', 'E': 'E', 'D': 'E'}
 
 # For each ASCII table format code, provides a default width (and decimal
 # precision) for when one isn't given explicitly in the column format
-ASCII_DEFAULT_WIDTHS= {'A': (1, 0), 'I': (10, 0), 'J': (15, 0),
-                       'E': (15, 7), 'F': (16, 7), 'D': (25, 17)}
+ASCII_DEFAULT_WIDTHS = {'A': (1, 0), 'I': (10, 0), 'J': (15, 0),
+                        'E': (15, 7), 'F': (16, 7), 'D': (25, 17)}
 
 
-
-
-# lists of column/field definition common names and keyword names, make
+# tuple of column/field definition common names and keyword names, make
 # sure to preserve the one-to-one correspondence when updating the list(s).
 # Use lists, instead of dictionaries so the names can be displayed in a
 # preferred order.
-KEYWORD_NAMES = ['TTYPE', 'TFORM', 'TUNIT', 'TNULL', 'TSCAL', 'TZERO',
-                 'TDISP', 'TBCOL', 'TDIM']
-KEYWORD_ATTRIBUTES = ['name', 'format', 'unit', 'null', 'bscale', 'bzero',
-                      'disp', 'start', 'dim']
+KEYWORD_NAMES = ('TTYPE', 'TFORM', 'TUNIT', 'TNULL', 'TSCAL', 'TZERO',
+                 'TDISP', 'TBCOL', 'TDIM')
+KEYWORD_ATTRIBUTES = ('name', 'format', 'unit', 'null', 'bscale', 'bzero',
+                      'disp', 'start', 'dim')
 """This is a list of the attributes that can be set on `Column` objects."""
 
 
-KEYWORD_TO_ATTRIBUTE = \
-    OrderedDict((keyword, attr)
-                for keyword, attr in zip(KEYWORD_NAMES, KEYWORD_ATTRIBUTES))
+KEYWORD_TO_ATTRIBUTE = OrderedDict(zip(KEYWORD_NAMES, KEYWORD_ATTRIBUTES))
 
-
-ATTRIBUTE_TO_KEYWORD = \
-    OrderedDict((value, key)
-                for key, value in KEYWORD_TO_ATTRIBUTE.items())
+ATTRIBUTE_TO_KEYWORD = OrderedDict(zip(KEYWORD_ATTRIBUTES, KEYWORD_NAMES))
 
 
 # TODO: Define a list of default comments to associate with each table keyword
@@ -244,7 +243,7 @@ class _ColumnFormat(_BaseColumnFormat):
         else:
             repeat = str(self.repeat)
 
-        return '%s%s%s' % (repeat, self.format, self.option)
+        return '{}{}{}'.format(repeat, self.format, self.option)
 
 
 class _AsciiColumnFormat(_BaseColumnFormat):
@@ -261,7 +260,7 @@ class _AsciiColumnFormat(_BaseColumnFormat):
     Conversions between the two column formats can be performed using the
     ``to/from_binary`` methods on this class, or the ``to/from_ascii``
     methods on the `_ColumnFormat` class.  But again, not all conversions are
-    possible and may result in a `~.exceptions.ValueError`.
+    possible and may result in a `ValueError`.
     """
 
     def __new__(cls, format, strict=False):
@@ -306,9 +305,9 @@ class _AsciiColumnFormat(_BaseColumnFormat):
         """
 
         if self.format in ('E', 'F', 'D'):
-            return '%s%s.%s' % (self.format, self.width, self.precision)
+            return '{}{}.{}'.format(self.format, self.width, self.precision)
 
-        return '%s%s' % (self.format, self.width)
+        return '{}{}'.format(self.format, self.width)
 
 
 class _FormatX(str):
@@ -326,7 +325,7 @@ class _FormatX(str):
 
     @property
     def tform(self):
-        return '%sX' % self.repeat
+        return '{}X'.format(self.repeat)
 
 
 # TODO: Table column formats need to be verified upon first reading the file;
@@ -337,10 +336,10 @@ class _FormatP(str):
 
     # As far as I can tell from my reading of the FITS standard, a type code is
     # *required* for P and Q formats; there is no default
-    _format_re_template = (r'(?P<repeat>\d+)?%s(?P<dtype>[LXBIJKAEDCM])'
-                            '(?:\((?P<max>\d*)\))?')
+    _format_re_template = (r'(?P<repeat>\d+)?{}(?P<dtype>[LXBIJKAEDCM])'
+                           r'(?:\((?P<max>\d*)\))?')
     _format_code = 'P'
-    _format_re = re.compile(_format_re_template % _format_code)
+    _format_re = re.compile(_format_re_template.format(_format_code))
     _descriptor_format = '2i4'
 
     def __new__(cls, dtype, repeat=None, max=None):
@@ -358,7 +357,7 @@ class _FormatP(str):
     def from_tform(cls, format):
         m = cls._format_re.match(format)
         if not m or m.group('dtype') not in FITS2NUMPY:
-            raise VerifyError('Invalid column format: %s' % format)
+            raise VerifyError('Invalid column format: {}'.format(format))
         repeat = m.group('repeat')
         array_dtype = m.group('dtype')
         max = m.group('max')
@@ -370,7 +369,7 @@ class _FormatP(str):
     def tform(self):
         repeat = '' if self.repeat is None else self.repeat
         max = '' if self.max is None else self.max
-        return '%s%s%s(%s)' % (repeat, self._format_code, self.format, max)
+        return '{}{}{}({})'.format(repeat, self._format_code, self.format, max)
 
 
 class _FormatQ(_FormatP):
@@ -381,7 +380,7 @@ class _FormatQ(_FormatP):
     """
 
     _format_code = 'Q'
-    _format_re = re.compile(_FormatP._format_re_template % _format_code)
+    _format_re = re.compile(_FormatP._format_re_template.format(_format_code))
     _descriptor_format = '2i8'
 
 
@@ -403,7 +402,8 @@ class ColumnAttribute(object):
 
         @ColumnAttribute('TTYPE')
         def name(col, name):
-            assert isinstance(name, str)
+            if not isinstance(name, str):
+                raise AssertionError
 
     The actual object returned by this decorator is the `ColumnAttribute`
     instance though, not the ``name`` function.  As such ``name`` is not a
@@ -423,21 +423,21 @@ class ColumnAttribute(object):
         # determined from the KEYWORD_NAMES/ATTRIBUTES lists.  This could be
         # make more flexible in the future, for example, to support custom
         # column attributes.
-        self._attr = KEYWORD_TO_ATTRIBUTE[self._keyword]
+        self._attr = '_' + KEYWORD_TO_ATTRIBUTE[self._keyword]
 
     def __get__(self, obj, objtype=None):
         if obj is None:
             return self
         else:
-            return getattr(obj, '_' + self._attr)
+            return getattr(obj, self._attr)
 
     def __set__(self, obj, value):
         if self._validator is not None:
             self._validator(obj, value)
 
-        old_value = getattr(obj, '_' + self._attr, None)
-        setattr(obj, '_' + self._attr, value)
-        obj._notify('column_attribute_changed', obj, self._attr, old_value,
+        old_value = getattr(obj, self._attr, None)
+        setattr(obj, self._attr, value)
+        obj._notify('column_attribute_changed', obj, self._attr[1:], old_value,
                     value)
 
     def __call__(self, func):
@@ -543,7 +543,7 @@ class Column(NotifierMixin):
         for attr in KEYWORD_ATTRIBUTES:
             setattr(self, attr, valid_kwargs.get(attr))
 
-        # TODO: For PyFITS 3.3 try to eliminate the following two special cases
+        # TODO: Try to eliminate the following two special cases
         # for recformat and dim:
         # This is not actually stored as an attribute on columns for some
         # reason
@@ -567,7 +567,7 @@ class Column(NotifierMixin):
             try:  # try to convert to a ndarray first
                 if array is not None:
                     array = np.array(array)
-            except:
+            except Exception:
                 try:  # then try to convert it to a strings array
                     itemsize = int(recformat[1:])
                     array = chararray.array(array, itemsize=itemsize)
@@ -578,7 +578,7 @@ class Column(NotifierMixin):
                         array = _VLF(array, dtype=recformat.dtype)
                     else:
                         raise ValueError('Data is inconsistent with the '
-                                         'format `%s`.' % format)
+                                         'format `{}`.'.format(format))
 
         array = self._convert_to_valid_data_type(array)
 
@@ -636,7 +636,7 @@ class Column(NotifierMixin):
 
         # Ideally the .array attribute never would have existed in the first
         # place, or would have been internal-only.  This is a legacy of the
-        # older design from PyFITS that needs to have continued support, for
+        # older design from Astropy that needs to have continued support, for
         # now.
 
         # One of the main problems with this design was that it created a
@@ -756,12 +756,13 @@ class Column(NotifierMixin):
 
         # This ensures that the new name can fit into a single FITS card
         # without any special extension like CONTINUE cards or the like.
-        assert (isinstance(name, string_types) and
-                len(str(Card('TTYPE', name))) == CARD_LENGTH), \
-                    ('Column name must be a string able to fit in a single '
-                     'FITS card--typically this means a maximum of 68 '
-                     'characters, though it may be fewer if the string '
-                     'contains special characters like quotes.')
+        if (not isinstance(name, string_types)
+                or len(str(Card('TTYPE', name))) != CARD_LENGTH):
+            raise AssertionError(
+                'Column name must be a string able to fit in a single '
+                'FITS card--typically this means a maximum of 68 '
+                'characters, though it may be fewer if the string '
+                'contains special characters like quotes.')
 
     format = ColumnAttribute('TFORM')
     unit = ColumnAttribute('TUNIT')
@@ -790,28 +791,6 @@ class Column(NotifierMixin):
         tmp.__dict__ = self.__dict__.copy()
         return tmp
 
-    if sys.version_info < (2, 7):
-        # This is only needed on Python 2.6, where it appears deepcopy has
-        # problems with weakrefs, and especially weak-keyed dicts.
-        def __deepcopy__(self, memo=None):
-            tmp = object.__new__(self.__class__)
-            tmp_dict = dict(self.__dict__)
-            array = self.array
-            listeners = None
-            if array is not None:
-                tmp_dict['array'] = array.copy()
-            tmp_dict['_parent_fits_rec'] = None
-
-            if '_listeners' in tmp_dict:
-                listners = tmp_dict['_listeners']
-                del tmp_dict['_listeners']
-
-            tmp.__dict__ = copy.deepcopy(tmp_dict, memo=memo)
-
-            if listeners is not None:
-                tmp.__dict__['_listeners'] = listeners
-            return tmp
-
     @staticmethod
     def _convert_format(format, cls):
         """The format argument to this class's initializer may come in many
@@ -827,7 +806,7 @@ class Column(NotifierMixin):
             return format, format.recformat
 
         if format in NUMPY2FITS:
-            with ignored(VerifyError):
+            with suppress(VerifyError):
                 # legit recarray format?
                 recformat = format
                 format = cls.from_recformat(format)
@@ -837,7 +816,7 @@ class Column(NotifierMixin):
             format = cls(format)
             recformat = format.recformat
         except VerifyError:
-            raise VerifyError('Illegal format `%s`.' % format)
+            raise VerifyError('Illegal format `{}`.'.format(format))
 
         return format, recformat
 
@@ -880,16 +859,16 @@ class Column(NotifierMixin):
                     msg = (
                         "ASCII table null option (TNULLn) is longer than "
                         "the column's character width and will be truncated "
-                        "(got %r)." % null)
+                        "(got {!r}).".format(null))
             else:
                 if not _is_int(null):
                     # Make this an exception instead of a warning, since any
                     # non-int value is meaningless
                     msg = (
                         'Column null option (TNULLn) must be an integer for '
-                        'binary table columns (got %r).  The invalid value '
+                        'binary table columns (got {!r}).  The invalid value '
                         'will be ignored for the purpose of formatting '
-                        'the data in this column.' % null)
+                        'the data in this column.'.format(null))
 
                 tnull_formats = ('B', 'I', 'J', 'K')
 
@@ -900,9 +879,9 @@ class Column(NotifierMixin):
                     # is in the range allowed by the column's format
                     msg = (
                         'Column null option (TNULLn) is invalid for binary '
-                        'table columns of type %r (got %r).  The invalid '
+                        'table columns of type {!r} (got {!r}).  The invalid '
                         'value will be ignored for the purpose of formatting '
-                        'the data in this column.' % (format, null))
+                        'the data in this column.'.format(format, null))
 
             if msg is None:
                 valid['null'] = null
@@ -915,9 +894,9 @@ class Column(NotifierMixin):
             msg = None
             if not isinstance(disp, string_types):
                 msg = (
-                    'Column disp option (TDISPn) must be a string (got %r).'
+                    'Column disp option (TDISPn) must be a string (got {!r}).'
                     'The invalid value will be ignored for the purpose of '
-                    'formatting the data in this column.' % disp)
+                    'formatting the data in this column.'.format(disp))
 
             if (isinstance(format, _AsciiColumnFormat) and
                     disp[0].upper() == 'L'):
@@ -941,9 +920,9 @@ class Column(NotifierMixin):
                 # The 'start' option only applies to ASCII columns
                 msg = (
                     'Column start option (TBCOLn) is not allowed for binary '
-                    'table columns (got %r).  The invalid keyword will be '
+                    'table columns (got {!r}).  The invalid keyword will be '
                     'ignored for the purpose of formatting the data in this '
-                    'column.'% start)
+                    'column.'.format(start))
             try:
                 start = int(start)
             except (TypeError, ValueError):
@@ -952,8 +931,8 @@ class Column(NotifierMixin):
             if not _is_int(start) and start < 1:
                 msg = (
                     'Column start option (TBCOLn) must be a positive integer '
-                    '(got %r).  The invalid value will be ignored for the '
-                    'purpose of formatting the data in this column.' % start)
+                    '(got {!r}).  The invalid value will be ignored for the '
+                    'purpose of formatting the data in this column.'.format(start))
 
             if msg is None:
                 valid['start'] = start
@@ -973,8 +952,8 @@ class Column(NotifierMixin):
             if isinstance(format, _AsciiColumnFormat):
                 msg = (
                     'Column dim option (TDIMn) is not allowed for ASCII table '
-                    'columns (got %r).  The invalid keyword will be ignored '
-                    'for the purpose of formatting this column.' % dim)
+                    'columns (got {!r}).  The invalid keyword will be ignored '
+                    'for the purpose of formatting this column.'.format(dim))
 
             elif isinstance(dim, string_types):
                 dims_tuple = _parse_tdim(dim)
@@ -991,11 +970,11 @@ class Column(NotifierMixin):
             if dims_tuple:
                 if reduce(operator.mul, dims_tuple) > format.repeat:
                     msg = (
-                        "The repeat count of the column format %r for column %r "
+                        "The repeat count of the column format {!r} for column {!r} "
                         "is fewer than the number of elements per the TDIM "
-                        "argument %r.  The invalid TDIMn value will be ignored "
-                        "for the purpose of formatting this column." %
-                        (name, format, dim))
+                        "argument {!r}.  The invalid TDIMn value will be ignored "
+                        "for the purpose of formatting this column.".format(
+                            name, format, dim))
 
             if msg is None:
                 valid['dim'] = dims_tuple
@@ -1018,7 +997,7 @@ class Column(NotifierMixin):
         """
 
         # If the given format string is unambiguously a Numpy dtype or one of
-        # the Numpy record format type specifiers supported by PyFITS then that
+        # the Numpy record format type specifiers supported by Astropy then that
         # should take priority--otherwise assume it is a FITS format
         if isinstance(format, np.dtype):
             format, _, _ = _dtype_to_recformat(format)
@@ -1062,11 +1041,11 @@ class Column(NotifierMixin):
             # "optional" codes), but it is also strictly a valid ASCII
             # table format, then assume an ASCII table column was being
             # requested (the more likely case, after all).
-            with ignored(VerifyError):
+            with suppress(VerifyError):
                 format = _AsciiColumnFormat(format, strict=True)
 
             # A safe guess which reflects the existing behavior of previous
-            # PyFITS versions
+            # Astropy versions
             guess_format = _ColumnFormat
 
         try:
@@ -1116,7 +1095,7 @@ class Column(NotifierMixin):
             elif 'L' in format:
                 # boolean needs to be scaled back to storage values ('T', 'F')
                 if array.dtype == np.dtype('bool'):
-                    return np.where(array == False, ord('F'), ord('T'))
+                    return np.where(array == np.False_, ord('F'), ord('T'))
                 else:
                     return np.where(array == 0, ord('F'), ord('T'))
             elif 'X' in format:
@@ -1163,25 +1142,8 @@ class ColDefs(NotifierMixin):
     _padding_byte = '\x00'
     _col_format_cls = _ColumnFormat
 
-    def __new__(cls, input, tbtype=None, ascii=False):
-        if tbtype is not None:
-            warnings.warn(
-                'The ``tbtype`` argument to `ColDefs` is deprecated as of '
-                'Astropy 0.4; instead the appropriate table type should be '
-                'inferred from the formats of the supplied columns.  Use the '
-                '``ascii=True`` argument to ensure that ASCII table columns '
-                'are used.', AstropyDeprecationWarning)
-        else:
-            tbtype = 'BinTableHDU'  # The old default
-
-        # Backwards-compat support
-        # TODO: Remove once the tbtype argument is removed entirely
-        if tbtype == 'BinTableHDU':
-            klass = cls
-        elif tbtype == 'TableHDU':
-            klass = _AsciiColDefs
-        else:
-            raise ValueError('Invalid table type: %s.' % tbtype)
+    def __new__(cls, input, ascii=False):
+        klass = cls
 
         if (hasattr(input, '_columns_type') and
                 issubclass(input._columns_type, ColDefs)):
@@ -1198,7 +1160,7 @@ class ColDefs(NotifierMixin):
     def __getnewargs__(self):
         return (self._arrays,)
 
-    def __init__(self, input, tbtype=None, ascii=False):
+    def __init__(self, input, ascii=False):
         """
         Parameters
         ----------
@@ -1207,16 +1169,10 @@ class ColDefs(NotifierMixin):
             An existing table HDU, an existing `ColDefs`, or any multi-field
             Numpy array or `numpy.recarray`.
 
-        **(Deprecated)** tbtype : str, optional
-            which table HDU, ``"BinTableHDU"`` (default) or
-            ``"TableHDU"`` (text table).
-            Now ColDefs for a normal (binary) table by default, but converted
-            automatically to ASCII table ColDefs in the appropriate contexts
-            (namely, when creating an ASCII table).
-
         ascii : bool
-        """
+            Use True to ensure that ASCII table columns are used.
 
+        """
         from .hdu.table import _TableBaseHDU
         from .fitsrec import FITS_rec
 
@@ -1255,8 +1211,8 @@ class ColDefs(NotifierMixin):
     def _init_from_sequence(self, columns):
         for idx, col in enumerate(columns):
             if not isinstance(col, Column):
-                raise TypeError(
-                    'Element %d in the ColDefs input is not a Column.' % idx)
+                raise TypeError('Element {} in the ColDefs input is not a '
+                                'Column.'.format(idx))
 
         self._init_from_coldefs(columns)
 
@@ -1281,12 +1237,13 @@ class ColDefs(NotifierMixin):
 
             # Check for unsigned ints.
             bzero = None
-            if 'I' in format and ftype == np.dtype('uint16'):
-                bzero = np.uint16(2**15)
-            elif 'J' in format and ftype == np.dtype('uint32'):
-                bzero = np.uint32(2**31)
-            elif 'K' in format and ftype == np.dtype('uint64'):
-                bzero = np.uint64(2**63)
+            if ftype.base.kind == 'u':
+                if 'I' in format:
+                    bzero = np.uint16(2**15)
+                elif 'J' in format:
+                    bzero = np.uint32(2**31)
+                elif 'K' in format:
+                    bzero = np.uint64(2**63)
 
             c = Column(name=cname, format=format,
                        array=array.view(np.ndarray)[cname], bzero=bzero,
@@ -1304,7 +1261,7 @@ class ColDefs(NotifierMixin):
             key = TDEF_RE.match(keyword)
             try:
                 keyword = key.group('label')
-            except:
+            except Exception:
                 continue  # skip if there is no match
             if keyword in KEYWORD_NAMES:
                 col = int(key.group('num'))
@@ -1322,7 +1279,7 @@ class ColDefs(NotifierMixin):
             valid_kwargs, invalid_kwargs = Column._verify_keywords(**kwargs)
             for val in invalid_kwargs.values():
                 warnings.warn(
-                    'Invalid keyword for column %d: %s' % (idx + 1, val[1]),
+                    'Invalid keyword for column {}: {}'.format(idx + 1, val[1]),
                     VerifyWarning)
             # Special cases for recformat and dim
             # TODO: Try to eliminate the need for these special cases
@@ -1404,16 +1361,12 @@ class ColDefs(NotifierMixin):
 
         Implements for example self.units, self.formats, etc.
         """
-
         cname = name[:-1]
         if cname in KEYWORD_ATTRIBUTES and name[-1] == 's':
             attr = []
-            for col in self:
+            for col in self.columns:
                 val = getattr(col, cname)
-                if val is not None:
-                    attr.append(val)
-                else:
-                    attr.append('')
+                attr.append(val if val is not None else '')
             return attr
         raise AttributeError(name)
 
@@ -1444,10 +1397,24 @@ class ColDefs(NotifierMixin):
                 else:
                     dt = np.dtype((dt.base, dim))
 
+            # On Python 2, force the `name` to `str`
+            # because Numpy structured arrays can't handle unicode `name`
+            # See https://github.com/astropy/astropy/issues/5204
+            if PY2:  # pragma: py2
+                name = str(name)
+
             fields.append((name, dt))
 
         return nh.realign_dtype(np.dtype(fields), offsets)
 
+    @lazyproperty
+    def names(self):
+        return [col.name for col in self.columns]
+
+    @lazyproperty
+    def formats(self):
+        return [col.format for col in self.columns]
+
     @lazyproperty
     def _arrays(self):
         return [col.array for col in self.columns]
@@ -1529,6 +1496,11 @@ class ColDefs(NotifierMixin):
             if col is column:
                 break
 
+        if attr == 'name':
+            del self.names
+        elif attr == 'format':
+            del self.formats
+
         self._notify('column_attribute_changed', column, idx, attr, old_value,
                      new_value)
 
@@ -1537,13 +1509,16 @@ class ColDefs(NotifierMixin):
         Append one `Column` to the column definition.
         """
 
-        assert isinstance(column, Column)
+        if not isinstance(column, Column):
+            raise AssertionError
 
         self._arrays.append(column.array)
         # Obliterate caches of certain things
         del self.dtype
         del self._recformats
         del self._dims
+        del self.names
+        del self.formats
 
         self.columns.append(column)
 
@@ -1571,6 +1546,8 @@ class ColDefs(NotifierMixin):
         del self.dtype
         del self._recformats
         del self._dims
+        del self.names
+        del self.formats
 
         del self.columns[indx]
 
@@ -1615,7 +1592,7 @@ class ColDefs(NotifierMixin):
         """
 
         if new_name != col_name and new_name in self.names:
-            raise ValueError('New name %s already exists.' % new_name)
+            raise ValueError('New name {} already exists.'.format(new_name))
         else:
             self.change_attrib(col_name, 'name', new_name)
 
@@ -1674,11 +1651,11 @@ class ColDefs(NotifierMixin):
         for attr in lst:
             if output:
                 if attr not in KEYWORD_ATTRIBUTES:
-                    output.write("'%s' is not an attribute of the column "
-                                 "definitions.\n" % attr)
+                    output.write("'{}' is not an attribute of the column "
+                                 "definitions.\n".format(attr))
                     continue
-                output.write("%s:\n" % attr)
-                output.write('    %s\n' % getattr(self, attr + 's'))
+                output.write("{}:\n".format(attr))
+                output.write('    {}\n'.format(getattr(self, attr + 's')))
             else:
                 ret[attr] = getattr(self, attr + 's')
 
@@ -1692,7 +1669,7 @@ class _AsciiColDefs(ColDefs):
     _padding_byte = ' '
     _col_format_cls = _AsciiColumnFormat
 
-    def __init__(self, input, tbtype=None, ascii=True):
+    def __init__(self, input, ascii=True):
         super(_AsciiColDefs, self).__init__(input)
 
         # if the format of an ASCII column has no width, add one
@@ -1707,7 +1684,6 @@ class _AsciiColDefs(ColDefs):
 
     @lazyproperty
     def dtype(self):
-        _itemsize = self.spans[-1] + self.starts[-1] - 1
         dtype = {}
 
         for j in range(len(self)):
@@ -1785,7 +1761,7 @@ class _VLF(np.ndarray):
                 # this handles ['abc'] and [['a','b','c']]
                 # equally, beautiful!
                 input = [chararray.array(x, itemsize=1) for x in input]
-            except:
+            except Exception:
                 raise ValueError(
                     'Inconsistent input data array: {0}'.format(input))
 
@@ -1857,11 +1833,11 @@ def _get_index(names, key):
             if count == 1:
                 indx = names.index(_key)
             elif count == 0:
-                raise KeyError("Key '%s' does not exist." % key)
+                raise KeyError("Key '{}' does not exist.".format(key))
             else:              # multiple match
-                raise KeyError("Ambiguous key name '%s'." % key)
+                raise KeyError("Ambiguous key name '{}'.".format(key))
     else:
-        raise KeyError("Illegal key '%s'." % repr(key))
+        raise KeyError("Illegal key '{!r}'.".format(key))
 
     return indx
 
@@ -1955,7 +1931,6 @@ def _makep(array, descr_output, format, nrows=None):
 
     if not nrows:
         nrows = len(array)
-    n = min(len(array), nrows)
 
     data_output = _VLF([None] * nrows, dtype=format.dtype)
 
@@ -1992,11 +1967,11 @@ def _parse_tformat(tform):
 
     try:
         (repeat, format, option) = TFORMAT_RE.match(tform.strip()).groups()
-    except:
+    except Exception:
         # TODO: Maybe catch this error use a default type (bytes, maybe?) for
         # unrecognized column types.  As long as we can determine the correct
         # byte width somehow..
-        raise VerifyError('Format %r is not recognized.' % tform)
+        raise VerifyError('Format {!r} is not recognized.'.format(tform))
 
     if repeat == '':
         repeat = 1
@@ -2015,7 +1990,7 @@ def _parse_ascii_tformat(tform, strict=False):
 
     match = TFORMAT_ASCII_RE.match(tform.strip())
     if not match:
-        raise VerifyError('Format %r is not recognized.' % tform)
+        raise VerifyError('Format {!r} is not recognized.'.format(tform))
 
     # Be flexible on case
     format = match.group('format')
@@ -2026,7 +2001,7 @@ def _parse_ascii_tformat(tform, strict=False):
         precision = match.group('precision')
         if width is None or precision is None:
             if strict:
-                raise VerifyError('Format %r is not unambiguously an ASCII '
+                raise VerifyError('Format {!r} is not unambiguously an ASCII '
                                   'table format.')
             else:
                 width = 0 if width is None else width
@@ -2036,7 +2011,7 @@ def _parse_ascii_tformat(tform, strict=False):
         width = match.group('width')
         if width is None:
             if strict:
-                raise VerifyError('Format %r is not unambiguously an ASCII '
+                raise VerifyError('Format {!r} is not unambiguously an ASCII '
                                   'table format.')
             else:
                 # Just use a default width of 0 if unspecified
@@ -2044,12 +2019,12 @@ def _parse_ascii_tformat(tform, strict=False):
         precision = 0
 
     def convert_int(val):
-        msg = ('Format %r is not valid--field width and decimal precision '
+        msg = ('Format {!r} is not valid--field width and decimal precision '
                'must be integers.')
         try:
             val = int(val)
         except (ValueError, TypeError):
-            raise VerifyError(msg % tform)
+            raise VerifyError(msg.format(tform))
 
         return val
 
@@ -2064,13 +2039,13 @@ def _parse_ascii_tformat(tform, strict=False):
         width, precision = ASCII_DEFAULT_WIDTHS[format]
 
     if width <= 0:
-        raise VerifyError("Format %r not valid--field width must be a "
-                          "positive integeter." % tform)
+        raise VerifyError("Format {!r} not valid--field width must be a "
+                          "positive integeter.".format(tform))
 
     if precision >= width:
-        raise VerifyError("Format %r not valid--the number of decimal digits "
-                          "must be less than the format's total width %s." &
-                          (tform, width))
+        raise VerifyError("Format {!r} not valid--the number of decimal digits "
+                          "must be less than the format's total "
+                          "width {}.".format(tform, width))
 
     return format, width, precision
 
@@ -2096,10 +2071,6 @@ def _scalar_to_format(value):
     FORMATORDER.
     """
 
-    # TODO: Numpy 1.6 and up has a min_scalar_type() function that can handle
-    # this; in the meantime we have to use our own implementation (which for
-    # now is pretty naive)
-
     # First, if value is a string, try to convert to the appropriate scalar
     # value
     for type_ in (int, float, complex):
@@ -2109,21 +2080,14 @@ def _scalar_to_format(value):
         except ValueError:
             continue
 
-    if isinstance(value, int) and value in (0, 1):
-        # Could be a boolean
-        return 'L'
-    elif isinstance(value, int):
-        for char in ('B', 'I', 'J', 'K'):
-            type_ = np.dtype(FITS2NUMPY[char]).type
-            if type_(value) == value:
-                return char
-    elif isinstance(value, float):
-        # For now just assume double precision
-        return 'D'
-    elif isinstance(value, complex):
-        return 'M'
-    else:
-        return 'A' + str(len(value))
+    numpy_dtype_str = np.min_scalar_type(value).str
+    numpy_dtype_str = numpy_dtype_str[1:]  # Strip endianness
+
+    try:
+        fits_format = NUMPY2FITS[numpy_dtype_str]
+        return FITSUPCONVERTERS.get(fits_format, fits_format)
+    except KeyError:
+        return "A" + str(len(value))
 
 
 def _cmp_recformats(f1, f2):
@@ -2171,7 +2135,7 @@ def _convert_fits2record(format):
     elif dtype == 'F':
         output_format = 'f8'
     else:
-        raise ValueError('Illegal format %s.' % format)
+        raise ValueError('Illegal format `{}`.'.format(format))
 
     return output_format
 
@@ -2215,7 +2179,7 @@ def _convert_record2fits(format):
             repeat = ''
         output_format = repeat + NUMPY2FITS[recformat]
     else:
-        raise ValueError('Illegal format %s.' % format)
+        raise ValueError('Illegal format `{}`.'.format(format))
 
     return output_format
 
@@ -2224,10 +2188,10 @@ def _dtype_to_recformat(dtype):
     """
     Utility function for converting a dtype object or string that instantiates
     a dtype (e.g. 'float32') into one of the two character Numpy format codes
-    that have been traditionally used by PyFITS.
+    that have been traditionally used by Astropy.
 
     In particular, use of 'a' to refer to character data is long since
-    deprecated in Numpy, but PyFITS remains heavily invested in its use
+    deprecated in Numpy, but Astropy remains heavily invested in its use
     (something to try to get away from sooner rather than later).
     """
 
@@ -2303,14 +2267,6 @@ def _convert_ascii_format(format, reverse=False):
         # values [for the non-standard J format code just always force 64-bit]
         if format == 'I' and width <= 4:
             recformat = 'i2'
-        elif format == 'F' and width > 7:
-            # 32-bit floats (the default) may not be accurate enough to support
-            # all values that can fit in this field, so upgrade to 64-bit
-            recformat = 'f8'
-        elif format == 'E' and precision > 6:
-            # Again upgrade to a 64-bit int if we require greater decimal
-            # precision
-            recformat = 'f8'
         elif format == 'A':
             recformat += str(width)
 
