@@ -495,6 +495,7 @@ class FITS_rec(np.recarray):
 
         # We got a view; change it back to our class, and add stuff
         out = out.view(type(self))
+        out._uint = self._uint
         out._coldefs = ColDefs(self._coldefs)
         arrays = []
         out._converted = {}
@@ -969,6 +970,10 @@ class FITS_rec(np.recarray):
                         field = test_overflow
                 else:
                     field += bzero
+
+            # mark the column as scaled
+            column._physical_values = True
+
         elif _bool and field.dtype != bool:
             field = np.equal(field, ord('T'))
         elif _str:
