@@ -9,8 +9,8 @@ import warnings
 
 import numpy as np
 
-from .base import DELAYED, ExtensionHDU
-from .image import _ImageBaseHDU, ImageHDU
+from .base import DELAYED, ExtensionHDU, BITPIX2DTYPE, DTYPE2BITPIX
+from .image import ImageHDU
 from .table import BinTableHDU
 from ..card import Card
 from ..column import Column, ColDefs, TDEF_RE
@@ -21,8 +21,9 @@ from ..util import (_is_pseudo_unsigned, _unsigned_zero, _is_int,
                     _get_array_mmap)
 
 from ....extern.six import string_types, iteritems
-from ....utils import lazyproperty, deprecated
-from ....utils.compat import ignored
+from ....extern.six.moves import range
+from ....utils import lazyproperty
+from ....utils.compat import suppress
 from ....utils.exceptions import (AstropyPendingDeprecationWarning,
                                   AstropyUserWarning)
 
@@ -139,7 +140,6 @@ class CompImageHeader(Header):
 
         super(CompImageHeader, self).__setitem__(key, value)
 
-
         if index is not None:
             remapped_keyword = self._remap_keyword(keyword)
             self._table_header[remapped_keyword, index] = value
@@ -160,7 +160,7 @@ class CompImageHeader(Header):
             keyword, index = key, None
 
         if key not in self:
-            raise KeyError("Keyword %r not found." % key)
+            raise KeyError("Keyword {!r} not found.".format(key))
 
         super(CompImageHeader, self).__delitem__(key)
 
@@ -184,7 +184,7 @@ class CompImageHeader(Header):
         elif not isinstance(card, Card):
             raise ValueError(
                 'The value appended to a Header must be either a keyword or '
-                '(keyword, value, [comment]) tuple; got: %r' % card)
+                '(keyword, value, [comment]) tuple; got: {!r}'.format(card))
 
         if self._is_reserved_keyword(card.keyword):
             return
@@ -194,7 +194,13 @@ class CompImageHeader(Header):
 
         remapped_keyword = self._remap_keyword(card.keyword)
         card = Card(remapped_keyword, card.value, card.comment)
-        self._table_header.append(card=card, useblanks=useblanks,
+
+        # Here we disable the use of blank cards, because the call above to
+        # Header.append may have already deleted a blank card in the table
+        # header, thanks to inheritance: Header.append calls 'del self[-1]'
+        # to delete a blank card, which calls CompImageHeader.__deltitem__,
+        # which deletes the blank card both in the image and the table headers!
+        self._table_header.append(card=card, useblanks=False,
                                   bottom=bottom, end=end)
 
     def insert(self, key, card, useblanks=True, after=False):
@@ -217,7 +223,7 @@ class CompImageHeader(Header):
         elif not isinstance(card, Card):
             raise ValueError(
                 'The value inserted into a Header must be either a keyword or '
-                '(keyword, value, [comment]) tuple; got: %r' % card)
+                '(keyword, value, [comment]) tuple; got: {!r}'.format(card))
 
         if self._is_reserved_keyword(card.keyword):
             return
@@ -237,7 +243,12 @@ class CompImageHeader(Header):
 
         card = Card(remapped_keyword, card.value, card.comment)
 
-        self._table_header.insert(remapped_index, card, useblanks=useblanks,
+        # Here we disable the use of blank cards, because the call above to
+        # Header.insert may have already deleted a blank card in the table
+        # header, thanks to inheritance: Header.insert calls 'del self[-1]'
+        # to delete a blank card, which calls CompImageHeader.__delitem__,
+        # which deletes the blank card both in the image and the table headers!
+        self._table_header.insert(remapped_index, card, useblanks=False,
                                   after=after)
 
     def _update(self, card):
@@ -290,9 +301,9 @@ class CompImageHeader(Header):
 
     @classmethod
     def _is_reserved_keyword(cls, keyword, warn=True):
-        msg = ('Keyword %r is reserved for use by the FITS Tiled Image '
+        msg = ('Keyword {!r} is reserved for use by the FITS Tiled Image '
                'Convention and will not be stored in the header for the '
-               'image being compressed.' % keyword)
+               'image being compressed.'.format(keyword))
 
         if keyword == 'TFIELDS':
             if warn:
@@ -331,12 +342,12 @@ class CompImageHeader(Header):
 
         is_naxisn = False
         if keyword[:5] == 'NAXIS':
-            with ignored(ValueError):
+            with suppress(ValueError):
                 index = int(keyword[5:])
                 is_naxisn = index > 0
 
         if is_naxisn:
-            return 'ZNAXIS%d' % index
+            return 'ZNAXIS{}'.format(index)
 
         # If the keyword does not need to be remapped then just return the
         # original keyword
@@ -355,7 +366,7 @@ class CompImageHeader(Header):
         keyword, repeat = self._keyword_from_index(idx)
         remapped_insert_keyword = self._remap_keyword(keyword)
 
-        with ignored(IndexError, KeyError):
+        with suppress(IndexError, KeyError):
             idx = self._table_header._cardindex((remapped_insert_keyword,
                                                  repeat))
 
@@ -431,7 +442,7 @@ class CompImageHDU(BinTableHDU):
 
         quantize_method : int, optional
             Floating point quantization dithering method; can be either
-            ``NO_DITHER`` (-1), ``SUBTRACTIVE_DITHER_1`` (1; default), or
+            ``NO_DITHER`` (-1; default), ``SUBTRACTIVE_DITHER_1`` (1), or
             ``SUBTRACTIVE_DITHER_2`` (2); see note below
 
         dither_seed : int, optional
@@ -459,7 +470,7 @@ class CompImageHDU(BinTableHDU):
                grid of rectangular tiles, and each tile of pixels is
                individually compressed.  The details of this FITS compression
                convention are described at the `FITS Support Office web site
-               <http://fits.gsfc.nasa.gov/registry/tilecompression.html>`_.
+               <https://fits.gsfc.nasa.gov/registry/tilecompression.html>`_.
                Basically, the compressed image tiles are stored in rows of a
                variable length array column in a FITS binary table.  The
                astropy.io.fits recognizes that this binary table extension
@@ -609,9 +620,9 @@ class CompImageHDU(BinTableHDU):
         compression_opts = {}
         for oldarg, newarg in self.DEPRECATED_KWARGS.items():
             if oldarg in kwargs:
-                warnings.warn('Keyword argument %s to %s is pending '
-                              'deprecation; use %s instead' %
-                              (oldarg, self.__class__.__name__, newarg),
+                warnings.warn('Keyword argument {} to {} is pending '
+                              'deprecation; use {} instead'.format(
+                        oldarg, self.__class__.__name__, newarg),
                               AstropyPendingDeprecationWarning)
                 compression_opts[newarg] = kwargs[oldarg]
                 del kwargs[oldarg]
@@ -676,7 +687,7 @@ class CompImageHDU(BinTableHDU):
         if xtension not in ('BINTABLE', 'A3DTABLE'):
             return False
 
-        if 'ZIMAGE' not in header or header['ZIMAGE'] != True:
+        if 'ZIMAGE' not in header or not header['ZIMAGE']:
             return False
 
         if COMPRESSION_SUPPORTED and COMPRESSION_ENABLED:
@@ -723,9 +734,9 @@ class CompImageHDU(BinTableHDU):
             input image header then the default name 'COMPRESSED_IMAGE' is used
 
         compression_type : str, optional
-            compression algorithm 'RICE_1', 'PLIO_1', 'GZIP_1', 'HCOMPRESS_1';
-            if this value is `None`, use value already in the header; if no
-            value already in the header, use 'RICE_1'
+            compression algorithm 'RICE_1', 'PLIO_1', 'GZIP_1', 'GZIP_2',
+            'HCOMPRESS_1'; if this value is `None`, use value already in the
+            header; if no value already in the header, use 'RICE_1'
 
         tile_size : sequence of int, optional
             compression tile sizes as a list; if this value is `None`, use
@@ -775,13 +786,13 @@ class CompImageHDU(BinTableHDU):
             if huge_hdu and not CFITSIO_SUPPORTS_Q_FORMAT:
                 raise IOError(
                     "Astropy cannot compress images greater than 4 GB in size "
-                    "(%s is %s bytes) without CFITSIO >= 3.35" %
-                    ((self.name, self.ver), self.data.nbytes))
+                    "({} is {} bytes) without CFITSIO >= 3.35".format(
+                        (self.name, self.ver), self.data.nbytes))
         else:
             huge_hdu = False
 
         # Update the extension name in the table header
-        if not name and not 'EXTNAME' in self._header:
+        if not name and 'EXTNAME' not in self._header:
             name = 'COMPRESSED_IMAGE'
 
         if name:
@@ -794,11 +805,11 @@ class CompImageHDU(BinTableHDU):
 
         # Set the compression type in the table header.
         if compression_type:
-            if compression_type not in ['RICE_1', 'GZIP_1', 'PLIO_1',
+            if compression_type not in ['RICE_1', 'GZIP_1', 'GZIP_2', 'PLIO_1',
                                         'HCOMPRESS_1']:
                 warnings.warn('Unknown compression type provided.  Default '
-                              '(%s) compression used.' %
-                              DEFAULT_COMPRESSION_TYPE, AstropyUserWarning)
+                              '({}) compression used.'.format(
+                        DEFAULT_COMPRESSION_TYPE), AstropyUserWarning)
                 compression_type = DEFAULT_COMPRESSION_TYPE
 
             self._header.set('ZCMPTYPE', compression_type,
@@ -947,7 +958,8 @@ class CompImageHDU(BinTableHDU):
         # dimensions in the image data array.
         self._header.set('NAXIS1', cols.dtype.itemsize,
                          'width of table in bytes')
-        self._header.set('TFIELDS', ncols, 'number of fields in each row')
+        self._header.set('TFIELDS', ncols, 'number of fields in each row',
+                         after='GCOUNT')
         self._header.set('ZIMAGE', True, 'extension contains compressed image',
                          after=after)
         self._header.set('ZBITPIX', zbitpix,
@@ -1083,7 +1095,7 @@ class CompImageHDU(BinTableHDU):
             if tile_size and len(tile_size) >= idx + 1:
                 ts = tile_size[idx]
             else:
-                if not ztile in self._header:
+                if ztile not in self._header:
                     # Default tile size
                     if not idx:
                         ts = self._image_header['NAXIS1']
@@ -1217,7 +1229,7 @@ class CompImageHDU(BinTableHDU):
                                            SUBTRACTIVE_DITHER_2]:
                     name = QUANTIZE_METHOD_NAMES[DEFAULT_QUANTIZE_METHOD]
                     warnings.warn('Unknown quantization method provided.  '
-                                  'Default method (%s) used.' % name)
+                                  'Default method ({}) used.'.format(name))
                     quantize_method = DEFAULT_QUANTIZE_METHOD
 
                 if quantize_method == NO_DITHER:
@@ -1360,21 +1372,6 @@ class CompImageHDU(BinTableHDU):
             for _ in range(required_blanks - table_blanks):
                 self._header.append()
 
-    @deprecated('0.3', alternative='(refactor your code)', pending=True)
-    def updateHeaderData(self, image_header,
-                         name=None,
-                         compressionType=None,
-                         tileSize=None,
-                         hcompScale=None,
-                         hcompSmooth=None,
-                         quantizeLevel=None):
-        self._update_header_data(image_header, name=name,
-                                 compression_type=compressionType,
-                                 tile_size=tileSize,
-                                 hcomp_scale=hcompScale,
-                                 hcomp_smooth=hcompSmooth,
-                                 quantize_level=quantizeLevel)
-
     @lazyproperty
     def data(self):
         # The data attribute is the image data (not the table data).
@@ -1404,7 +1401,11 @@ class CompImageHDU(BinTableHDU):
             if self._bscale != 1:
                 np.multiply(data, self._bscale, data)
             if self._bzero != 0:
-                data += self._bzero
+                # We have to explcitly cast self._bzero to prevent numpy from
+                # raising an error when doing self.data += self._bzero, and we
+                # do this instead of self.data = self.data + self._bzero to
+                # avoid doubling memory usage.
+                np.add(data, self._bzero, out=data, casting='unsafe')
 
             if zblank is not None:
                 data = np.where(blanks, np.nan, data)
@@ -1418,9 +1419,9 @@ class CompImageHDU(BinTableHDU):
     def data(self, data):
         if (data is not None) and (not isinstance(data, np.ndarray) or
                 data.dtype.fields is not None):
-            raise TypeError('CompImageHDU data has incorrect type:%s; '
-                            'dtype.fields = %s' %
-                            (type(data), data.dtype.fields))
+            raise TypeError('CompImageHDU data has incorrect type:{}; '
+                            'dtype.fields = {}'.format(
+                    type(data), data.dtype.fields))
 
     @lazyproperty
     def compressed_data(self):
@@ -1461,12 +1462,6 @@ class CompImageHDU(BinTableHDU):
             # welcome go ahead and force a garbage collection
             gc.collect()
 
-    @lazyproperty
-    @deprecated('0.3', alternative='the ``compressed_data`` attribute',
-                pending=True)
-    def compData(self):
-        return self.compressed_data
-
     @property
     def shape(self):
         """
@@ -1512,7 +1507,6 @@ class CompImageHDU(BinTableHDU):
         else:
             image_header.set('XTENSION', 'IMAGE', before=0)
 
-
         image_header.set('BITPIX', self._header['ZBITPIX'],
                          self._header.comments['ZBITPIX'], before=1)
 
@@ -1533,7 +1527,7 @@ class CompImageHDU(BinTableHDU):
         for keyword in list(image_header['NAXIS?*']):
             try:
                 n = int(keyword[5:])
-            except:
+            except Exception:
                 continue
 
             if n > naxis:
@@ -1624,9 +1618,9 @@ class CompImageHDU(BinTableHDU):
             for idx in range(self.header['NAXIS']):
                 _shape += (self.header['NAXIS' + str(idx + 1)],)
 
-            _format = _ImageBaseHDU.NumCode[self.header['BITPIX']]
+            _format = BITPIX2DTYPE[self.header['BITPIX']]
 
-        return (self.name, class_name, len(self.header), _shape,
+        return (self.name, self.ver, class_name, len(self.header), _shape,
                 _format)
 
     def _update_compressed_data(self):
@@ -1635,7 +1629,7 @@ class CompImageHDU(BinTableHDU):
         """
 
         # Check to see that the image_header matches the image data
-        image_bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
+        image_bitpix = DTYPE2BITPIX[self.data.dtype.name]
 
         if image_bitpix != self._orig_bitpix or self.data.shape != self.shape:
             self._update_header_data(self.header)
@@ -1649,13 +1643,19 @@ class CompImageHDU(BinTableHDU):
             # Convert the unsigned array to signed
             self.data = np.array(
                 self.data - _unsigned_zero(self.data.dtype),
-                dtype='=i%d' % self.data.dtype.itemsize)
+                dtype='=i{}'.format(self.data.dtype.itemsize))
             should_swap = False
         else:
             should_swap = not self.data.dtype.isnative
 
         if should_swap:
-            self.data.byteswap(True)
+
+            if self.data.flags.writeable:
+                self.data.byteswap(True)
+            else:
+                # For read-only arrays, there is no way around making
+                # a byteswapped copy of the data.
+                self.data = self.data.byteswap(False)
 
         try:
             nrows = self._header['NAXIS2']
@@ -1669,7 +1669,6 @@ class CompImageHDU(BinTableHDU):
             # First delete the original compressed data, if it exists
             del self.compressed_data
 
-
             # Compress the data.
             # The current implementation of compress_hdu assumes the empty
             # compressed data table has already been initialized in
@@ -1693,16 +1692,6 @@ class CompImageHDU(BinTableHDU):
         self.compressed_data._heapoffset = self._theap
         self.compressed_data._heapsize = heapsize
 
-    @deprecated('0.3', alternative='(refactor your code)')
-    def updateCompressedData(self):
-        self._update_compressed_data()
-
-    @deprecated('0.3',
-                alternative='(refactor your code; this function no '
-                            'longer does anything)')
-    def updateHeader(self):
-        pass
-
     def scale(self, type=None, option='old', bscale=1, bzero=0):
         """
         Scale image data by using ``BSCALE`` and ``BZERO``.
@@ -1737,7 +1726,7 @@ class CompImageHDU(BinTableHDU):
 
         # Determine the destination (numpy) data type
         if type is None:
-            type = _ImageBaseHDU.NumCode[self._bitpix]
+            type = BITPIX2DTYPE[self._bitpix]
         _type = getattr(np, type)
 
         # Determine how to scale the data
@@ -1768,12 +1757,16 @@ class CompImageHDU(BinTableHDU):
 
         # Do the scaling
         if _zero != 0:
-            self.data += -_zero
+            # We have to explicitly cast self._bzero to prevent numpy from
+            # raising an error when doing self.data -= _zero, and we
+            # do this instead of self.data = self.data - _zero to
+            # avoid doubling memory usage.
+            np.subtract(self.data, _zero, out=self.data, casting='unsafe')
             self.header['BZERO'] = _zero
         else:
             # Delete from both headers
             for header in (self.header, self._header):
-                with ignored(KeyError):
+                with suppress(KeyError):
                     del header['BZERO']
 
         if _scale != 1:
@@ -1781,14 +1774,14 @@ class CompImageHDU(BinTableHDU):
             self.header['BSCALE'] = _scale
         else:
             for header in (self.header, self._header):
-                with ignored(KeyError):
+                with suppress(KeyError):
                     del header['BSCALE']
 
         if self.data.dtype.type != _type:
             self.data = np.array(np.around(self.data), dtype=_type)  # 0.7.7.1
 
         # Update the BITPIX Card to match the data
-        self._bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
+        self._bitpix = DTYPE2BITPIX[self.data.dtype.name]
         self._bzero = self.header.get('BZERO', 0)
         self._bscale = self.header.get('BSCALE', 1)
         # Update BITPIX for the image header specifically
@@ -1809,7 +1802,7 @@ class CompImageHDU(BinTableHDU):
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._scale_back:
-            self.scale(_ImageBaseHDU.NumCode[self._orig_bitpix])
+            self.scale(BITPIX2DTYPE[self._orig_bitpix])
 
         if self._has_data:
             self._update_compressed_data()
@@ -1875,9 +1868,6 @@ class CompImageHDU(BinTableHDU):
         if (closed and self._data_loaded and
                 _get_array_mmap(self.compressed_data) is not None):
             del self.compressed_data
-            # Close off the deprected compData attribute as well if it has been
-            # used
-            del self.compData
 
     # TODO: This was copied right out of _ImageBaseHDU; get rid of it once we
     # find a way to rewrite this class as either a subclass or wrapper for an
@@ -1910,7 +1900,7 @@ class CompImageHDU(BinTableHDU):
                 # Make sure to delete from both the image header and the table
                 # header; later this will be streamlined
                 for header in (self.header, self._header):
-                    with ignored(KeyError):
+                    with suppress(KeyError):
                         del header[keyword]
                         # Since _update_header_scale_info can, currently, be
                         # called *after* _prewriteto(), replace these with
@@ -1920,7 +1910,7 @@ class CompImageHDU(BinTableHDU):
             if dtype is None:
                 dtype = self._dtype_for_bitpix()
             if dtype is not None:
-                self.header['BITPIX'] = _ImageBaseHDU.ImgCode[dtype.name]
+                self.header['BITPIX'] = DTYPE2BITPIX[dtype.name]
 
             self._bzero = 0
             self._bscale = 1
@@ -1935,12 +1925,12 @@ class CompImageHDU(BinTableHDU):
                 "Seed for random dithering must be either between 1 and "
                 "10000 inclusive, 0 for autogeneration from the system "
                 "clock, or -1 for autogeneration from a checksum of the first "
-                "image tile (got %s)" % seed)
+                "image tile (got {})".format(seed))
 
         if seed == DITHER_SEED_CHECKSUM:
             # Determine the tile dimensions from the ZTILEn keywords
             naxis = self._header['ZNAXIS']
-            tile_dims = [self._header['ZTILE%d' % (idx + 1)]
+            tile_dims = [self._header['ZTILE{}'.format(idx + 1)]
                          for idx in range(naxis)]
             tile_dims.reverse()
 
