@@ -91,11 +91,12 @@
 /* Include the Python C API */
 
 #include <math.h>
+#include <string.h>
 
 #include <Python.h>
+#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #include <numpy/arrayobject.h>
 #include <fitsio2.h>
-#include <string.h>
 #include "compressionmodule.h"
 
 /* Some defines for Python3 support--bytes objects should be used where */
@@ -184,7 +185,7 @@ void bitpix_to_datatypes(int bitpix, int* datatype, int* npdatatype) {
     switch (bitpix) {
         case BYTE_IMG:
             *datatype = TBYTE;
-            *npdatatype = NPY_INT8;
+            *npdatatype = NPY_UINT8;
             break;
         case SHORT_IMG:
             *datatype = TSHORT;
@@ -221,6 +222,8 @@ int compress_type_from_string(char* zcmptype) {
         return RICE_1;
     } else if (0 == strcmp(zcmptype, "GZIP_1")) {
         return GZIP_1;
+    } else if (0 == strcmp(zcmptype, "GZIP_2")) {
+        return GZIP_2;
     } else if (0 == strcmp(zcmptype, "PLIO_1")) {
         return PLIO_1;
     } else if (0 == strcmp(zcmptype, "HCOMPRESS_1")) {
@@ -270,7 +273,7 @@ int get_header_string(PyObject* header, char* keyword, char* val, char* def) {
     }
     else {
         PyErr_Clear();
-        *val = def;
+        strncpy(val, def, 72);
         retval = 1;
     }
 
@@ -628,7 +631,8 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
                 get_header_float(header, keyword, &(Fptr->hcomp_scale),
                                  DEFAULT_HCOMP_SCALE);
             }
-        } else if (Fptr->zbitpix < 0 && 0 == strcmp(zname, "NOISEBIT")) {
+        }
+        if (Fptr->zbitpix < 0 && 0 == strcmp(zname, "NOISEBIT")) {
              get_header_float(header, keyword, &(Fptr->quantize_level),
                               DEFAULT_QUANTIZE_LEVEL);
              if (Fptr->quantize_level == 0.0) {
@@ -724,6 +728,8 @@ void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
     }
 
     if (0 != get_header_int(header, "ZNAXIS", &znaxis, 0)) {
+        PyErr_SetString(PyExc_TypeError,
+                        "ZNAXIS keyword not present in header.");
         goto fail;
     }
 
@@ -744,6 +750,8 @@ void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
 
     // Get the ZBITPIX header value; if this is missing we're in trouble
     if (0 != get_header_int(header, "ZBITPIX", &zbitpix, 0)) {
+        PyErr_SetString(PyExc_TypeError,
+                        "ZBITPIX keyword not present in header.");
         goto fail;
     }
 
@@ -763,6 +771,13 @@ void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
     }
 
     *buf = calloc(*bufsize, sizeof(char));
+    if (*buf == NULL) {
+        // Checking if calloc failed.
+        PyErr_SetString(PyExc_MemoryError,
+                        "Failed to allocate memory for output data buffer.");
+        goto fail;
+    }
+
 fail:
     Py_XDECREF(header);
     return;
@@ -887,10 +902,10 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     PyObject* retval = NULL;
     tcolumn* columns = NULL;
 
-    void* outbuf;
+    void* outbuf = NULL;
     size_t outbufsize;
 
-    PyArrayObject* indata;
+    PyArrayObject* indata = NULL;
     PyArrayObject* tmp;
     npy_intp znaxis;
     int datatype;
@@ -898,7 +913,7 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     unsigned long long heapsize;
 
     fitsfile* fileptr;
-    FITSfile* Fptr;
+    FITSfile* Fptr = NULL;
     int status = 0;
 
     if (!PyArg_ParseTuple(args, "O:compression.compress_hdu", &hdu))
@@ -931,8 +946,8 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
 
     indata = (PyArrayObject*) PyObject_GetAttrString(hdu, "data");
 
-    fits_write_img(fileptr, datatype, 1, PyArray_SIZE(indata), indata->data,
-                   &status);
+    fits_write_img(fileptr, datatype, 1, PyArray_SIZE(indata),
+                   PyArray_DATA(indata), &status);
     if (status != 0) {
         process_status_err(status);
         goto fail;
@@ -962,6 +977,7 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     tmp = (PyArrayObject*) PyArray_SimpleNewFromData(1, &znaxis, NPY_UBYTE,
                                                      outbuf);
 
+    PyArray_ENABLEFLAGS(tmp, NPY_ARRAY_OWNDATA);
 
     // Leaves refcount of tmp untouched, so its refcount should remain as 1
     retval = Py_BuildValue("KN", heapsize, tmp);
@@ -977,7 +993,9 @@ fail:
 cleanup:
     if (columns != NULL) {
         PyMem_Free(columns);
-        Fptr->tableptr = NULL;
+	if (Fptr != NULL) {
+	    Fptr->tableptr = NULL;
+	}
     }
 
     if (fileptr != NULL) {
@@ -1057,8 +1075,8 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
     /* Create and allocate a new array for the decompressed data */
     outdata = (PyArrayObject*) PyArray_SimpleNew(zndim, znaxis, npdatatype);
 
-    fits_read_img(fileptr, datatype, 1, arrsize, NULL, outdata->data, &anynul,
-                  &status);
+    fits_read_img(fileptr, datatype, 1, arrsize, NULL, PyArray_DATA(outdata),
+                  &anynul, &status);
     if (status != 0) {
         process_status_err(status);
         outdata = NULL;
