@@ -163,9 +163,33 @@ class TestConvenience(FitsTestCase):
         fits.tabledump(temp_filename, datafile=self.temp('test_tb.txt'))
         assert os.path.isfile(self.temp('test_tb.txt'))
 
+    def test_append_filename(self):
+        """
+        Test fits.append with a filename argument.
+        """
+        data = np.arange(6)
+        testfile = self.temp('test_append_1.fits')
+
+        # Test case 1: creation of file
+        fits.append(testfile, data=data, checksum=True)
+
+        # Test case 2: append to existing file, with verify=True
+        # Also test that additional keyword can be passed to fitsopen
+        fits.append(testfile, data=data * 2, checksum=True, ignore_blank=True)
+
+        # Test case 3: append to existing file, with verify=False
+        fits.append(testfile, data=data * 3, checksum=True, verify=False)
+
+        with fits.open(testfile, checksum=True) as hdu1:
+            np.testing.assert_array_equal(hdu1[0].data, data)
+            np.testing.assert_array_equal(hdu1[1].data, data * 2)
+            np.testing.assert_array_equal(hdu1[2].data, data * 3)
+
     @pytest.mark.parametrize('mode', ['wb', 'wb+', 'ab', 'ab+'])
     def test_append_filehandle(self, tmpdir, mode):
-
+        """
+        Test fits.append with a file handle argument.
+        """
         append_file = tmpdir.join('append.fits')
         with append_file.open(mode) as handle:
             fits.append(filename=handle, data=np.ones((4, 4)))
