@@ -247,7 +247,9 @@ No differences found.\n""".format(version, tmp_a, tmp_b)
         tmp_d = self.temp('sub/')
         assert fitsdiff.main(["-q", self.data_dir, tmp_d]) == 1
         assert fitsdiff.main(["-q", tmp_d, self.data_dir]) == 1
-        assert fitsdiff.main(["-q", self.data_dir, self.data_dir]) == 0
+        with pytest.warns(UserWarning, match="Field 'ORBPARM' has a repeat "
+                          "count of 0 in its format code"):
+            assert fitsdiff.main(["-q", self.data_dir, self.data_dir]) == 0
 
         # no match
         tmp_c = self.data('arange.fits')
@@ -256,7 +258,10 @@ No differences found.\n""".format(version, tmp_a, tmp_b)
         assert "'arange.fits' has no match in" in err
 
         # globbing
-        assert fitsdiff.main(["-q", self.data_dir+'/*.fits', self.data_dir]) == 0
+        with pytest.warns(UserWarning, match="Field 'ORBPARM' has a repeat "
+                          "count of 0 in its format code"):
+            assert fitsdiff.main(["-q", self.data_dir+'/*.fits',
+                                  self.data_dir]) == 0
         assert fitsdiff.main(["-q", self.data_dir+'/g*.fits', tmp_d]) == 0
 
         # one file and a directory
