@@ -1,29 +1,29 @@
 # Licensed under a 3-clause BSD style license - see PYFITS.rst
 
-import os
 import platform
 
+import pytest
 import numpy as np
+from numpy.testing import assert_array_equal
 
 from ....io import fits
 from . import FitsTestCase
-import platform
-from ....tests.helper import pytest, ignore_warnings
+from ....tests.helper import ignore_warnings
 
 
 class TestUintFunctions(FitsTestCase):
     @classmethod
     def setup_class(cls):
-        cls.utypes = ('u2','u4','u8')
+        cls.utypes = ('u2', 'u4', 'u8')
         cls.utype_map = {'u2': np.uint16, 'u4': np.uint32, 'u8': np.uint64}
         cls.itype_map = {'u2': np.int16, 'u4': np.int32, 'u8': np.int64}
         cls.format_map = {'u2': 'I', 'u4': 'J', 'u8': 'K'}
 
     # Test of 64-bit compressed image is disabled.  cfitsio library doesn't
     # like it
-    @pytest.mark.parametrize(('utype','compressed'),
+    @pytest.mark.parametrize(('utype', 'compressed'),
         [('u2', False), ('u4', False), ('u8', False), ('u2', True),
-         ('u4',True)]) #,('u8',True)])
+         ('u4', True)])  # ,('u8',True)])
     def test_uint(self, utype, compressed):
         bits = 8*int(utype[1])
         if platform.architecture()[0] == '64bit' or bits != 64:
@@ -37,7 +37,7 @@ class TestUintFunctions(FitsTestCase):
             hdu.scale('int{0:d}'.format(bits), '', bzero=2 ** (bits-1))
 
             with ignore_warnings():
-                hdu.writeto(self.temp('tempfile.fits'), clobber=True)
+                hdu.writeto(self.temp('tempfile.fits'), overwrite=True)
 
             with fits.open(self.temp('tempfile.fits'), uint=True) as hdul:
                 assert hdul[hdu_number].data.dtype == self.utype_map[utype]
@@ -55,7 +55,7 @@ class TestUintFunctions(FitsTestCase):
                         # TODO: Enable these lines if CompImageHDUs ever grow
                         # .section support
                         sec = hdul[hdu_number].section[:1]
-                        assert sec.dtype.name == 'uint%s' % bits
+                        assert sec.dtype.name == 'uint{}'.format(bits)
                         assert (sec == d1[:1]).all()
 
     @pytest.mark.parametrize('utype', ('u2', 'u4', 'u8'))
@@ -69,7 +69,7 @@ class TestUintFunctions(FitsTestCase):
         if platform.architecture()[0] == '64bit' or bits != 64:
             bzero = self.utype_map[utype](2**(bits-1))
             one = self.utype_map[utype](1)
-            u0 = np.arange(bits+1,dtype=self.utype_map[utype])
+            u0 = np.arange(bits+1, dtype=self.utype_map[utype])
             u = 2**u0 - one
             if bits == 64:
                 u[63] = bzero - one
@@ -88,10 +88,10 @@ class TestUintFunctions(FitsTestCase):
             # table.data.base.base
             assert (table.data.base.base[utype] == uu).all()
             hdu0 = fits.PrimaryHDU()
-            hdulist = fits.HDUList([hdu0,table])
+            hdulist = fits.HDUList([hdu0, table])
 
             with ignore_warnings():
-                hdulist.writeto(self.temp('tempfile.fits'), clobber=True)
+                hdulist.writeto(self.temp('tempfile.fits'), overwrite=True)
 
             # Test write of unsigned int
             del hdulist
@@ -105,7 +105,7 @@ class TestUintFunctions(FitsTestCase):
             v = u.view(dtype=[(utype, self.utype_map[utype])])
 
             with ignore_warnings():
-                fits.writeto(self.temp('tempfile2.fits'), v, clobber=True)
+                fits.writeto(self.temp('tempfile2.fits'), v, overwrite=True)
 
             with fits.open(self.temp('tempfile2.fits'), uint=True) as hdulist3:
                 hdudata3 = hdulist3[1].data
@@ -113,3 +113,30 @@ class TestUintFunctions(FitsTestCase):
                         table.data.base.base[utype]).all()
                 assert (hdudata3[utype] == table.data[utype]).all()
                 assert (hdudata3[utype] == u).all()
+
+    def test_uint_slice(self):
+        """
+        Fix for https://github.com/astropy/astropy/issues/5490
+        if data is sliced first, make sure the data is still converted as uint
+        """
+        # create_data:
+        dataref = np.arange(2**16, dtype=np.uint16)
+        tbhdu = fits.BinTableHDU.from_columns([
+            fits.Column(name='a', format='I',
+                        array=np.arange(2**16, dtype=np.int16)),
+            fits.Column(name='b', format='I', bscale=1, bzero=2**15,
+                        array=dataref)
+        ])
+        tbhdu.writeto(self.temp('test_scaled_slicing.fits'))
+
+        with fits.open(self.temp('test_scaled_slicing.fits')) as hdulist:
+            data = hdulist[1].data
+        assert_array_equal(data['b'], dataref)
+        sel = data['a'] >= 0
+        assert_array_equal(data[sel]['b'], dataref[sel])
+        assert data[sel]['b'].dtype == dataref[sel].dtype
+
+        with fits.open(self.temp('test_scaled_slicing.fits')) as hdulist:
+            data = hdulist[1].data
+        assert_array_equal(data[sel]['b'], dataref[sel])
+        assert data[sel]['b'].dtype == dataref[sel].dtype
