@@ -57,14 +57,14 @@ def is_hdf5(origin, filepath, fileobj, *args, **kwargs):
     except ImportError:
         return False
     else:
-        return isinstance(args[0], (h5py.highlevel.File, h5py.highlevel.Group, h5py.highlevel.Dataset))
+        return isinstance(args[0], (h5py.File, h5py.Group, h5py.Dataset))
 
 
 def read_table_hdf5(input, path=None):
     """
     Read a Table object from an HDF5 file
 
-    This requires `h5py <http://h5py.org/>`_ to be installed. If more than one
+    This requires `h5py <http://www.h5py.org/>`_ to be installed. If more than one
     table is present in the HDF5 file or group, the first table is read in and
     a warning is displayed.
 
@@ -84,7 +84,7 @@ def read_table_hdf5(input, path=None):
     except ImportError:
         raise Exception("h5py is required to read and write HDF5 files")
 
-    if isinstance(input, (h5py.highlevel.File, h5py.highlevel.Group)):
+    if isinstance(input, (h5py.File, h5py.Group)):
 
         # If a path was specified, follow the path
 
@@ -99,7 +99,7 @@ def read_table_hdf5(input, path=None):
         # is one we can proceed otherwise an error is raised. If it is a
         # dataset, we just proceed with the reading.
 
-        if isinstance(input, h5py.highlevel.Group):
+        if isinstance(input, h5py.Group):
 
             # Find all structured arrays in group
             arrays = _find_all_structured_arrays(input)
@@ -115,7 +115,7 @@ def read_table_hdf5(input, path=None):
                               AstropyUserWarning)
                 return read_table_hdf5(input, path=path)
 
-    elif not isinstance(input, h5py.highlevel.Dataset):
+    elif not isinstance(input, h5py.Dataset):
 
         # If a file object was passed, then we need to extract the filename
         # because h5py cannot properly read in file objects.
@@ -168,7 +168,7 @@ def write_table_hdf5(table, output, path=None, compression=False,
     """
     Write a Table object to an HDF5 file
 
-    This requires `h5py <http://h5py.org/>`_ to be installed.
+    This requires `h5py <http://www.h5py.org/>`_ to be installed.
 
     Parameters
     ----------
@@ -217,7 +217,7 @@ def write_table_hdf5(table, output, path=None, compression=False,
     else:
         group, name = None, path
 
-    if isinstance(output, (h5py.highlevel.File, h5py.highlevel.Group)):
+    if isinstance(output, (h5py.File, h5py.Group)):
 
         if group:
             try:
