@@ -64,7 +64,7 @@ def delay_doc_updates(cls):
 
     Notes
     -----
-    Registering mutliple readers and writers can cause significant overhead
+    Registering multiple readers and writers can cause significant overhead
     because the documentation of the corresponding ``read`` and ``write``
     methods are build every time.
 
@@ -586,15 +586,19 @@ def _is_best_match(class1, class2, format_classes):
     Determine if class2 is the "best" match for class1 in the list
     of classes.  It is assumed that (class2 in classes) is True.
     class2 is the the best match if:
-      - class1 is class2 => class1 was directly registered.
-      - OR class1 is a subclass of class2 and class1 is not in classes.
-        In this case the subclass will use the parent reader/writer.
+
+    - ``class1`` is a subclass of ``class2`` AND
+    - ``class2`` is the nearest ancestor of ``class1`` that is in classes
+      (which includes the case that ``class1 is class2``)
     """
-    # The set with the classes is only created if class1 is not class2 and
-    # class1 is a subclass of class2.
-    return (class1 is class2 or
-            (issubclass(class1, class2) and
-             class1 not in {cls for fmt, cls in format_classes}))
+    if issubclass(class1, class2):
+        classes = {cls for fmt, cls in format_classes}
+        for parent in class1.__mro__:
+            if parent is class2:  # class2 is closest registered ancestor
+                return True
+            if parent in classes:  # class2 was superceded
+                return False
+    return False
 
 
 def _get_valid_format(mode, cls, path, fileobj, args, kwargs):
@@ -607,7 +611,9 @@ def _get_valid_format(mode, cls, path, fileobj, args, kwargs):
 
     if len(valid_formats) == 0:
         format_table_str = _get_format_table_str(cls, mode.capitalize())
-        raise IORegistryError("Format could not be identified.\n"
+        raise IORegistryError("Format could not be identified based on the"
+                              " file name or contents, please provide a"
+                              " 'format' argument.\n"
                               "The available formats are:\n"
                               "{0}".format(format_table_str))
     elif len(valid_formats) > 1:
