@@ -3,32 +3,28 @@
 Test the conversion to/from astropy.table
 """
 
-import os
-import shutil
-import tempfile
-from distutils import version
+# TEST_UNICODE_LITERALS
 
+import io
+import os
+
+import pytest
 import numpy as np
-from ....tests.helper import pytest
 
 from ....utils.data import get_pkg_data_filename, get_pkg_data_fileobj
 from ..table import parse, writeto
 from .. import tree
+from ....extern.six.moves import zip
 
-numpy_lt_1p5 = version.LooseVersion(np.__version__) < version.LooseVersion('1.5')
-
-TMP_DIR = None
-def setup_module():
-    global TMP_DIR
-    TMP_DIR = tempfile.mkdtemp()
+try:
+    import pathlib
+except ImportError:
+    HAS_PATHLIB = False
+else:
+    HAS_PATHLIB = True
 
 
-def teardown_module():
-    shutil.rmtree(TMP_DIR)
-
-
-@pytest.mark.xfail('numpy_lt_1p5')
-def test_table():
+def test_table(tmpdir):
     # Read the VOTABLE
     votable = parse(
         get_pkg_data_filename('data/regression.xml'),
@@ -79,11 +75,10 @@ def test_table():
         if 'arraysize' in d:
             assert field.arraysize == d['arraysize']
 
-    writeto(votable2, os.path.join(TMP_DIR, "through_table.xml"))
+    writeto(votable2, os.path.join(str(tmpdir), "through_table.xml"))
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
-def test_read_through_table_interface():
+def test_read_through_table_interface(tmpdir):
     from ....table import Table
 
     with get_pkg_data_fileobj('data/regression.xml', encoding='binary') as fd:
@@ -91,7 +86,10 @@ def test_read_through_table_interface():
 
     assert len(t) == 5
 
-    fn = os.path.join(TMP_DIR, "table_interface.xml")
+    # Issue 8354
+    assert t['float'].format is None
+
+    fn = os.path.join(str(tmpdir), "table_interface.xml")
     t.write(fn, table_id='FOO', format='votable')
 
     with open(fn, 'rb') as fd:
@@ -100,7 +98,6 @@ def test_read_through_table_interface():
     assert len(t2) == 5
 
 
-@pytest.mark.xfail('numpy_lt_1p5')
 def test_read_through_table_interface2():
     from ....table import Table
 
@@ -108,3 +105,77 @@ def test_read_through_table_interface2():
         t = Table.read(fd, format='votable', table_id='last_table')
 
     assert len(t) == 0
+
+
+def test_names_over_ids():
+    with get_pkg_data_fileobj('data/names.xml', encoding='binary') as fd:
+        votable = parse(fd)
+
+    table = votable.get_first_table().to_table(use_names_over_ids=True)
+
+    assert table.colnames == [
+        'Name', 'GLON', 'GLAT', 'RAdeg', 'DEdeg', 'Jmag', 'Hmag', 'Kmag',
+        'G3.6mag', 'G4.5mag', 'G5.8mag', 'G8.0mag', '4.5mag', '8.0mag',
+        'Emag', '24mag', 'f_Name']
+
+
+def test_table_read_with_unnamed_tables():
+    """
+    Issue #927
+    """
+    from ....table import Table
+
+    with get_pkg_data_fileobj('data/names.xml', encoding='binary') as fd:
+        t = Table.read(fd, format='votable')
+
+    assert len(t) == 1
+
+
+@pytest.mark.skipif('not HAS_PATHLIB')
+def test_votable_path_object():
+    """
+    Testing when votable is passed as pathlib.Path object #4412.
+    """
+    fpath = pathlib.Path(get_pkg_data_filename('data/names.xml'))
+    table = parse(fpath).get_first_table().to_table()
+
+    assert len(table) == 1
+    assert int(table[0][3]) == 266
+
+
+def test_from_table_without_mask():
+    from ....table import Table, Column
+    t = Table()
+    c = Column(data=[1, 2, 3], name='a')
+    t.add_column(c)
+    output = io.BytesIO()
+    t.write(output, format='votable')
+
+
+def test_write_with_format():
+    from ....table import Table, Column
+    t = Table()
+    c = Column(data=[1, 2, 3], name='a')
+    t.add_column(c)
+
+    output = io.BytesIO()
+    t.write(output, format='votable', tabledata_format="binary")
+    obuff = output.getvalue()
+    assert b'VOTABLE version="1.3"' in obuff
+    assert b'BINARY' in obuff
+    assert b'TABLEDATA' not in obuff
+
+    output = io.BytesIO()
+    t.write(output, format='votable', tabledata_format="binary2")
+    obuff = output.getvalue()
+    assert b'VOTABLE version="1.3"' in obuff
+    assert b'BINARY2' in obuff
+    assert b'TABLEDATA' not in obuff
+
+
+def test_empty_table():
+    votable = parse(
+        get_pkg_data_filename('data/empty_table.xml'),
+        pedantic=False)
+    table = votable.get_first_table()
+    astropy_table = table.to_table()
