@@ -342,7 +342,7 @@ class Gaussian2D(Fittable2DModel):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Gaussian_function
+    .. [1] https://en.wikipedia.org/wiki/Gaussian_function
     """
 
     amplitude = Parameter(default=1)
@@ -688,7 +688,7 @@ class Sersic1D(Fittable1DModel):
     Parameters
     ----------
     amplitude : float
-        Central surface brightness, within r_eff.
+        Surface brightness at r_eff.
     r_eff : float
         Effective (half-light) radius
     n : float
@@ -1703,15 +1703,6 @@ class Box1D(Fittable1DModel):
         else:
             return result
 
-    @classmethod
-    def fit_deriv(cls, x, amplitude, x_0, width):
-        """One dimensional Box model derivative with respect to parameters"""
-
-        d_amplitude = cls.evaluate(x, 1, x_0, width)
-        d_x_0 = np.zeros_like(x)
-        d_width = np.zeros_like(x)
-        return [d_amplitude, d_x_0, d_width]
-
     @property
     def bounding_box(self):
         """
@@ -2193,7 +2184,7 @@ class AiryDisk2D(Fittable2DModel):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Airy_disk
+    .. [1] https://en.wikipedia.org/wiki/Airy_disk
     """
 
     amplitude = Parameter(default=1)
@@ -2316,7 +2307,7 @@ class Moffat1D(Fittable1DModel):
         Derivation of the formula is available in
         `this notebook by Yoonsoo Bach <http://nbviewer.jupyter.org/github/ysbach/AO_2017/blob/master/04_Ground_Based_Concept.ipynb#1.2.-Moffat>`_.
         """
-        return 2.0 * self.gamma * np.sqrt(2.0 ** (1.0 / self.alpha) - 1.0)
+        return 2.0 * np.abs(self.gamma) * np.sqrt(2.0 ** (1.0 / self.alpha) - 1.0)
 
     @staticmethod
     def evaluate(x, amplitude, x_0, gamma, alpha):
@@ -2393,7 +2384,7 @@ class Moffat2D(Fittable2DModel):
         Derivation of the formula is available in
         `this notebook by Yoonsoo Bach <http://nbviewer.jupyter.org/github/ysbach/AO_2017/blob/master/04_Ground_Based_Concept.ipynb#1.2.-Moffat>`_.
         """
-        return 2.0 * self.gamma * np.sqrt(2.0 ** (1.0 / self.alpha) - 1.0)
+        return 2.0 * np.abs(self.gamma) * np.sqrt(2.0 ** (1.0 / self.alpha) - 1.0)
 
     @staticmethod
     def evaluate(x, y, amplitude, x_0, y_0, gamma, alpha):
@@ -2443,7 +2434,7 @@ class Sersic2D(Fittable2DModel):
     Parameters
     ----------
     amplitude : float
-        Central surface brightness, within r_eff.
+        Surface brightness at r_eff.
     r_eff : float
         Effective (half-light) radius
     n : float
