@@ -6,20 +6,31 @@ from __future__ import (absolute_import, unicode_literals, division,
 import inspect
 from copy import deepcopy
 
+import pytest
 import numpy as np
 
-from numpy.testing.utils import (assert_allclose, assert_array_equal,
-                                 assert_almost_equal)
+from numpy.testing import assert_allclose, assert_array_equal
 
 from ...extern.six.moves import cPickle as pickle
-from ...tests.helper import pytest
 
+from ...utils import minversion
 from ..core import Model, ModelDefinitionError
 from ..parameters import Parameter
 from ..models import (Const1D, Shift, Scale, Rotation2D, Gaussian1D,
                       Gaussian2D, Polynomial1D, Polynomial2D,
                       Chebyshev2D, Legendre2D, Chebyshev1D, Legendre1D,
-                      AffineTransformation2D, Identity, Mapping)
+                      AffineTransformation2D, Identity, Mapping,
+                      Tabular1D)
+
+
+try:
+    import scipy
+    from scipy import optimize  # pylint: disable=W0611
+    HAS_SCIPY = True
+except ImportError:
+    HAS_SCIPY = False
+
+HAS_SCIPY_14 = HAS_SCIPY and minversion(scipy, "0.14")
 
 
 @pytest.mark.parametrize(('expr', 'result'),
@@ -50,6 +61,34 @@ def test_two_model_class_arithmetic_1d(expr, result):
     assert isinstance(out, float)
 
 
+@pytest.mark.parametrize(('expr', 'result'),
+                         [(lambda x, y: x + y, [5.0, 5.0]),
+                          (lambda x, y: x - y, [-1.0, -1.0]),
+                          (lambda x, y: x * y, [6.0, 6.0]),
+                          (lambda x, y: x / y, [2.0 / 3.0, 2.0 / 3.0]),
+                          (lambda x, y: x ** y, [8.0, 8.0])])
+def test_model_set(expr, result):
+    s = expr(Const1D((2, 2), n_models=2), Const1D((3, 3), n_models=2))
+    out = s(0, model_set_axis=False)
+
+    assert_array_equal(out, result)
+
+
+@pytest.mark.parametrize(('expr', 'result'),
+                         [(lambda x, y: x + y, [5.0, 5.0]),
+                          (lambda x, y: x - y, [-1.0, -1.0]),
+                          (lambda x, y: x * y, [6.0, 6.0]),
+                          (lambda x, y: x / y, [2.0 / 3.0, 2.0 / 3.0]),
+                          (lambda x, y: x ** y, [8.0, 8.0])])
+def test_model_set_raises_value_error(expr, result):
+    """Check that creating model sets with components whose _n_models are
+       different raise a value error
+    """
+
+    with pytest.raises(ValueError):
+        s = expr(Const1D((2, 2), n_models=2), Const1D(3, n_models=1))
+
+
 @pytest.mark.parametrize(('expr', 'result'),
                          [(lambda x, y: x + y, 5.0),
                           (lambda x, y: x - y, -1.0),
@@ -154,6 +193,29 @@ def test_simple_two_model_class_compose_2d():
     assert_allclose(r3(0, 1), (0, -1), atol=1e-10)
 
 
+def test_n_submodels():
+    """
+    Test that CompoundModel.n_submodels properly returns the number
+    of components.
+    """
+    g2 = Gaussian1D() + Gaussian1D()
+    assert g2.n_submodels() == 2
+
+    g3 = g2 + Gaussian1D()
+    assert g3.n_submodels() == 3
+
+    g5 = g3 | g2
+    assert g5.n_submodels() == 5
+
+    g7 = g5 / g2
+    assert g7.n_submodels() == 7
+
+    # make sure it works as class method
+    p = Polynomial1D + Polynomial1D
+
+    assert p.n_submodels() == 2
+
+
 def test_expression_formatting():
     """
     Test that the expression strings from compound models are formatted
@@ -230,9 +292,7 @@ def test_indexing_on_class():
 
     M = Gaussian1D + p
     assert M[0] is Gaussian1D
-    assert M[1] is p
-    assert M['Gaussian1D'] is M[0]
-    assert M['p'] is M[1]
+    assert isinstance(M['p'], Polynomial1D)
 
     m = g + p
     assert isinstance(m[0], Gaussian1D)
@@ -376,7 +436,6 @@ def test_indexing_on_instance():
     const.amplitude = 137
     assert m.amplitude_1 == 137
 
-
     # Similar couple of tests, but now where the compound model was created
     # from model instances
     g = Gaussian1D(1, 2, 3, name='g')
@@ -524,7 +583,7 @@ def test_slicing_on_instances_2():
     with pytest.raises(IndexError):
         m['x']
     with pytest.raises(IndexError):
-        m['a' : 'r']
+        m['a': 'r']
 
     with pytest.raises(ModelDefinitionError):
         assert m[-4:4].submodel_names == ('b', 'c', 'd')
@@ -559,7 +618,7 @@ def test_slicing_on_instances_3():
     with pytest.raises(IndexError):
         m['x']
     with pytest.raises(IndexError):
-        m['a' : 'r']
+        m['a': 'r']
     assert m[-4:4].submodel_names == ('b', 'c', 'd')
     assert m[-4:-2].submodel_names == ('b', 'c')
 
@@ -838,3 +897,50 @@ def test_pickle_compound_fallback():
     gg = (Gaussian1D + Gaussian1D)()
     with pytest.raises(RuntimeError):
         pickle.dumps(gg)
+
+
+def test_update_parameters():
+    offx = Shift(1)
+    scl = Scale(2)
+    m = offx | scl
+    assert(m(1) == 4)
+
+    offx.offset = 42
+    assert(m(1) == 4)
+
+    m.factor_1 = 100
+    assert(m(1) == 200)
+    m2 = m | offx
+    assert(m2(1) == 242)
+
+
+def test_name():
+    offx = Shift(1)
+    scl = Scale(2)
+    m = offx | scl
+    scl.name = "scale"
+    assert m._submodel_names == ('None_0', 'None_1')
+    assert m.name is None
+    m.name = "M"
+    assert m.name == "M"
+    m1 = m.rename("M1")
+    assert m.name == "M"
+    assert m1.name == "M1"
+
+@pytest.mark.skipif("not HAS_SCIPY_14")
+def test_tabular_in_compound():
+    """
+    Issue #7411 - evaluate should not change the shape of the output.
+    """
+    t = Tabular1D(points=([1, 5, 7],), lookup_table=[12, 15, 19],
+                  bounds_error=False)
+    rot = Rotation2D(2)
+    p = Polynomial1D(1)
+    x = np.arange(12).reshape((3,4))
+    # Create a compound model which does ot execute Tabular.__call__,
+    # but model.evaluate and is followed by a Rotation2D which
+    # checks the exact shapes.
+    model = p & t | rot
+    x1, y1 = model(x, x)
+    assert x1.ndim == 2
+    assert y1.ndim == 2
