@@ -1,9 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
+
+import pytest
 import numpy as np
 from numpy.testing import assert_allclose
-from ...tests.helper import pytest, assert_quantity_allclose
+
+from ...tests.helper import assert_quantity_allclose
 from ..utils import (extract_array, add_array, subpixel_indices,
                      block_reduce, block_replicate,
                      overlap_slices, NoOverlapError, PartialOverlapError,
@@ -11,6 +14,7 @@ from ..utils import (extract_array, add_array, subpixel_indices,
 from ...wcs import WCS
 from ...coordinates import SkyCoord
 from ... import units as u
+from ...extern.six.moves import zip
 
 try:
     import skimage  # pylint: disable=W0611
@@ -31,7 +35,7 @@ test_slices = [slice(10.52, 3.12), slice(5.62, 12.97),
 
 subsampling = 5
 
-test_pos_bad = [(-1, -4), (-1, 0), (6, 2), (6, 6)]
+test_pos_bad = [(-1, -4), (-2, 0), (6, 2), (6, 6)]
 
 
 def test_slices_different_dim():
@@ -69,6 +73,37 @@ def test_slices_partial_overlap():
         assert 'Arrays overlap only partially.' in str(e.value)
 
 
+def test_slices_edges():
+    """
+    Test overlap_slices when extracting along edges.
+    """
+
+    slc_lg, slc_sm = overlap_slices((10, 10), (3, 3), (1, 1), mode='strict')
+    assert slc_lg[0].start == slc_lg[1].start == 0
+    assert slc_lg[0].stop == slc_lg[1].stop == 3
+    assert slc_sm[0].start == slc_sm[1].start == 0
+    assert slc_sm[0].stop == slc_sm[1].stop == 3
+
+    slc_lg, slc_sm = overlap_slices((10, 10), (3, 3), (8, 8), mode='strict')
+    assert slc_lg[0].start == slc_lg[1].start == 7
+    assert slc_lg[0].stop == slc_lg[1].stop == 10
+    assert slc_sm[0].start == slc_sm[1].start == 0
+    assert slc_sm[0].stop == slc_sm[1].stop == 3
+
+    # test (0, 0) shape
+    slc_lg, slc_sm = overlap_slices((10, 10), (0, 0), (0, 0))
+    assert slc_lg[0].start == slc_lg[0].stop == 0
+    assert slc_lg[1].start == slc_lg[1].stop == 0
+    assert slc_sm[0].start == slc_sm[0].stop == 0
+    assert slc_sm[1].start == slc_sm[1].stop == 0
+
+    slc_lg, slc_sm = overlap_slices((10, 10), (0, 0), (5, 5))
+    assert slc_lg[0].start == slc_lg[0].stop == 5
+    assert slc_lg[1].start == slc_lg[1].stop == 5
+    assert slc_sm[0].start == slc_sm[0].stop == 0
+    assert slc_sm[1].start == slc_sm[1].stop == 0
+
+
 def test_slices_overlap_wrong_mode():
     '''Call overlap_slices with non-existing mode.'''
     with pytest.raises(ValueError) as e:
@@ -90,7 +125,7 @@ def test_extract_array_1d_even():
     '''
     assert np.all(extract_array(np.arange(4), (2, ), (0, ), fill_value=-99) == np.array([-99, 0]))
     for i in [1, 2, 3]:
-        assert np.all(extract_array(np.arange(4), (2, ), (i, )) == np.array([i -1 , i]))
+        assert np.all(extract_array(np.arange(4), (2, ), (i, )) == np.array([i - 1, i]))
     assert np.all(extract_array(np.arange(4.), (2, ), (4, ), fill_value=np.inf) == np.array([3, np.inf]))
 
 
@@ -105,13 +140,13 @@ def test_extract_array_1d_odd():
     '''
     assert np.all(extract_array(np.arange(4), (3,), (-1, ), fill_value=-99) == np.array([-99, -99, 0]))
     assert np.all(extract_array(np.arange(4), (3,), (0, ), fill_value=-99) == np.array([-99, 0, 1]))
-    for i in [1,2]:
+    for i in [1, 2]:
         assert np.all(extract_array(np.arange(4), (3,), (i, )) == np.array([i-1, i, i+1]))
     assert np.all(extract_array(np.arange(4), (3,), (3, ), fill_value=-99) == np.array([2, 3, -99]))
     arrayin = np.arange(4.)
     extracted = extract_array(arrayin, (3,), (4, ))
     assert extracted[0] == 3
-    assert np.isnan(extracted[1]) # since I cannot use `==` to test for nan
+    assert np.isnan(extracted[1])  # since I cannot use `==` to test for nan
     assert extracted.dtype == arrayin.dtype
 
 
@@ -134,7 +169,7 @@ def test_extract_array_1d_trim():
     '''
     assert np.all(extract_array(np.arange(4), (2, ), (0, ), mode='trim') == np.array([0]))
     for i in [1, 2, 3]:
-        assert np.all(extract_array(np.arange(4), (2, ), (i, ), mode='trim') == np.array([i -1 , i]))
+        assert np.all(extract_array(np.arange(4), (2, ), (i, ), mode='trim') == np.array([i - 1, i]))
     assert np.all(extract_array(np.arange(4.), (2, ), (4, ), mode='trim') == np.array([3]))
 
 
@@ -330,10 +365,10 @@ class TestCutout2D(object):
         rho = np.pi / 3.
         scale = 0.05 / 3600.
         wcs.wcs.cd = [[scale*np.cos(rho), -scale*np.sin(rho)],
-                        [scale*np.sin(rho), scale*np.cos(rho)]]
+                      [scale*np.sin(rho), scale*np.cos(rho)]]
         wcs.wcs.ctype = ['RA---TAN', 'DEC--TAN']
-        wcs.wcs.crval = [self.position.ra.to(u.deg).value,
-                         self.position.dec.to(u.deg).value]
+        wcs.wcs.crval = [self.position.ra.to_value(u.deg),
+                         self.position.dec.to_value(u.deg)]
         wcs.wcs.crpix = [3, 3]
         self.wcs = wcs
 
@@ -387,7 +422,7 @@ class TestCutout2D(object):
 
     def test_size_angle_without_wcs(self):
         with pytest.raises(ValueError):
-            Cutout2D(self.data, (2, 2), (3, 3* u.arcsec))
+            Cutout2D(self.data, (2, 2), (3, 3 * u.arcsec))
 
     def test_cutout_trim_overlap(self):
         c = Cutout2D(self.data, (0, 0), (3, 3), mode='trim')
@@ -406,7 +441,7 @@ class TestCutout2D(object):
     def test_cutout_partial_overlap_fill_value(self):
         fill_value = -99
         c = Cutout2D(self.data, (0, 0), (3, 3), mode='partial',
-                   fill_value=fill_value)
+                     fill_value=fill_value)
         assert c.data.shape == (3, 3)
         assert c.data[1, 1] == 0
         assert c.data[0, 0] == fill_value
@@ -449,7 +484,7 @@ class TestCutout2D(object):
 
     def test_skycoord_partial(self):
         c = Cutout2D(self.data, self.position, (3, 3), wcs=self.wcs,
-                   mode='partial')
+                     mode='partial')
         skycoord_original = self.position.from_pixel(c.center_original[1],
                                                      c.center_original[0],
                                                      self.wcs)
@@ -457,3 +492,10 @@ class TestCutout2D(object):
                                                    c.center_cutout[0], c.wcs)
         assert_quantity_allclose(skycoord_original.ra, skycoord_cutout.ra)
         assert_quantity_allclose(skycoord_original.dec, skycoord_cutout.dec)
+
+    def test_naxis_update(self):
+        xsize = 2
+        ysize = 3
+        c = Cutout2D(self.data, self.position, (ysize, xsize), wcs=self.wcs)
+        assert c.wcs._naxis[0] == xsize
+        assert c.wcs._naxis[1] == ysize
