@@ -35,7 +35,7 @@ function.
 References
 ----------
 .. [1] http://adsabs.harvard.edu/abs/2012arXiv1207.5578S
-.. [2] http://astroML.org/ http://github.com/astroML/astroML/
+.. [2] http://astroML.org/ https://github.com//astroML/astroML/
 """
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
@@ -277,7 +277,7 @@ class FitnessFunc(object):
 
         Note that there was an error in this equation in the original Scargle
         paper (the "log" was missing). The following corrected form is taken
-        from http://arxiv.org/abs/1304.2818
+        from https://arxiv.org/abs/1304.2818
         """
         return 4 - np.log(73.53 * self.p0 * (N ** -0.478))
 
@@ -292,15 +292,14 @@ class FitnessFunc(object):
         If ``ncp_prior`` is not explicitly defined, compute it from ``gamma``
         or ``p0``.
         """
-        if self.ncp_prior is not None:
-            return self.ncp_prior
-        elif self.gamma is not None:
+
+        if self.gamma is not None:
             return -np.log(self.gamma)
         elif self.p0 is not None:
             return self.p0_prior(N)
         else:
-            raise ValueError("``ncp_prior`` is not defined, and cannot compute "
-                             "it as neither ``gamma`` nor ``p0`` is defined.")
+            raise ValueError("``ncp_prior`` cannot be computed as neither "
+                             "``gamma`` nor ``p0`` is defined.")
 
     def fit(self, t, x=None, sigma=None):
         """Fit the Bayesian Blocks model given the specified fitness function.
@@ -343,6 +342,9 @@ class FitnessFunc(object):
         # Compute ncp_prior if not defined
         if self.ncp_prior is None:
             ncp_prior = self.compute_ncp_prior(N)
+        else:
+            ncp_prior = self.ncp_prior
+
         # ----------------------------------------------------------------
         # Start with first data cell; add one cell at each iteration
         # ----------------------------------------------------------------
