@@ -4,13 +4,12 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import collections
+import operator
 
 import numpy as np
-from numpy import ma
 
 from ..extern import six
-from ..utils import deprecated
-from ..utils.compat import NUMPY_LT_1_8
+
 
 class Row(object):
     """A class to represent one row of a Table object.
@@ -23,9 +22,11 @@ class Row(object):
       ...               dtype=('int32', 'int32'))
       >>> row = table[1]
       >>> row
-      <Row 1 of table
-       values=(2, 4)
-       dtype=[('a', '<i4'), ('b', '<i4')]>
+      <Row index=1>
+        a     b
+      int32 int32
+      ----- -----
+          2     4
       >>> row['a']
       2
       >>> row[1]
@@ -34,13 +35,16 @@ class Row(object):
 
     def __init__(self, table, index):
         self._table = table
-        self._index = index
+        self._index = operator.index(index)
 
         n = len(table)
         if index < -n or index >= n:
             raise IndexError('index {0} out of range for table with length {1}'
                              .format(index, len(table)))
 
+        # Ensure that the row index is positive [#8422]
+        self._index = self._index % n
+
     def __getitem__(self, item):
         return self._table.columns[item][self._index]
 
@@ -89,17 +93,6 @@ class Row(object):
     def index(self):
         return self._index
 
-    @property
-    @deprecated('0.4', alternative=':attr:`Row.as_void`')
-    def data(self):
-        """
-        Returns a *read-only* copy of the row values in the form of np.void or
-        np.ma.mvoid objects.  This corresponds to the object types returned for
-        row indexing of a pure numpy structured array or masked array. This
-        method is slow and its use is deprecated.
-        """
-        return self.as_void()
-
     def as_void(self):
         """
         Returns a *read-only* copy of the row values in the form of np.void or
@@ -131,28 +124,7 @@ class Row(object):
 
             # Make np.void version of values, and then the final mvoid row
             row_vals = np.array([vals], dtype=self.dtype)[0]
-            try:
-                void_row = np.ma.mvoid(data=row_vals, mask=row_mask)
-            except ValueError as err:
-                # Another bug (or maybe same?) that is fixed in 1.8 prevents
-                # accessing a row in masked array if it has object-type members.
-                # >>> x = np.ma.empty(1, dtype=[('a', 'O')])
-                # >>> x['a'] = 1
-                # >>> x['a'].mask = True
-                # >>> x[0]
-                # ValueError: Setting void-array with object members using buffer. [numpy.ma.core]
-                #
-                # All we do here is re-raise with a more informative message
-                msg = six.text_type(err)
-                if ('Setting void-array with object members' in msg and
-                        NUMPY_LT_1_8):
-                    raise ValueError(
-                        'Cannot convert masked table row with Object type '
-                        'columns using as_void(), due to a bug in Numpy '
-                        '{0}.  Please upgrade to Numpy 1.8 or newer.'.format(
-                            np.__version__))
-                else:
-                    raise
+            void_row = np.ma.mvoid(data=row_vals, mask=row_mask)
         else:
             void_row = np.array([vals], dtype=self.dtype)[0]
         return void_row
@@ -173,9 +145,36 @@ class Row(object):
     def dtype(self):
         return self._table.dtype
 
+    def _base_repr_(self, html=False):
+        """
+        Display row as a single-line table but with appropriate header line.
+        """
+        index = self.index if (self.index >= 0) else self.index + len(self._table)
+        table = self._table[index:index + 1]
+        descr_vals = [self.__class__.__name__,
+                      'index={0}'.format(self.index)]
+        if table.masked:
+            descr_vals.append('masked=True')
+
+        return table._base_repr_(html, descr_vals, max_width=-1,
+                                 tableid='table{0}'.format(id(self._table)))
+
+    def _repr_html_(self):
+        return self._base_repr_(html=True)
+
     def __repr__(self):
-        return "<{3} {0} of table\n values={1!r}\n dtype={2}>".format(
-            self.index, self.as_void(), self.dtype, self.__class__.__name__)
+        return self._base_repr_(html=False)
+
+    def __unicode__(self):
+        index = self.index if (self.index >= 0) else self.index + len(self._table)
+        return '\n'.join(self.table[index:index + 1].pformat(max_width=-1))
+    if not six.PY2:
+        __str__ = __unicode__
+
+    def __bytes__(self):
+        return six.text_type(self).encode('utf-8')
+    if six.PY2:
+        __str__ = __bytes__
 
 
 collections.Sequence.register(Row)
