@@ -4,12 +4,21 @@
 
 from __future__ import print_function  # For print debugging with python 2 or 3
 
-from collections import OrderedDict
+from collections import OrderedDict, Mapping
+from ...extern import six
 
+import pytest
 import numpy as np
 
-from ...tests.helper import pytest
-from ...table import Column, TableColumns
+from ...table import Column, TableColumns, Table
+
+# Unfortunatly the python2 UserDict.UserDict is not a Mapping so it is not
+# possible to use "from six.moves import UserDict". Instead we have to use
+# IterableUserDict (which is a Mapping) here.
+if six.PY2:
+    from UserDict import IterableUserDict as UserDict
+else:
+    from collections import UserDict
 
 
 class TestTableColumnsInit():
@@ -19,19 +28,19 @@ class TestTableColumnsInit():
         x1 = np.arange(10.)
         x2 = np.arange(5.)
         x3 = np.arange(7.)
-        col_list = [('x1',x1), ('x2',x2), ('x3',x3)]
+        col_list = [('x1', x1), ('x2', x2), ('x3', x3)]
         tc_list = TableColumns(col_list)
         for col in col_list:
             assert col[0] in tc_list
             assert tc_list[col[0]] is col[1]
 
-        col_tuple = (('x1',x1), ('x2',x2), ('x3',x3))
+        col_tuple = (('x1', x1), ('x2', x2), ('x3', x3))
         tc_tuple = TableColumns(col_tuple)
         for col in col_tuple:
             assert col[0] in tc_tuple
             assert tc_tuple[col[0]] is col[1]
 
-        col_dict = dict([('x1',x1), ('x2',x2), ('x3',x3)])
+        col_dict = dict([('x1', x1), ('x2', x2), ('x3', x3)])
         tc_dict = TableColumns(col_dict)
         for col in tc_dict.keys():
             assert col in tc_dict
@@ -44,7 +53,7 @@ class TestTableColumnsInit():
             assert tc[col.name] is col
 
 
-#pytest.mark.usefixtures('table_type')
+# pytest.mark.usefixtures('table_type')
 class BaseInitFrom():
     def _setup(self, table_type):
         pass
@@ -80,6 +89,7 @@ class BaseInitFrom():
         with pytest.raises(ValueError):
             table_type(self.data, names=('a',), dtype=('i4'))
 
+
 @pytest.mark.usefixtures('table_type')
 class BaseInitFromListLike(BaseInitFrom):
 
@@ -227,6 +237,7 @@ class TestInitFromColsList(BaseInitFromListLike):
         t['x'][0] = 100
         assert self.data[0][0] == 100
 
+
 @pytest.mark.usefixtures('table_type')
 class TestInitFromNdarrayStruct(BaseInitFromDictLike):
 
@@ -276,6 +287,17 @@ class TestInitFromDict(BaseInitFromDictLike):
                           ('c', np.array([3, 5], dtype='i8'))])
 
 
+@pytest.mark.usefixtures('table_type')
+class TestInitFromMapping(BaseInitFromDictLike):
+
+    def _setup(self, table_type):
+        self.data = UserDict([('a', Column([1, 3], name='x')),
+                              ('b', [2, 4]),
+                              ('c', np.array([3, 5], dtype='i8'))])
+        assert isinstance(self.data, Mapping)
+        assert not isinstance(self.data, dict)
+
+
 @pytest.mark.usefixtures('table_type')
 class TestInitFromOrderedDict(BaseInitFromDictLike):
 
@@ -316,6 +338,7 @@ class TestInitFromRow(BaseInitFromDictLike):
         assert np.all(self.data['x'] == np.array([1, 3]))
         assert self.data.meta['comments'][1] == 'comment2'
 
+
 @pytest.mark.usefixtures('table_type')
 class TestInitFromTable(BaseInitFromDictLike):
 
@@ -443,6 +466,7 @@ class TestInitFromRows():
             table_type(data=[[1]], rows=[[1]])
         assert "Cannot supply both `data` and `rows` values" in str(err)
 
+
 @pytest.mark.usefixtures('table_type')
 def test_init_and_ref_from_multidim_ndarray(table_type):
     """
@@ -466,3 +490,10 @@ def test_init_and_ref_from_multidim_ndarray(table_type):
         else:
             assert nd[str('a')][0] == -200
             assert nd[str('b')][1][1] == -100
+
+
+def test_init_from_rows_as_generator():
+    rows = ((1 + ii, 2 + ii) for ii in range(2))
+    t = Table(rows=rows)
+    assert np.all(t['col0'] == [1, 2])
+    assert np.all(t['col1'] == [2, 3])
