@@ -4,7 +4,8 @@ import itertools
 import numpy as np
 import operator
 
-from ...tests.helper import pytest
+import pytest
+
 from .. import (Time, TimeDelta, OperandTypeError, ScaleValueError,
                 TIME_SCALES, TIME_DELTA_SCALES)
 from ... import units as u
@@ -140,27 +141,30 @@ class TestTimeDelta():
         assert allclose_jd(out.jd, [0.0, -100.0])
         assert not out.isscalar
 
-    def test_copy_timedelta(self):
+    @pytest.mark.parametrize('values', [(2455197.5, 2455198.5),
+                                        ([2455197.5], [2455198.5])])
+    def test_copy_timedelta(self, values):
         """Test copying the values of a TimeDelta object by passing it into the
         Time initializer.
         """
-        t = Time(2455197.5, format='jd', scale='utc')
-        t2 = Time(2455198.5, format='jd', scale='utc')
+        val1, val2 = values
+        t = Time(val1, format='jd', scale='utc')
+        t2 = Time(val2, format='jd', scale='utc')
         dt = t2 - t
 
         dt2 = TimeDelta(dt, copy=False)
-        assert dt.jd == dt2.jd
+        assert np.all(dt.jd == dt2.jd)
         assert dt._time.jd1 is dt2._time.jd1
         assert dt._time.jd2 is dt2._time.jd2
 
         dt2 = TimeDelta(dt, copy=True)
-        assert dt.jd == dt2.jd
+        assert np.all(dt.jd == dt2.jd)
         assert dt._time.jd1 is not dt2._time.jd1
         assert dt._time.jd2 is not dt2._time.jd2
 
         # Include initializers
         dt2 = TimeDelta(dt, format='sec')
-        assert allclose_sec(dt2.val, 86400.0)
+        assert allclose_sec(dt2.value, 86400.0)
 
     def test_neg_abs(self):
         for dt in (self.dt, self.dt_array):
@@ -230,10 +234,22 @@ class TestTimeDelta():
             assert not hasattr(t3, '_delta_tdb_tt')
             assert not hasattr(t3, '_delta_ut1_utc')
 
+    def test_set_format(self):
+        """
+        Test basics of setting format attribute.
+        """
+        dt = TimeDelta(86400.0, format='sec')
+        assert dt.value == 86400.0
+        assert dt.format == 'sec'
+
+        dt.format = 'jd'
+        assert dt.value == 1.0
+        assert dt.format == 'jd'
+
 
 class TestTimeDeltaScales():
     """Test scale conversion for Time Delta.
-    Go through @taldcroft's list of expected behaviour from #1932"""
+    Go through @taldcroft's list of expected behavior from #1932"""
 
     def setup(self):
         # pick a date that includes a leap second for better testing
@@ -244,7 +260,7 @@ class TestTimeDeltaScales():
         self.dt = dict((scale, self.t[scale]-self.t[scale][0])
                        for scale in TIME_SCALES)
 
-    def test_delta_scales_defintion(self):
+    def test_delta_scales_definition(self):
         for scale in list(TIME_DELTA_SCALES) + [None]:
             TimeDelta([0., 1., 10.], format='sec', scale=scale)
 
