@@ -9,12 +9,12 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 from ..extern import six
 from ..extern.six.moves import zip
-if six.PY2:
-    import cmath
 
 import inspect
+import operator
 import textwrap
 import warnings
+
 import numpy as np
 
 from ..utils.decorators import lazyproperty, deprecated
@@ -24,12 +24,14 @@ from .utils import (is_effectively_unity, sanitize_scale, validate_power,
                     resolve_fractions)
 from . import format as unit_format
 
-# TODO: Support functional units, e.g. log(x), ln(x)
+if six.PY2:
+    import cmath
+
 
 __all__ = [
-    'UnitsError', 'UnitsWarning', 'UnitConversionError', 'UnitBase',
-    'NamedUnit', 'IrreducibleUnit', 'Unit', 'def_unit', 'CompositeUnit',
-    'PrefixUnit', 'UnrecognizedUnit', 'get_current_unit_registry',
+    'UnitsError', 'UnitsWarning', 'UnitConversionError', 'UnitTypeError',
+    'UnitBase', 'NamedUnit', 'IrreducibleUnit', 'Unit', 'CompositeUnit',
+    'PrefixUnit', 'UnrecognizedUnit', 'def_unit', 'get_current_unit_registry',
     'set_enabled_units', 'add_enabled_units',
     'set_enabled_equivalencies', 'add_enabled_equivalencies',
     'dimensionless_unscaled', 'one']
@@ -95,8 +97,8 @@ def _normalize_equivalencies(equivalencies):
         else:
             raise ValueError(
                 "Invalid equivalence entry {0}: {1!r}".format(i, equiv))
-        if not (isinstance(funit, UnitBase) and
-                (isinstance(tunit, UnitBase) or tunit is None) and
+        if not (funit is Unit(funit) and
+                (tunit is None or tunit is Unit(tunit)) and
                 six.callable(a) and
                 six.callable(b)):
             raise ValueError(
@@ -110,15 +112,28 @@ class _UnitRegistry(object):
     """
     Manages a registry of the enabled units.
     """
-    def __init__(self, init=[], equivalencies=[]):
-        if isinstance(init, _UnitRegistry):
-            equivalencies = init.equivalencies
-            init = init.all_units
 
-        self._reset_units()
-        self._reset_equivalencies()
-        self.add_enabled_units(init)
-        self.add_enabled_equivalencies(equivalencies)
+    def __init__(self, init=[], equivalencies=[]):
+
+        if isinstance(init, _UnitRegistry):
+            # If passed another registry we don't need to rebuild everything.
+            # but because these are mutable types we don't want to create
+            # conflicts so everything needs to be copied.
+            self._equivalencies = init._equivalencies.copy()
+            self._all_units = init._all_units.copy()
+            self._registry = init._registry.copy()
+            self._non_prefix_units = init._non_prefix_units.copy()
+            # The physical type is a dictionary containing sets as values.
+            # All of these must be copied otherwise we could alter the old
+            # registry.
+            self._by_physical_type = {k: v.copy() for k, v in
+                                      six.iteritems(init._by_physical_type)}
+
+        else:
+            self._reset_units()
+            self._reset_equivalencies()
+            self.add_enabled_units(init)
+            self.add_enabled_equivalencies(equivalencies)
 
     def _reset_units(self):
         self._all_units = set()
@@ -185,7 +200,7 @@ class _UnitRegistry(object):
                 if (st in self._registry and unit != self._registry[st]):
                     raise ValueError(
                         "Object with name {0!r} already exists in namespace. "
-                        "Filter the set of units to avoid name classes before "
+                        "Filter the set of units to avoid name clashes before "
                         "enabling them.".format(st))
 
             for st in unit._names:
@@ -207,9 +222,7 @@ class _UnitRegistry(object):
         ----------
         unit : UnitBase instance
         """
-        return self._by_physical_type.get(
-            unit._get_physical_type_id(),
-            set())
+        return self._by_physical_type.get(unit._get_physical_type_id(), set())
 
     @property
     def equivalencies(self):
@@ -258,9 +271,7 @@ class _UnitRegistry(object):
 class _UnitContext(object):
     def __init__(self, init=[], equivalencies=[]):
         _unit_registries.append(
-            _UnitRegistry(
-                init=init,
-                equivalencies=equivalencies))
+            _UnitRegistry(init=init, equivalencies=equivalencies))
 
     def __enter__(self):
         pass
@@ -311,11 +322,13 @@ def set_enabled_units(units):
       AU           | 1.49598e+11 m   | au, astronomical_unit ,
       Angstrom     | 1e-10 m         | AA, angstrom          ,
       cm           | 0.01 m          | centimeter            ,
+      earthRad     | 6.3781e+06 m    | R_earth, Rearth       ,
+      jupiterRad   | 7.1492e+07 m    | R_jup, Rjup, R_jupiter, Rjupiter ,
       lyr          | 9.46073e+15 m   | lightyear             ,
       m            | irreducible     | meter                 ,
       micron       | 1e-06 m         |                       ,
       pc           | 3.08568e+16 m   | parsec                ,
-      solRad       | 6.95508e+08 m   | R_sun, Rsun           ,
+      solRad       | 6.957e+08 m     | R_sun, Rsun           ,
     ]
     """
     # get a context with a new registry, using equivalencies of the current one
@@ -358,15 +371,19 @@ def add_enabled_units(units):
       AU           | 1.49598e+11 m   | au, astronomical_unit ,
       Angstrom     | 1e-10 m         | AA, angstrom          ,
       cm           | 0.01 m          | centimeter            ,
+      earthRad     | 6.3781e+06 m    | R_earth, Rearth       ,
       ft           | 0.3048 m        | foot                  ,
+      fur          | 201.168 m       | furlong               ,
       inch         | 0.0254 m        |                       ,
+      jupiterRad   | 7.1492e+07 m    | R_jup, Rjup, R_jupiter, Rjupiter ,
       lyr          | 9.46073e+15 m   | lightyear             ,
       m            | irreducible     | meter                 ,
       mi           | 1609.34 m       | mile                  ,
       micron       | 1e-06 m         |                       ,
+      mil          | 2.54e-05 m      | thou                  ,
       nmi          | 1852 m          | nauticalmile, NM      ,
       pc           | 3.08568e+16 m   | parsec                ,
-      solRad       | 6.95508e+08 m   | R_sun, Rsun           ,
+      solRad       | 6.957e+08 m     | R_sun, Rsun           ,
       yd           | 0.9144 m        | yard                  ,
     ]
     """
@@ -401,11 +418,13 @@ def set_enabled_equivalencies(equivalencies):
         >>> from astropy import units as u
         >>> with u.set_enabled_equivalencies(u.dimensionless_angles()):
         ...     phase = 0.5 * u.cycle
-        ...     np.exp(1j*phase)  # doctest: +FLOAT_CMP
-        <Quantity (-1+1.2246063538223773e-16j)>
+        ...     np.exp(1j*phase)  # doctest: +SKIP
+        <Quantity  -1. +1.22464680e-16j>
     """
+    # doctest skipped as the complex number formatting changed in numpy 1.14.
+    #
     # get a context with a new registry, using all units of the current one
-    context = _UnitContext(get_current_unit_registry().all_units)
+    context = _UnitContext(get_current_unit_registry())
     # in this new current registry, enable the equivalencies requested
     get_current_unit_registry().set_enabled_equivalencies(equivalencies)
     return context
@@ -456,14 +475,18 @@ class UnitConversionError(UnitsError, ValueError):
     """
 
 
-# Maintain error in old location for backward compatibility
-from .format import fits as _fits
-_fits.UnitScaleError = UnitScaleError
+class UnitTypeError(UnitsError, TypeError):
+    """
+    Used specifically for errors in setting to units not allowed by a class.
+
+    E.g., would be raised if the unit of an `~astropy.coordinates.Angle`
+    instances were set to a non-angular unit.
+    """
 
 
 class UnitsWarning(AstropyWarning):
     """
-    The base class for unit-specific exceptions.
+    The base class for unit-specific warnings.
     """
 
 
@@ -496,22 +519,22 @@ class UnitBase(object):
         -------
         Latex string
         """
-        return unit_format.Latex().to_string(self)
+        return unit_format.Latex.to_string(self)
 
     def __bytes__(self):
         """Return string representation for unit"""
-        return unit_format.Generic().to_string(self).encode('ascii')
+        return unit_format.Generic.to_string(self).encode('unicode_escape')
     if six.PY2:
         __str__ = __bytes__
 
     def __unicode__(self):
         """Return string representation for unit"""
-        return unit_format.Generic().to_string(self)
-    if six.PY3:
+        return unit_format.Generic.to_string(self)
+    if not six.PY2:
         __str__ = __unicode__
 
     def __repr__(self):
-        string = unit_format.Generic().to_string(self)
+        string = unit_format.Generic.to_string(self)
         if six.PY2:
             string = string.encode('unicode_escape')
 
@@ -539,7 +562,6 @@ class UnitBase(object):
         raise AttributeError(
             "Can not get names from unnamed units. "
             "Perhaps you meant to_string()?")
-        return self._names
 
     @property
     def name(self):
@@ -590,6 +612,7 @@ class UnitBase(object):
             The name of a format or a formatter object.  If not
             provided, defaults to the generic format.
         """
+
         f = unit_format.get_format(format)
         return f.to_string(self)
 
@@ -639,24 +662,30 @@ class UnitBase(object):
                 return self
             return CompositeUnit(1, [self, m], [1, -1], _error_check=False)
 
-        # Cannot handle this as Unit, re-try as Quantity
-        from .quantity import Quantity
-        return Quantity(1, self) / m
+        try:
+            # Cannot handle this as Unit, re-try as Quantity
+            from .quantity import Quantity
+            return Quantity(1, self) / m
+        except TypeError:
+            return NotImplemented
 
     def __rdiv__(self, m):
         if isinstance(m, (bytes, six.text_type)):
             return Unit(m) / self
 
-        # Cannot handle this as Unit.  Here, m cannot be a Quantity,
-        # so we make it into one, fasttracking when it does not have a unit,
-        # for the common case of <array> / <unit>.
-        from .quantity import Quantity
-        if hasattr(m, 'unit'):
-            result = Quantity(m)
-            result /= self
-            return result
-        else:
-            return Quantity(m, self**(-1))
+        try:
+            # Cannot handle this as Unit.  Here, m cannot be a Quantity,
+            # so we make it into one, fasttracking when it does not have a
+            # unit, for the common case of <array> / <unit>.
+            from .quantity import Quantity
+            if hasattr(m, 'unit'):
+                result = Quantity(m)
+                result /= self
+                return result
+            else:
+                return Quantity(m, self**(-1))
+        except TypeError:
+            return NotImplemented
 
     __truediv__ = __div__
 
@@ -674,8 +703,11 @@ class UnitBase(object):
             return CompositeUnit(1, [self, m], [1, 1], _error_check=False)
 
         # Cannot handle this as Unit, re-try as Quantity.
-        from .quantity import Quantity
-        return Quantity(1, self) * m
+        try:
+            from .quantity import Quantity
+            return Quantity(1, self) * m
+        except TypeError:
+            return NotImplemented
 
     def __rmul__(self, m):
         if isinstance(m, (bytes, six.text_type)):
@@ -684,13 +716,16 @@ class UnitBase(object):
         # Cannot handle this as Unit.  Here, m cannot be a Quantity,
         # so we make it into one, fasttracking when it does not have a unit
         # for the common case of <array> * <unit>.
-        from .quantity import Quantity
-        if hasattr(m, 'unit'):
-            result = Quantity(m)
-            result *= self
-            return result
-        else:
-            return Quantity(m, self)
+        try:
+            from .quantity import Quantity
+            if hasattr(m, 'unit'):
+                result = Quantity(m)
+                result *= self
+                return result
+            else:
+                return Quantity(m, self)
+        except TypeError:
+            return NotImplemented
 
     def __hash__(self):
         # This must match the hash used in CompositeUnit for a unit
@@ -704,7 +739,13 @@ class UnitBase(object):
         try:
             other = Unit(other, parse_strict='silent')
         except (ValueError, UnitsError, TypeError):
-            return False
+            return NotImplemented
+
+        # Other is Unit-like, but the test below requires it is a UnitBase
+        # instance; if it is not, give up (so that other can try).
+        if not isinstance(other, UnitBase):
+            return NotImplemented
+
         try:
             return is_effectively_unity(self._to(other))
         except UnitsError:
@@ -756,8 +797,8 @@ class UnitBase(object):
         if isinstance(other, tuple):
             return any(self.is_equivalent(u, equivalencies=equivalencies)
                        for u in other)
-        else:
-            other = Unit(other, parse_strict='silent')
+
+        other = Unit(other, parse_strict='silent')
 
         return self._is_equivalent(other, equivalencies)
 
@@ -778,12 +819,12 @@ class UnitBase(object):
             other = other.decompose()
             for a, b, forward, backward in equivalencies:
                 if b is None:
-                    # after canceling, is what's left convertable
+                    # after canceling, is what's left convertible
                     # to dimensionless (according to the equivalency)?
                     try:
                         (other/unit).decompose([a])
                         return True
-                    except:
+                    except Exception:
                         pass
                 else:
                     if(a._is_equivalent(unit) and b._is_equivalent(other) or
@@ -802,16 +843,11 @@ class UnitBase(object):
                 return func(_condition_arg(v) / scale1) * scale2
             return convert
 
-        orig_unit = unit
-        orig_other = other
-
-        unit = self.decompose()
-        other = other.decompose()
-
         for funit, tunit, a, b in equivalencies:
             if tunit is None:
                 try:
-                    ratio_in_funit = (other/unit).decompose([funit])
+                    ratio_in_funit = (other.decompose() /
+                                      unit.decompose()).decompose([funit])
                     return make_converter(ratio_in_funit.scale, a, 1.)
                 except UnitsError:
                     pass
@@ -839,8 +875,8 @@ class UnitBase(object):
                 unit_str = "'{0}'".format(unit_str)
             return unit_str
 
-        unit_str = get_err_str(orig_unit)
-        other_str = get_err_str(orig_other)
+        unit_str = get_err_str(unit)
+        other_str = get_err_str(other)
 
         raise UnitConversionError(
             "{0} and {1} are not convertible".format(
@@ -849,44 +885,32 @@ class UnitBase(object):
     def _get_converter(self, other, equivalencies=[]):
         other = Unit(other)
 
+        # First see if it is just a scaling.
         try:
             scale = self._to(other)
         except UnitsError:
+            pass
+        else:
+            return lambda val: scale * _condition_arg(val)
+
+        # if that doesn't work, maybe we can do it with equivalencies?
+        try:
             return self._apply_equivalencies(
                 self, other, self._normalize_equivalencies(equivalencies))
-        return lambda val: scale * _condition_arg(val)
+        except UnitsError as exc:
+            # Last hope: maybe other knows how to do it?
+            # We assume the equivalencies have the unit itself as first item.
+            # TODO: maybe better for other to have a `_back_converter` method?
+            if hasattr(other, 'equivalencies'):
+                for funit, tunit, a, b in other.equivalencies:
+                    if other is funit:
+                        try:
+                            return lambda v: b(self._get_converter(
+                                tunit, equivalencies=equivalencies)(v))
+                        except Exception:
+                            pass
 
-    @deprecated('1.0')
-    def get_converter(self, other, equivalencies=[]):
-        """
-        Return the conversion function to convert values from ``self``
-        to the specified unit.
-
-        Parameters
-        ----------
-        other : unit object or string
-            The unit to convert to.
-
-        equivalencies : list of equivalence pairs, optional
-            A list of equivalence pairs to try if the units are not
-            directly convertible.  See :ref:`unit_equivalencies`.
-            This list is in addition to possible global defaults set by, e.g.,
-            `set_enabled_equivalencies`.
-            Use `None` to turn off all equivalencies.
-
-        Returns
-        -------
-        func : callable
-            A callable that normally expects a single argument that is
-            a scalar value or an array of values (or anything that may
-            be converted to an array).
-
-        Raises
-        ------
-        UnitsError
-            If units are inconsistent
-        """
-        return self._get_converter(other, equivalencies=equivalencies)
+            raise exc
 
     def _to(self, other):
         """
@@ -903,17 +927,20 @@ class UnitBase(object):
         if self is other:
             return 1.0
 
-        self_decomposed = self.decompose()
-        other_decomposed = other.decompose()
+        # Don't presume decomposition is possible; e.g.,
+        # conversion to function units is through equivalencies.
+        if isinstance(other, UnitBase):
+            self_decomposed = self.decompose()
+            other_decomposed = other.decompose()
 
-        # Check quickly whether equivalent.  This is faster than
-        # `is_equivalent`, because it doesn't generate the entire
-        # physical type list of both units.  In other words it "fails
-        # fast".
-        if(self_decomposed.powers == other_decomposed.powers and
-           all(self_base is other_base for (self_base, other_base)
-               in zip(self_decomposed.bases, other_decomposed.bases))):
-            return self_decomposed.scale / other_decomposed.scale
+            # Check quickly whether equivalent.  This is faster than
+            # `is_equivalent`, because it doesn't generate the entire
+            # physical type list of both units.  In other words it "fails
+            # fast".
+            if(self_decomposed.powers == other_decomposed.powers and
+               all(self_base is other_base for (self_base, other_base)
+                   in zip(self_decomposed.bases, other_decomposed.bases))):
+                return self_decomposed.scale / other_decomposed.scale
 
         raise UnitConversionError(
             "'{0!r}' is not a scaled version of '{1!r}'".format(self, other))
@@ -927,7 +954,7 @@ class UnitBase(object):
         other : unit object or string
             The unit to convert to.
 
-        value : scalar int or float, or sequence convertable to array, optional
+        value : scalar int or float, or sequence convertible to array, optional
             Value(s) in the current unit to be converted to the
             specified unit.  If not provided, defaults to 1.0
 
@@ -1057,7 +1084,7 @@ class UnitBase(object):
                 cached_results[key] = results
                 return results
 
-        partial_results.sort(key=lambda x: x[0])
+        partial_results.sort(key=operator.itemgetter(0))
 
         # ...we have to recurse and try to further compose
         results = []
@@ -1075,7 +1102,7 @@ class UnitBase(object):
                     (len(subcomposed.bases), subcomposed, tunit))
 
         if len(results):
-            results.sort(key=lambda x: x[0])
+            results.sort(key=operator.itemgetter(0))
 
             min_length = results[0][0]
             subresults = set()
@@ -1341,6 +1368,7 @@ class UnitBase(object):
         A class to handle pretty-printing the result of
         `find_equivalent_units`.
         """
+
         def __repr__(self):
             if len(self) == 0:
                 return "[]"
@@ -1377,7 +1405,7 @@ class UnitBase(object):
         equivalencies : list of equivalence pairs, optional
             A list of equivalence pairs to also list.  See
             :ref:`unit_equivalencies`.
-            Any list given, including an empty one, supercedes global defaults
+            Any list given, including an empty one, supersedes global defaults
             that may be in effect (as set by `set_enabled_equivalencies`)
 
         units : set of units to search in, optional
@@ -1449,15 +1477,8 @@ class NamedUnit(UnitBase):
     ValueError
         If any of the given unit names are not valid Python tokens.
     """
-    def __init__(self, st, register=None, doc=None, format=None,
-                 namespace=None):
-        if register is not None:
-            warnings.warn(
-                "The registry kwarg was removed in astropy 0.3. "
-                "Use the namespace kwarg to inject the unit into "
-                "a namespace and add_enabled_units() to enable it "
-                "in the global unit registry.",
-                DeprecationWarning)
+
+    def __init__(self, st, doc=None, format=None, namespace=None):
 
         UnitBase.__init__(self)
 
@@ -1468,7 +1489,7 @@ class NamedUnit(UnitBase):
         elif isinstance(st, tuple):
             if not len(st) == 2:
                 raise ValueError("st must be string, list or 2-tuple")
-            self._names = st[0] + st[1]
+            self._names = st[0] + [n for n in st[1] if n not in st[0]]
             if not len(self._names):
                 raise ValueError("must provide at least one name")
             self._short_names = st[0][:]
@@ -1562,16 +1583,6 @@ class NamedUnit(UnitBase):
         """
         return self._long_names
 
-    def register(self, add_to_namespace=False):
-        raise NotImplementedError(
-            "The register method has been removed in astropy 0.3. "
-            "Use add_enabled_units/set_enabled_units instead.")
-
-    def deregister(self, remove_from_namespace=False):
-        raise NotImplementedError(
-            "The deregister method has been removed in astropy 0.3. "
-            "Use add_enabled_units/set_enabled_units instead.")
-
     def _inject(self, namespace=None):
         """
         Injects the unit, and all of its aliases, in the given
@@ -1600,12 +1611,18 @@ def _recreate_irreducible_unit(cls, names, registered):
     """
     registry = get_current_unit_registry().registry
     if names[0] in registry:
+        # If in local registry return that object.
         return registry[names[0]]
     else:
+        # otherwise, recreate the unit.
         unit = cls(names)
         if registered:
+            # If not in local registry but registered in origin registry,
+            # enable unit in local registry.
             get_current_unit_registry().add_enabled_units([unit])
 
+        return unit
+
 
 class IrreducibleUnit(NamedUnit):
     """
@@ -1615,6 +1632,7 @@ class IrreducibleUnit(NamedUnit):
     Examples are meters, seconds, kilograms, amperes, etc.  There is
     only once instance of such a unit per type.
     """
+
     def __reduce__(self):
         # When IrreducibleUnit objects are passed to other processes
         # over multiprocessing, they need to be recreated to be the
@@ -1627,8 +1645,16 @@ class IrreducibleUnit(NamedUnit):
                 (self.__class__, list(self.names), self.name in registry),
                 self.__dict__)
 
+    @property
+    def represents(self):
+        """The unit that this named unit represents.
+
+        For an irreducible unit, that is always itself.
+        """
+        return self
+
     def decompose(self, bases=set()):
-        if len(bases) and not self in bases:
+        if len(bases) and self not in bases:
             for base in bases:
                 try:
                     scale = self._to(base)
@@ -1651,7 +1677,7 @@ class IrreducibleUnit(NamedUnit):
 class UnrecognizedUnit(IrreducibleUnit):
     """
     A unit that did not parse correctly.  This allows for
-    roundtripping it as a string, but no unit operations actually work
+    round-tripping it as a string, but no unit operations actually work
     on it.
 
     Parameters
@@ -1674,10 +1700,10 @@ class UnrecognizedUnit(IrreducibleUnit):
 
     def __unicode__(self):
         return self.name
-    if six.PY3:
+    if not six.PY2:
         __str__ = __unicode__
 
-    def to_string(self, format=unit_format.Generic):
+    def to_string(self, format=None):
         return self.name
 
     def _unrecognized_operator(self, *args, **kwargs):
@@ -1690,8 +1716,12 @@ class UnrecognizedUnit(IrreducibleUnit):
         _unrecognized_operator
 
     def __eq__(self, other):
-        other = Unit(other, parse_strict='silent')
-        return isinstance(other, UnrecognizedUnit) and self.name == other.name
+        try:
+            other = Unit(other, parse_strict='silent')
+        except (ValueError, UnitsError, TypeError):
+            return NotImplemented
+
+        return isinstance(other, type(self)) and self.name == other.name
 
     def __ne__(self, other):
         return not (self == other)
@@ -1725,7 +1755,7 @@ class _UnitMetaClass(InheritDocstrings):
                  doc=None, parse_strict='raise'):
 
         # Short-circuit if we're already a unit
-        if isinstance(s, UnitBase):
+        if hasattr(s, '_get_physical_type_id'):
             return s
 
         # turn possible Quantity input for s or represents into a Unit
@@ -1769,7 +1799,7 @@ class _UnitMetaClass(InheritDocstrings):
                 format = unit_format.Generic
 
             f = unit_format.get_format(format)
-            if six.PY3 and isinstance(s, bytes):
+            if not six.PY2 and isinstance(s, bytes):
                 s = s.decode('ascii')
 
             try:
@@ -1778,9 +1808,9 @@ class _UnitMetaClass(InheritDocstrings):
                 if parse_strict == 'silent':
                     pass
                 else:
-                    # Deliberately not isinstance here. Subclasses
+                    # Deliberately not issubclass here. Subclasses
                     # should use their name.
-                    if type(f) is not unit_format.Generic:
+                    if f is not unit_format.Generic:
                         format_clause = f.name + ' '
                     else:
                         format_clause = ''
@@ -1890,23 +1920,20 @@ class Unit(NamedUnit):
         If any of the given unit names are not valid Python tokens.
     """
 
-    def __init__(self, st, represents=None, register=None, doc=None,
+    def __init__(self, st, represents=None, doc=None,
                  format=None, namespace=None):
 
-        if register is not None:
-            warnings.warn(
-                "The registry kwarg was removed in astropy 0.3. "
-                "Use the namespace kwarg to inject the unit into "
-                "a namespace and add_enabled_units() to enable it "
-                "in the global unit registry.",
-                DeprecationWarning)
-
         represents = Unit(represents)
         self._represents = represents
 
         NamedUnit.__init__(self, st, namespace=namespace, doc=doc,
                            format=format)
 
+    @property
+    def represents(self):
+        """The unit that this named unit represents."""
+        return self._represents
+
     def decompose(self, bases=set()):
         return self._represents.decompose(bases=bases)
 
@@ -1916,6 +1943,19 @@ class Unit(NamedUnit):
     def __hash__(self):
         return hash(self.name) + hash(self._represents)
 
+    @classmethod
+    def _from_physical_type_id(cls, physical_type_id):
+        # get string bases and powers from the ID tuple
+        bases = [cls(base) for base, _ in physical_type_id]
+        powers = [power for _, power in physical_type_id]
+
+        if len(physical_type_id) == 1 and powers[0] == 1:
+            unit = bases[0]
+        else:
+            unit = CompositeUnit(1, bases, powers)
+
+        return unit
+
 
 class PrefixUnit(Unit):
     """
@@ -1948,6 +1988,7 @@ class CompositeUnit(UnitBase):
         A sequence of powers (in parallel with ``bases``) for each
         of the base units.
     """
+
     def __init__(self, scale, bases, powers, decompose=False,
                  decompose_bases=set(), _error_check=True):
         # There are many cases internal to astropy.units where we
@@ -1962,7 +2003,7 @@ class CompositeUnit(UnitBase):
                 if not isinstance(base, UnitBase):
                     raise TypeError(
                         "bases must be sequence of UnitBase instances")
-            powers = [validate_power(p, support_tuples=True) for p in powers]
+            powers = [validate_power(p) for p in powers]
 
         self._scale = scale
         self._bases = bases
@@ -2061,8 +2102,7 @@ class CompositeUnit(UnitBase):
         new_parts.sort(key=lambda x: (-x[1], getattr(x[0], 'name', '')))
 
         self._bases = [x[0] for x in new_parts]
-        self._powers = [validate_power(x[1], support_tuples=True)
-                        for x in new_parts]
+        self._powers = [validate_power(x[1]) for x in new_parts]
         self._scale = sanitize_scale(scale)
 
     def __copy__(self):
@@ -2187,9 +2227,8 @@ def _add_prefixes(u, excludes=[], namespace=None, prefixes=False):
                        namespace=namespace, format=format)
 
 
-def def_unit(s, represents=None, register=None, doc=None,
-             format=None, prefixes=False, exclude_prefixes=[],
-             namespace=None):
+def def_unit(s, represents=None, doc=None, format=None, prefixes=False,
+             exclude_prefixes=[], namespace=None):
     """
     Factory function for defining new units.
 
@@ -2245,13 +2284,6 @@ def def_unit(s, represents=None, register=None, doc=None,
         The newly-defined unit, or a matching unit that was already
         defined.
     """
-    if register is not None:
-        warnings.warn(
-            "The register kwarg was removed in astropy 0.3. "
-            "Use the namespace kwarg to inject the unit into "
-            "a namespace and add_enabled_units() to enable it "
-            "in the global unit registry.",
-            DeprecationWarning)
 
     if represents is not None:
         result = Unit(s, represents, namespace=namespace, doc=doc,
@@ -2292,15 +2324,17 @@ def _condition_arg(value):
     if isinstance(value, np.ndarray) and value.dtype.kind in ['i', 'f', 'c']:
         return value
 
-    try:
-        avalue = np.array(value)
-        assert avalue.dtype.kind in ['i', 'f', 'c']
-        return avalue
-    except (ValueError, AssertionError):
+    avalue = np.array(value)
+    if avalue.dtype.kind not in ['i', 'f', 'c']:
         raise ValueError("Value not scalar compatible or convertible to "
                          "an int, float, or complex array")
+    return avalue
 
 
 dimensionless_unscaled = CompositeUnit(1, [], [], _error_check=False)
 # Abbreviation of the above, see #1980
 one = dimensionless_unscaled
+
+# Maintain error in old location for backward compatibility
+# TODO: Is this still needed? Should there be a deprecation warning?
+unit_format.fits.UnitScaleError = UnitScaleError
