@@ -16,35 +16,14 @@ from . import core
 from . import si
 from . import astrophys
 from . import cgs
+from . import imperial
 
 
 __all__ = ['def_physical_type', 'get_physical_type']
 
 
 _physical_unit_mapping = {}
-
-
-def _tuple_repr(unit):
-    """
-    Creates a canonical representation of a (possibly compound)
-    unit that can be used as a dictionary key.  This makes looking
-    up other units with the same signature easy.
-
-    Parameters
-    ----------
-    unit : `~astropy.units.UnitBase` instance
-        The unit to create a tuple for.
-
-    Returns
-    -------
-    canonical : tuple
-        A canonical, hashable representation of the unit.
-    """
-    unit = unit.decompose()
-    r = zip([unicode(x) for x in unit.bases], unit.powers)
-    r.sort()
-    r = tuple(r)
-    return r
+_unit_physical_mapping = {}
 
 
 def def_physical_type(unit, name):
@@ -59,12 +38,13 @@ def def_physical_type(unit, name):
     name : str
         The physical name of the unit.
     """
-    r = _tuple_repr(unit)
+    r = unit._get_physical_type_id()
     if r in _physical_unit_mapping:
         raise ValueError(
             "{0!r} ({1!r}) already defined as {2!r}".format(
                 r, name, _physical_unit_mapping[r]))
     _physical_unit_mapping[r] = name
+    _unit_physical_mapping[name] = r
 
 
 def get_physical_type(unit):
@@ -84,7 +64,7 @@ def get_physical_type(unit):
         The name of the physical quantity, or unknown if not
         known.
     """
-    r = _tuple_repr(unit)
+    r = unit._get_physical_type_id()
     return _physical_unit_mapping.get(r, 'unknown')
 
 
@@ -102,6 +82,8 @@ for unit, name in [
     (si.g, 'mass'),
     (si.mol, 'amount of substance'),
     (si.K, 'temperature'),
+    (si.deg_C, 'temperature'),
+    (imperial.deg_F, 'temperature'),
     (si.N, 'force'),
     (si.J, 'energy'),
     (si.Pa, 'pressure'),
@@ -135,13 +117,19 @@ for unit, name in [
     (si.H, 'inductance'),
     (si.cd, 'luminous intensity'),
     (si.lm, 'luminous flux'),
-    (si.lx, 'luminous emittence/illuminance'),
+    (si.lx, 'luminous emittance/illuminance'),
     (si.W / si.sr, 'radiant intensity'),
     (si.cd / si.m ** 2, 'luminance'),
     (astrophys.Jy, 'spectral flux density'),
     (cgs.erg / si.angstrom / si.cm ** 2 / si.s, 'spectral flux density wav'),
+    (astrophys.photon / si.Hz / si.cm ** 2 / si.s, 'photon flux density'),
+    (astrophys.photon / si.AA / si.cm ** 2 / si.s, 'photon flux density wav'),
     (astrophys.R, 'photon flux'),
     (astrophys.bit, 'data quantity'),
-    (astrophys.bit / si.s, 'bandwidth')
+    (astrophys.bit / si.s, 'bandwidth'),
+    (cgs.Franklin, 'electrical charge (ESU)'),
+    (cgs.statampere, 'electrical current (ESU)'),
+    (cgs.Biot, 'electrical current (EMU)'),
+    (cgs.abcoulomb, 'electrical charge (EMU)')
 ]:
     def_physical_type(unit, name)
