@@ -7,21 +7,23 @@
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
+import sys
 import copy
 import decimal
+from fractions import Fraction
 
+import pytest
 import numpy as np
 from numpy.testing import (assert_allclose, assert_array_equal,
                            assert_array_almost_equal)
 
-
-from ...tests.helper import raises, pytest
+from ...tests.helper import raises
 from ...utils import isiterable, minversion
-from ...utils.compat import NUMPY_LT_1_7
-from ...utils.compat.fractions import Fraction
+from ...utils.compat import NUMPY_LT_1_10, NUMPY_LT_1_14
+from ...utils.compat.numpy import matmul
 from ... import units as u
 from ...units.quantity import _UNIT_NOT_INITIALISED
-from ...extern.six.moves import xrange
+from ...extern.six.moves import range
 from ...extern.six.moves import cPickle as pickle
 from ...extern import six
 
@@ -30,7 +32,7 @@ try:
     matplotlib.use('Agg')
     import matplotlib.pyplot as plt
     from distutils.version import LooseVersion
-    MATPLOTLIB_LT_14 = LooseVersion(matplotlib.__version__) < LooseVersion("1.4")
+    MATPLOTLIB_LT_15 = LooseVersion(matplotlib.__version__) < LooseVersion("1.5")
     HAS_MATPLOTLIB = True
 except ImportError:
     HAS_MATPLOTLIB = False
@@ -85,6 +87,42 @@ class TestQuantityCreation(object):
         with pytest.raises(ValueError):  # Until @mdboom fixes the errors in units
             q1 = u.Quantity(11.412, unit="testingggg")
 
+    def test_nan_inf(self):
+        # Not-a-number
+        q = u.Quantity('nan', unit='cm')
+        assert np.isnan(q.value)
+
+        q = u.Quantity('NaN', unit='cm')
+        assert np.isnan(q.value)
+
+        q = u.Quantity('-nan', unit='cm')  # float() allows this
+        assert np.isnan(q.value)
+
+        q = u.Quantity('nan cm')
+        assert np.isnan(q.value)
+        assert q.unit == u.cm
+
+        # Infinity
+        q = u.Quantity('inf', unit='cm')
+        assert np.isinf(q.value)
+
+        q = u.Quantity('-inf', unit='cm')
+        assert np.isinf(q.value)
+
+        q = u.Quantity('inf cm')
+        assert np.isinf(q.value)
+        assert q.unit == u.cm
+
+        q = u.Quantity('Infinity', unit='cm')  # float() allows this
+        assert np.isinf(q.value)
+
+        # make sure these strings don't parse...
+        with pytest.raises(TypeError):
+            q = u.Quantity('', unit='cm')
+
+        with pytest.raises(TypeError):
+            q = u.Quantity('spam', unit='cm')
+
     def test_unit_property(self):
         # test getting and setting 'unit' attribute
         q1 = u.Quantity(11.4, unit=u.meter)
@@ -106,10 +144,13 @@ class TestQuantityCreation(object):
         assert q2.value == float(q1.value)
         assert q2.unit == q1.unit
 
-        # but we should preserve float32
-        a3 = np.array([1.,2.], dtype=np.float32)
-        q3 = u.Quantity(a3, u.yr)
-        assert q3.dtype == a3.dtype
+        # but we should preserve any float32 or even float16
+        a3_32 = np.array([1., 2.], dtype=np.float32)
+        q3_32 = u.Quantity(a3_32, u.yr)
+        assert q3_32.dtype == a3_32.dtype
+        a3_16 = np.array([1., 2.], dtype=np.float16)
+        q3_16 = u.Quantity(a3_16, u.yr)
+        assert q3_16.dtype == a3_16.dtype
         # items stored as objects by numpy should be converted to float
         # by default
         q4 = u.Quantity(decimal.Decimal('10.25'), u.m)
@@ -327,6 +368,32 @@ class TestQuantityOperations(object):
         assert_array_almost_equal(new_quantity.value, 1489.355288, decimal=7)
         assert new_quantity.unit == u.Unit("m^3")
 
+    @pytest.mark.skipif(sys.version_info[:2] < (3, 5),
+                        reason="__matmul__ only introduced in Python 3.5")
+    def test_matrix_multiplication(self):
+        # We cannot write @ as an operator since class gets parsed also on
+        # Python <3.5, so we use eval instead.
+        a = np.eye(3)
+        q = a * u.m
+        result1 = eval("q @ a")
+        assert np.all(result1 == q)
+        result2 = eval("a @ q")
+        assert np.all(result2 == q)
+        result3 = eval("q @ q")
+        assert np.all(result3 == a * u.m ** 2)
+        # less trivial case.
+        q2 = np.array([[[1., 0., 0.],
+                        [0., 1., 0.],
+                        [0., 0., 1.]],
+                       [[0., 1., 0.],
+                        [0., 0., 1.],
+                        [1., 0., 0.]],
+                       [[0., 0., 1.],
+                        [1., 0., 0.],
+                        [0., 1., 0.]]]) / u.s
+        result4 = eval("q @ q2")
+        assert np.all(result4 == matmul(a, q2.value) * q.unit * q2.unit)
+
     def test_unary(self):
 
         # Test the minus unary operator
@@ -461,9 +528,9 @@ class TestQuantityOperations(object):
         # quantities with units should never convert, or be usable as an index
         q1 = u.Quantity(1, u.m)
 
-        converter_err_msg = ("Only dimensionless scalar quantities "
+        converter_err_msg = ("only dimensionless scalar quantities "
                              "can be converted to Python scalars")
-        index_err_msg = ("Only integer dimensionless scalar quantities "
+        index_err_msg = ("only integer dimensionless scalar quantities "
                          "can be converted to a Python index")
         with pytest.raises(TypeError) as exc:
             float(q1)
@@ -489,11 +556,11 @@ class TestQuantityOperations(object):
         # dimensionless but scaled is OK, however
         q2 = u.Quantity(1.23, u.m / u.km)
 
-        assert float(q2) == float(q2.to(u.dimensionless_unscaled).value)
-        assert int(q2) == int(q2.to(u.dimensionless_unscaled).value)
+        assert float(q2) == float(q2.to_value(u.dimensionless_unscaled))
+        assert int(q2) == int(q2.to_value(u.dimensionless_unscaled))
 
         if six.PY2:
-            assert long(q2) == long(q2.to(u.dimensionless_unscaled).value)
+            assert long(q2) == long(q2.to_value(u.dimensionless_unscaled))
 
         with pytest.raises(TypeError) as exc:
             q2.__index__()
@@ -508,7 +575,7 @@ class TestQuantityOperations(object):
             assert long(q3) == 1
 
         with pytest.raises(TypeError) as exc:
-            q1.__index__()
+            q3.__index__()
         assert exc.value.args[0] == index_err_msg
 
         # integer dimensionless unscaled is good for all
@@ -540,6 +607,15 @@ class TestQuantityOperations(object):
             q5.__index__()
         assert exc.value.args[0] == index_err_msg
 
+    # Fails for numpy >=1.10; see https://github.com/numpy/numpy/issues/5074
+    # It seems unlikely this will be resolved, so xfail'ing it.
+    @pytest.mark.xfail(not NUMPY_LT_1_10,
+                       reason="list multiplication only works for numpy <=1.10")
+    def test_numeric_converter_to_index_in_practice(self):
+        """Test that use of __index__ actually works."""
+        q4 = u.Quantity(2, u.dimensionless_unscaled, dtype=int)
+        assert q4 * ['a', 'b', 'c'] == ['a', 'b', 'c', 'a', 'b', 'c']
+
     def test_array_converters(self):
 
         # Scalar quantity
@@ -553,26 +629,55 @@ class TestQuantityOperations(object):
 
 def test_quantity_conversion():
     q1 = u.Quantity(0.1, unit=u.meter)
-
+    value = q1.value
+    assert value == 0.1
+    value_in_km = q1.to_value(u.kilometer)
+    assert value_in_km == 0.0001
     new_quantity = q1.to(u.kilometer)
     assert new_quantity.value == 0.0001
 
     with pytest.raises(u.UnitsError):
         q1.to(u.zettastokes)
+    with pytest.raises(u.UnitsError):
+        q1.to_value(u.zettastokes)
+
+
+def test_quantity_value_views():
+    q1 = u.Quantity([1., 2.], unit=u.meter)
+    # views if the unit is the same.
+    v1 = q1.value
+    v1[0] = 0.
+    assert np.all(q1 == [0., 2.] * u.meter)
+    v2 = q1.to_value()
+    v2[1] = 3.
+    assert np.all(q1 == [0., 3.] * u.meter)
+    v3 = q1.to_value('m')
+    v3[0] = 1.
+    assert np.all(q1 == [1., 3.] * u.meter)
+    v4 = q1.to_value('cm')
+    v4[0] = 0.
+    # copy if different unit.
+    assert np.all(q1 == [1., 3.] * u.meter)
 
 
 def test_quantity_conversion_with_equiv():
     q1 = u.Quantity(0.1, unit=u.meter)
+    v2 = q1.to_value(u.Hz, equivalencies=u.spectral())
+    assert_allclose(v2, 2997924580.0)
     q2 = q1.to(u.Hz, equivalencies=u.spectral())
-    assert_allclose(q2.value, 2997924580.0)
+    assert_allclose(q2.value, v2)
 
     q1 = u.Quantity(0.4, unit=u.arcsecond)
+    v2 = q1.to_value(u.au, equivalencies=u.parallax())
     q2 = q1.to(u.au, equivalencies=u.parallax())
+    v3 = q2.to_value(u.arcminute, equivalencies=u.parallax())
     q3 = q2.to(u.arcminute, equivalencies=u.parallax())
 
-    assert_allclose(q2.value, 515662.015)
+    assert_allclose(v2, 515662.015)
+    assert_allclose(q2.value, v2)
     assert q2.unit == u.au
-    assert_allclose(q3.value, 0.0066666667)
+    assert_allclose(v3, 0.0066666667)
+    assert_allclose(q3.value, v3)
     assert q3.unit == u.arcminute
 
 
@@ -598,10 +703,13 @@ def test_quantity_conversion_equivalency_passed_on():
     assert_allclose(q4.value, q5.value)
 
 # Regression test for issue #2315, divide-by-zero error when examining 0*unit
+
+
 def test_self_equivalency():
     assert u.deg.is_equivalent(0*u.radian)
     assert u.deg.is_equivalent(1*u.radian)
 
+
 def test_si():
     q1 = 10. * u.m * u.s ** 2 / (200. * u.ms) ** 2  # 250 meters
     assert q1.si.value == 250
@@ -677,16 +785,23 @@ class TestQuantityDisplay(object):
     def test_dimensionless_quantity_repr(self):
         q2 = u.Quantity(1., unit='m-1')
         q3 = u.Quantity(1, unit='m-1', dtype=int)
-        assert repr(self.scalarintq * q2) == "<Quantity 1.0>"
+        if NUMPY_LT_1_14:
+            assert repr(self.scalarintq * q2) == "<Quantity 1.0>"
+            assert repr(self.arrq * q2) == "<Quantity [ 1. , 2.3, 8.9]>"
+        else:
+            assert repr(self.scalarintq * q2) == "<Quantity 1.>"
+            assert repr(self.arrq * q2) == "<Quantity [1. , 2.3, 8.9]>"
         assert repr(self.scalarintq * q3) == "<Quantity 1>"
-        assert repr(self.arrq * q2) == "<Quantity [ 1. , 2.3, 8.9]>"
 
     def test_dimensionless_quantity_str(self):
         q2 = u.Quantity(1., unit='m-1')
         q3 = u.Quantity(1, unit='m-1', dtype=int)
         assert str(self.scalarintq * q2) == "1.0"
         assert str(self.scalarintq * q3) == "1"
-        assert str(self.arrq * q2) == "[ 1.   2.3  8.9]"
+        if NUMPY_LT_1_14:
+            assert str(self.arrq * q2) == "[ 1.   2.3  8.9]"
+        else:
+            assert str(self.arrq * q2) == "[1.  2.3 8.9]"
 
     def test_dimensionless_quantity_format(self):
         q1 = u.Quantity(3.14)
@@ -701,10 +816,16 @@ class TestQuantityDisplay(object):
         assert repr(self.scalarfloatq) == "<Quantity 1.3 m>"
 
     def test_array_quantity_str(self):
-        assert str(self.arrq) == "[ 1.   2.3  8.9] m"
+        if NUMPY_LT_1_14:
+            assert str(self.arrq) == "[ 1.   2.3  8.9] m"
+        else:
+            assert str(self.arrq) == "[1.  2.3 8.9] m"
 
     def test_array_quantity_repr(self):
-        assert repr(self.arrq) == "<Quantity [ 1. , 2.3, 8.9] m>"
+        if NUMPY_LT_1_14:
+            assert repr(self.arrq) == "<Quantity [ 1. , 2.3, 8.9] m>"
+        else:
+            assert repr(self.arrq) == "<Quantity [1. , 2.3, 8.9] m>"
 
     def test_scalar_quantity_format(self):
         assert format(self.scalarintq, '02d') == "01 m"
@@ -720,17 +841,11 @@ class TestQuantityDisplay(object):
         from ...units.quantity import conf
 
         q2scalar = u.Quantity(1.5e14, 'm/s')
-        assert self.scalarintq._repr_latex_() == '$1 \\; \\mathrm{m}$'
-        assert self.scalarfloatq._repr_latex_() == '$1.3 \\; \\mathrm{m}$'
+        assert self.scalarintq._repr_latex_() == r'$1 \; \mathrm{m}$'
+        assert self.scalarfloatq._repr_latex_() == r'$1.3 \; \mathrm{m}$'
         assert (q2scalar._repr_latex_() ==
-                '$1.5 \\times 10^{14} \\; \\mathrm{\\frac{m}{s}}$')
-
-        if NUMPY_LT_1_7:
-            with pytest.raises(NotImplementedError):
-                self.arrq._repr_latex_()
-            return  # all arrays should fail
-
-        assert self.arrq._repr_latex_() == '$[1,~2.3,~8.9] \; \mathrm{m}$'
+                r'$1.5 \times 10^{14} \; \mathrm{\frac{m}{s}}$')
+        assert self.arrq._repr_latex_() == r'$[1,~2.3,~8.9] \; \mathrm{m}$'
 
         qmed = np.arange(100)*u.m
         qbig = np.arange(1000)*u.m
@@ -739,7 +854,17 @@ class TestQuantityDisplay(object):
         pops = np.get_printoptions()
         oldlat = conf.latex_array_threshold
         try:
-            #check thresholding behavior
+            # check precision behavior
+            q = u.Quantity(987654321.123456789, 'm/s')
+            qa = np.array([7.89123, 123456789.987654321, 0]) * u.cm
+            np.set_printoptions(precision=8)
+            assert q._repr_latex_() == r'$9.8765432 \times 10^{8} \; \mathrm{\frac{m}{s}}$'
+            assert qa._repr_latex_() == r'$[7.89123,~1.2345679 \times 10^{8},~0] \; \mathrm{cm}$'
+            np.set_printoptions(precision=2)
+            assert q._repr_latex_() == r'$9.9 \times 10^{8} \; \mathrm{\frac{m}{s}}$'
+            assert qa._repr_latex_() == r'$[7.9,~1.2 \times 10^{8},~0] \; \mathrm{cm}$'
+
+            # check thresholding behavior
             conf.latex_array_threshold = 100  # should be default
             lsmed = qmed._repr_latex_()
             assert r'\dots' not in lsmed
@@ -756,7 +881,6 @@ class TestQuantityDisplay(object):
             lsvbig = qvbig._repr_latex_()
             assert r'\dots' in lsvbig
 
-
             conf.latex_array_threshold = -1  # means use the numpy threshold
             np.set_printoptions(threshold=99)
             lsmed = qmed._repr_latex_()
@@ -774,7 +898,6 @@ class TestQuantityDisplay(object):
         assert qinfnan._repr_latex_() == r'$[\infty,~-\infty,~{\rm NaN}] \; \mathrm{m}$'
 
 
-
 def test_decompose():
     q1 = 5 * u.N
     assert q1.decompose() == (5 * u.kg * u.m * u.s ** -2)
@@ -830,7 +953,7 @@ def test_arrays():
 
     # but with multiple dtypes, single elements are OK; need to use str()
     # since numpy under python2 cannot handle unicode literals
-    a = np.array([(1.,2.,3.), (4.,5.,6.), (7.,8.,9.)],
+    a = np.array([(1., 2., 3.), (4., 5., 6.), (7., 8., 9.)],
                  dtype=[(str('x'), np.float),
                         (str('y'), np.float),
                         (str('z'), np.float)])
@@ -855,7 +978,7 @@ def test_arrays():
     assert qkpc1x == qkpcx1
 
     # can also create from lists, will auto-convert to arrays
-    qsec = u.Quantity(list(xrange(10)), u.second)
+    qsec = u.Quantity(list(range(10)), u.second)
     assert isinstance(qsec.value, np.ndarray)
 
     # quantity math should work with arrays
@@ -892,7 +1015,7 @@ def test_array_indexing_slicing():
 
 
 def test_array_setslice():
-    q = np.array([1., 2., 3. ]) * u.m
+    q = np.array([1., 2., 3.]) * u.m
     q[1:2] = np.array([400.]) * u.cm
     assert np.all(q == np.array([1., 4., 3.]) * u.m)
 
@@ -1062,19 +1185,52 @@ def test_quantity_pickelability():
     assert q1.unit == q2.unit
 
 
-def test_quantity_from_string():
-    with pytest.raises(TypeError):
-        q = u.Quantity(u.m * "5")
-        # the reverse should also fail once #1408 is in
+def test_quantity_initialisation_from_string():
+    q = u.Quantity('1')
+    assert q.unit == u.dimensionless_unscaled
+    assert q.value == 1.
+    q = u.Quantity('1.5 m/s')
+    assert q.unit == u.m/u.s
+    assert q.value == 1.5
+    assert u.Unit(q) == u.Unit('1.5 m/s')
+    q = u.Quantity('.5 m')
+    assert q == u.Quantity(0.5, u.m)
+    q = u.Quantity('-1e1km')
+    assert q == u.Quantity(-10, u.km)
+    q = u.Quantity('-1e+1km')
+    assert q == u.Quantity(-10, u.km)
+    q = u.Quantity('+.5km')
+    assert q == u.Quantity(.5, u.km)
+    q = u.Quantity('+5e-1km')
+    assert q == u.Quantity(.5, u.km)
+    q = u.Quantity('5', u.m)
+    assert q == u.Quantity(5., u.m)
+    q = u.Quantity('5 km', u.m)
+    assert q.value == 5000.
+    assert q.unit == u.m
+    q = u.Quantity('5Em')
+    assert q == u.Quantity(5., u.Em)
 
     with pytest.raises(TypeError):
-        q = u.Quantity('5', u.m)
-
+        u.Quantity('')
     with pytest.raises(TypeError):
-        q = u.Quantity(['5'], u.m)
-
+        u.Quantity('m')
     with pytest.raises(TypeError):
-        q = u.Quantity(np.array(['5']), u.m)
+        u.Quantity('1.2.3 deg')
+    with pytest.raises(TypeError):
+        u.Quantity('1+deg')
+    with pytest.raises(TypeError):
+        u.Quantity('1-2deg')
+    with pytest.raises(TypeError):
+        u.Quantity('1.2e-13.3m')
+    with pytest.raises(TypeError):
+        u.Quantity(['5'])
+    with pytest.raises(TypeError):
+        u.Quantity(np.array(['5']))
+    with pytest.raises(ValueError):
+        u.Quantity('5E')
+    with pytest.raises(ValueError):
+        u.Quantity('5 foo')
 
 
 def test_unsupported():
@@ -1109,6 +1265,7 @@ def test_quantity_fraction_power():
     # the value of the quantity to a Fraction. [#3922]
     assert q.dtype.kind == 'f'
 
+
 def test_inherit_docstrings():
     assert u.Quantity.argmax.__doc__ == np.ndarray.argmax.__doc__
 
@@ -1119,7 +1276,7 @@ def test_quantity_from_table():
     This also generically checks the use of *anything* with a `unit` attribute
     passed into Quantity
     """
-    from... table import Table
+    from astropy.table import Table
 
     t = Table(data=[np.arange(5), np.arange(5)], names=['a', 'b'])
     t['a'].unit = u.kpc
@@ -1142,6 +1299,28 @@ def test_quantity_from_table():
     assert qbp.unit == u.pc
     assert_array_equal(qbp.value, t['b'])
 
+
+def test_assign_slice_with_quantity_like():
+    # Regression tests for gh-5961
+    from ... table import Table, Column
+    # first check directly that we can use a Column to assign to a slice.
+    c = Column(np.arange(10.), unit=u.mm)
+    q = u.Quantity(c)
+    q[:2] = c[:2]
+    # next check that we do not fail the original problem.
+    t = Table()
+    t['x'] = np.arange(10) * u.mm
+    t['y'] = np.ones(10) * u.mm
+    assert type(t['x']) is Column
+
+    xy = np.vstack([t['x'], t['y']]).T * u.mm
+    ii = [0, 2, 4]
+
+    assert xy[ii, 0].unit == t['x'][ii].unit
+    # should not raise anything
+    xy[ii, 0] = t['x'][ii]
+
+
 def test_insert():
     """
     Test Quantity.insert method.  This does not test the full capabilities
@@ -1152,7 +1331,7 @@ def test_insert():
 
     # Insert a compatible float with different units
     q2 = q.insert(0, 1 * u.km)
-    assert np.all(q2.value == [ 1000,  1,  2])
+    assert np.all(q2.value == [1000, 1, 2])
     assert q2.unit is u.m
     assert q2.dtype.kind == 'f'
 
@@ -1168,17 +1347,17 @@ def test_insert():
     # Tests with multi-dim quantity
     q = [[1, 2], [3, 4]] * u.m
     q2 = q.insert(1, [10, 20] * u.m, axis=0)
-    assert np.all(q2.value == [[  1,  2],
-                               [ 10, 20],
-                               [  3,  4]])
+    assert np.all(q2.value == [[1, 2],
+                               [10, 20],
+                               [3, 4]])
 
     q2 = q.insert(1, [10, 20] * u.m, axis=1)
-    assert np.all(q2.value == [[  1, 10,  2],
-                               [  3, 20,  4]])
+    assert np.all(q2.value == [[1, 10, 2],
+                               [3, 20, 4]])
 
     q2 = q.insert(1, 10 * u.m, axis=1)
-    assert np.all(q2.value == [[  1,  10, 2],
-                               [  3,  10, 4]])
+    assert np.all(q2.value == [[1, 10, 2],
+                               [3, 10, 4]])
 
 
 def test_repr_array_of_quantity():
@@ -1191,18 +1370,75 @@ def test_repr_array_of_quantity():
     """
 
     a = np.array([1 * u.m, 2 * u.s], dtype=object)
-    if NUMPY_LT_1_7:
-        # Numpy 1.6.x has some different defaults for how to display object
-        # arrays (it uses the str() of the objects instead of the repr()
-        assert repr(a) == 'array([1.0 m, 2.0 s], dtype=object)'
-        assert str(a) == '[1.0 m 2.0 s]'
-    else:
+    if NUMPY_LT_1_14:
         assert repr(a) == 'array([<Quantity 1.0 m>, <Quantity 2.0 s>], dtype=object)'
         assert str(a) == '[<Quantity 1.0 m> <Quantity 2.0 s>]'
+    else:
+        assert repr(a) == 'array([<Quantity 1. m>, <Quantity 2. s>], dtype=object)'
+        assert str(a) == '[<Quantity 1. m> <Quantity 2. s>]'
+
+
+class TestSpecificTypeQuantity(object):
+    def setup(self):
+        class Length(u.SpecificTypeQuantity):
+            _equivalent_unit = u.m
+
+        class Length2(Length):
+            _default_unit = u.m
+
+        class Length3(Length):
+            _unit = u.m
+
+        self.Length = Length
+        self.Length2 = Length2
+        self.Length3 = Length3
+
+    def test_creation(self):
+        l = self.Length(np.arange(10.)*u.km)
+        assert type(l) is self.Length
+        with pytest.raises(u.UnitTypeError):
+            self.Length(np.arange(10.) * u.hour)
+
+        with pytest.raises(u.UnitTypeError):
+            self.Length(np.arange(10.))
+
+        l2 = self.Length2(np.arange(5.))
+        assert type(l2) is self.Length2
+        assert l2._default_unit is self.Length2._default_unit
+
+        with pytest.raises(u.UnitTypeError):
+            self.Length3(np.arange(10.))
+
+    def test_view(self):
+        l = (np.arange(5.) * u.km).view(self.Length)
+        assert type(l) is self.Length
+        with pytest.raises(u.UnitTypeError):
+            (np.arange(5.) * u.s).view(self.Length)
+
+        v = np.arange(5.).view(self.Length)
+        assert type(v) is self.Length
+        assert v._unit is None
+
+        l3 = np.ones((2, 2)).view(self.Length3)
+        assert type(l3) is self.Length3
+        assert l3.unit is self.Length3._unit
+
+    def test_operation_precedence_and_fallback(self):
+        l = self.Length(np.arange(5.)*u.cm)
+        sum1 = l + 1.*u.m
+        assert type(sum1) is self.Length
+        sum2 = 1.*u.km + l
+        assert type(sum2) is self.Length
+        sum3 = l + l
+        assert type(sum3) is self.Length
+        res1 = l * (1.*u.m)
+        assert type(res1) is u.Quantity
+        res2 = l * l
+        assert type(res2) is u.Quantity
 
 
 @pytest.mark.skipif('not HAS_MATPLOTLIB')
-@pytest.mark.xfail('MATPLOTLIB_LT_14')
+@pytest.mark.xfail('MATPLOTLIB_LT_15')
 class TestQuantityMatplotlib(object):
     """Test if passing matplotlib quantities works.
 
@@ -1222,3 +1458,15 @@ class TestQuantityMatplotlib(object):
         x = u.Quantity([4, 5, 6], 'second')
         y = [1, 3, 4] * u.m
         plt.scatter(x, y)
+
+
+def test_unit_class_override():
+    class MyQuantity(u.Quantity):
+        pass
+
+    my_unit = u.Unit("my_deg", u.deg)
+    my_unit._quantity_class = MyQuantity
+    q1 = u.Quantity(1., my_unit)
+    assert type(q1) is u.Quantity
+    q2 = u.Quantity(1., my_unit, subok=True)
+    assert type(q2) is MyQuantity
