@@ -286,8 +286,9 @@ def deprecated_attribute(name, since, message=None, alternative=None,
 def deprecated_renamed_argument(old_name, new_name, since,
                                 arg_in_kwargs=False, relax=False,
                                 pending=False,
-                                warning_type=AstropyDeprecationWarning):
-    """Deprecate a _renamed_ function argument.
+                                warning_type=AstropyDeprecationWarning,
+                                alternative=''):
+    """Deprecate a _renamed_ or _removed_ function argument.
 
     The decorator assumes that the argument with the ``old_name`` was removed
     from the function signature and the ``new_name`` replaced it at the
@@ -300,8 +301,9 @@ def deprecated_renamed_argument(old_name, new_name, since,
     old_name : str or list/tuple thereof
         The old name of the argument.
 
-    new_name : str or list/tuple thereof
-        The new name of the argument.
+    new_name : str or list/tuple thereof or `None`
+        The new name of the argument. Set this to `None` to remove the
+        argument ``old_name`` instead of renaming it.
 
     since : str or number or list/tuple thereof
         The release at which the old argument became deprecated.
@@ -329,6 +331,12 @@ def deprecated_renamed_argument(old_name, new_name, since,
         Warning to be issued.
         Default is `~astropy.utils.exceptions.AstropyDeprecationWarning`.
 
+    alternative : str, optional
+        An alternative function or class name that the user may use in
+        place of the deprecated object if ``new_name`` is None. The deprecation
+        warning will tell the user about this alternative if provided.
+
+
     Raises
     ------
     TypeError
@@ -446,8 +454,24 @@ def deprecated_renamed_argument(old_name, new_name, since,
 
         for i in range(n):
             # Determine the position of the argument.
-            if new_name[i] in arguments:
-                param = arguments[new_name[i]]
+            if arg_in_kwargs[i]:
+                pass
+            else:
+                if new_name[i] is None:
+                    continue
+                elif new_name[i] in arguments:
+                    param = arguments[new_name[i]]
+                # In case the argument is not found in the list of arguments
+                # the only remaining possibility is that it should be caught
+                # by some kind of **kwargs argument.
+                # This case has to be explicitly specified, otherwise throw
+                # an exception!
+                else:
+                    raise TypeError('"{}" was not specified in the function '
+                                    'signature. If it was meant to be part of '
+                                    '"**kwargs" then set "arg_in_kwargs" to "True"'
+                                    '.'.format(new_name[i]))
+
                 # There are several possibilities now:
 
                 # 1.) Positional or keyword argument:
@@ -465,19 +489,6 @@ def deprecated_renamed_argument(old_name, new_name, since,
                     raise TypeError('cannot replace argument "{0}" of kind '
                                     '{1!r}.'.format(new_name[i], param.kind))
 
-            # In case the argument is not found in the list of arguments
-            # the only remaining possibility is that it should be caught
-            # by some kind of **kwargs argument.
-            # This case has to be explicitly specified, otherwise throw
-            # an exception!
-            elif arg_in_kwargs[i]:
-                position[i] = None
-            else:
-                raise TypeError('"{}" was not specified in the function '
-                                'signature. If it was meant to be part of '
-                                '"**kwargs" then set "arg_in_kwargs" to "True"'
-                                '.'.format(new_name[i]))
-
         @functools.wraps(function)
         def wrapper(*args, **kwargs):
             for i in range(n):
@@ -486,15 +497,19 @@ def deprecated_renamed_argument(old_name, new_name, since,
                 # parameter was renamed to newkeyword.
                 if old_name[i] in kwargs:
                     value = kwargs.pop(old_name[i])
-                    # Display the deprecation warning only when it's only
+                    # Display the deprecation warning only when it's not
                     # pending.
                     if not pending[i]:
-                        warnings.warn(
-                            '"{0}" was deprecated in version {1} '
-                            'and will be removed in a future version. '
-                            'Use argument "{2}" instead.'
-                            ''.format(old_name[i], since[i], new_name[i]),
-                            warning_type, stacklevel=2)
+                        message = ('"{0}" was deprecated in version {1} '
+                                   'and will be removed in a future version. '
+                                   .format(old_name[i], since[i]))
+                        if new_name[i] is not None:
+                            message += ('Use argument "{}" instead.'
+                                        .format(new_name[i]))
+                        elif alternative:
+                            message += ('\n        Use {} instead.'
+                                        .format(alternative))
+                        warnings.warn(message, warning_type, stacklevel=2)
 
                     # Check if the newkeyword was given as well.
                     newarg_in_args = (position[i] is not None and
@@ -516,9 +531,11 @@ def deprecated_renamed_argument(old_name, new_name, since,
                                     'cannot specify both "{}" and "{}"'
                                     '.'.format(old_name[i], new_name[i]))
                     else:
-                        # If the new argument isn't specified just pass the old
-                        # one with the name of the new argument to the function
-                        kwargs[new_name[i]] = value
+                        # Pass the value of the old argument with the
+                        # name of the new argument to the function
+                        if new_name[i] is not None:
+                            kwargs[new_name[i]] = value
+
             return function(*args, **kwargs)
 
         return wrapper
