@@ -101,7 +101,7 @@ def format_exception(msg, *args, **kwargs):
         different behavior regarding `sys.exc_info` (the latter will not
         carry it outside a handled exception), so it's not wise to use this
         outside of an ``except`` clause - if it is, this will substitute
-        '<unkown>' for the 4 formatting arguments.
+        '<unkonwn>' for the 4 formatting arguments.
     """
 
     tb = traceback.extract_tb(sys.exc_info()[2], limit=1)
