@@ -3,19 +3,21 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import os
+from distutils.version import LooseVersion
+
+from ..mpl_normalize import simple_norm
 from ... import log
 from ...io.fits import getdata
-from ..ui import scale_image
 
 
-def fits2bitmap(filename, ext=0, out_fn=None, scale='linear',
+def fits2bitmap(filename, ext=0, out_fn=None, stretch='linear',
                 power=1.0, asinh_a=0.1, min_cut=None, max_cut=None,
                 min_percent=None, max_percent=None, percent=None,
                 cmap='Greys_r'):
     """
-    Create a bitmap file from a FITS image, applying a
-    scaling/stretching transform between minimum and maximum cut levels
-    and a matplotlib colormap.
+    Create a bitmap file from a FITS image, applying a stretching
+    transform between minimum and maximum cut levels and a matplotlib
+    colormap.
 
     Parameters
     ----------
@@ -28,25 +30,24 @@ def fits2bitmap(filename, ext=0, out_fn=None, scale='linear',
         The filename of the output bitmap image.  The type of bitmap
         is determined by the filename extension (e.g. '.jpg', '.png').
         The default is a PNG file with the same name as the FITS file.
-    scale : {{'linear', 'sqrt', 'power', log', 'asinh'}}
-        The scaling/stretch function to apply to the image.  The default
-        is 'linear'.
+    stretch : {{'linear', 'sqrt', 'power', log', 'asinh'}}
+        The stretching function to apply to the image.  The default is
+        'linear'.
     power : float, optional
-        The power index for ``scale='power'`` image scaling.  The
-        default is 1.0.
+        The power index for ``stretch='power'``.  The default is 1.0.
     asinh_a : float, optional
-        For ``scale='asinh'`` image scaling, the value where the asinh
-        curve transitions from linear to logarithmic behavior, expressed
-        as a fraction of the normalized image.  Must be in the range
-        between 0 and 1.
+        For ``stretch='asinh'``, the value where the asinh curve
+        transitions from linear to logarithmic behavior, expressed as a
+        fraction of the normalized image.  Must be in the range between
+        0 and 1.  The default is 0.1.
     min_cut : float, optional
         The pixel value of the minimum cut level.  Data values less than
-        ``min_cut`` will set to ``min_cut`` before scaling the image.
+        ``min_cut`` will set to ``min_cut`` before stretching the image.
         The default is the image minimum.  ``min_cut`` overrides
         ``min_percent``.
     max_cut : float, optional
         The pixel value of the maximum cut level.  Data values greater
-        than ``min_cut`` will set to ``min_cut`` before scaling the
+        than ``min_cut`` will set to ``min_cut`` before stretching the
         image.  The default is the image maximum.  ``max_cut`` overrides
         ``max_percent``.
     min_percent : float, optional
@@ -68,10 +69,10 @@ def fits2bitmap(filename, ext=0, out_fn=None, scale='linear',
         The matplotlib color map name.  The default is 'Greys_r'.
     """
 
+    import matplotlib
     import matplotlib.cm as cm
     import matplotlib.image as mimg
 
-
     # __main__ gives ext as a string
     try:
         ext = int(ext)
@@ -94,23 +95,36 @@ def fits2bitmap(filename, ext=0, out_fn=None, scale='linear',
             out_fn = os.path.splitext(out_fn)[0]
         out_fn += '.png'
 
-    if cmap not in cm.datad:
-        log.critical('{0} is not a valid matplotlib colormap '
-                     'name'.format(cmap))
+    # need to explicitly define the output format due to a bug in
+    # matplotlib (<= 2.1), otherwise the format will always be PNG
+    out_format = os.path.splitext(out_fn)[1][1:]
+
+    # workaround for matplotlib 2.0.0 bug where png images are inverted
+    # (mpl-#7656)
+    if (out_format.lower() == 'png' and
+            LooseVersion(matplotlib.__version__) == LooseVersion('2.0.0')):
+        image = image[::-1]
+
+    try:
+        cm.get_cmap(cmap)
+    except ValueError:
+        log.critical('{0} is not a valid matplotlib colormap name.'
+                     .format(cmap))
         return 1
 
-    image_scaled = scale_image(image, scale=scale, power=power,
-                               asinh_a=asinh_a, min_cut=min_cut,
-                               max_cut=max_cut, min_percent=min_percent,
-                               max_percent=max_percent, percent=percent)
+    norm = simple_norm(image, stretch=stretch, power=power, asinh_a=asinh_a,
+                       min_cut=min_cut, max_cut=max_cut,
+                       min_percent=min_percent, max_percent=max_percent,
+                       percent=percent)
 
-    mimg.imsave(out_fn, image_scaled, cmap=cmap)
-    log.info('Saved file to {0}'.format(out_fn))
+    mimg.imsave(out_fn, norm(image), cmap=cmap, origin='lower',
+                format=out_format)
+    log.info('Saved file to {0}.'.format(out_fn))
 
 
 def main(args=None):
 
-    from ...utils.compat import argparse
+    import argparse
 
     parser = argparse.ArgumentParser(
         description='Create a bitmap file from a FITS image.')
@@ -120,16 +134,16 @@ def main(args=None):
     parser.add_argument('-o', metavar='filename', type=str, default=None,
                         help='Filename for the output image (Default is a '
                         'PNG file with the same name as the FITS file).')
-    parser.add_argument('--scale', type=str, default='linear',
-                        help='Type of image scaling ("linear", "sqrt", '
+    parser.add_argument('--stretch', type=str, default='linear',
+                        help='Type of image stretching ("linear", "sqrt", '
                         '"power", "log", or "asinh") (Default is "linear").')
     parser.add_argument('--power', type=float, default=1.0,
-                        help='Power index for "power" scaling (Default is '
+                        help='Power index for "power" stretching (Default is '
                              '1.0).')
     parser.add_argument('--asinh_a', type=float, default=0.1,
                         help='The value in normalized image where the asinh '
                              'curve transitions from linear to logarithmic '
-                             'behavior (used only for "asinh" scaling) '
+                             'behavior (used only for "asinh" stretch) '
                              '(Default is 0.1).')
     parser.add_argument('--min_cut', type=float, default=None,
                         help='The pixel value of the minimum cut level '
@@ -156,7 +170,7 @@ def main(args=None):
 
     for filename in args.filename:
         fits2bitmap(filename, ext=args.ext, out_fn=args.o,
-                    scale=args.scale, min_cut=args.min_cut,
+                    stretch=args.stretch, min_cut=args.min_cut,
                     max_cut=args.max_cut, min_percent=args.min_percent,
                     max_percent=args.max_percent, percent=args.percent,
                     power=args.power, asinh_a=args.asinh_a, cmap=args.cmap)
