@@ -3,16 +3,16 @@
          mdroe@stsci.edu
 */
 
-#include "astropy_wcs.h"
-#include "wcslib_wrap.h"
-#include "wcslib_tabprm_wrap.h"
-#include "wcslib_units_wrap.h"
-#include "wcslib_wtbarr_wrap.h"
-#include "distortion_wrap.h"
-#include "sip_wrap.h"
-#include "docstrings.h"
-#include "astropy_wcs_api.h"
-#include "unit_list_proxy.h"
+#include "astropy_wcs/astropy_wcs.h"
+#include "astropy_wcs/wcslib_wrap.h"
+#include "astropy_wcs/wcslib_tabprm_wrap.h"
+#include "astropy_wcs/wcslib_units_wrap.h"
+#include "astropy_wcs/wcslib_wtbarr_wrap.h"
+#include "astropy_wcs/distortion_wrap.h"
+#include "astropy_wcs/sip_wrap.h"
+#include "astropy_wcs/docstrings.h"
+#include "astropy_wcs/astropy_wcs_api.h"
+#include "astropy_wcs/unit_list_proxy.h"
 
 #include <structmember.h> /* from Python */
 
@@ -85,6 +85,7 @@ static void
 Wcs_dealloc(
     Wcs* self) {
 
+  PyObject_GC_UnTrack(self);
   Wcs_clear(self);
   pipeline_free(&self->x);
   Py_TYPE(self)->tp_free((PyObject*)self);
@@ -142,6 +143,7 @@ Wcs_init(
         return -1;
       }
 
+      Py_XDECREF(self->py_det2im[i]);
       self->py_det2im[i] = py_det2im[i];
       self->x.det2im[i] = &(((PyDistLookup*)py_det2im[i])->x);
     }
@@ -155,6 +157,7 @@ Wcs_init(
       return -1;
     }
 
+    Py_XDECREF(self->py_sip);
     self->py_sip = py_sip;
     self->x.sip = &(((PySip*)py_sip)->x);
   }
@@ -168,6 +171,7 @@ Wcs_init(
         return -1;
       }
 
+      Py_XDECREF(self->py_distortion_lookup[i]);
       self->py_distortion_lookup[i] = py_distortion_lookup[i];
       self->x.cpdis[i] = &(((PyDistLookup*)py_distortion_lookup[i])->x);
     }
@@ -181,6 +185,7 @@ Wcs_init(
       return -1;
     }
 
+    Py_XDECREF(self->py_wcsprm);
     self->py_wcsprm = py_wcsprm;
     self->x.wcs = &(((PyWcsprm*)py_wcsprm)->x);
   }
@@ -218,7 +223,7 @@ Wcs_all_pix2world(
 
   naxis = self->x.wcs->naxis;
 
-  pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny(pixcrd_obj, PyArray_DOUBLE, 2, 2);
+  pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny(pixcrd_obj, NPY_DOUBLE, 2, 2);
   if (pixcrd == NULL) {
     return NULL;
   }
@@ -231,7 +236,7 @@ Wcs_all_pix2world(
     goto exit;
   }
 
-  world = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+  world = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (world == NULL) {
     goto exit;
   }
@@ -255,19 +260,21 @@ Wcs_all_pix2world(
 
   if (status == 0 || status == 8) {
     return (PyObject*)world;
-  } else if (status == -1) {
-    PyErr_SetString(
-      PyExc_ValueError,
-      "Wrong number of dimensions in input array.  Expected 2.");
-    return NULL;
   } else {
-    Py_DECREF(world);
+    Py_XDECREF(world);
     if (status == -1) {
-      /* exception already set */
+      PyErr_SetString(
+        PyExc_ValueError,
+        "Wrong number of dimensions in input array.  Expected 2.");
       return NULL;
     } else {
-      wcserr_to_python_exc(self->x.err);
-      return NULL;
+      if (status == -1) {
+        /* exception already set */
+        return NULL;
+      } else {
+        wcserr_to_python_exc(self->x.err);
+        return NULL;
+      }
     }
   }
 }
@@ -296,7 +303,7 @@ Wcs_p4_pix2foc(
     return pixcrd_obj;
   }
 
-  pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny(pixcrd_obj, PyArray_DOUBLE, 2, 2);
+  pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny(pixcrd_obj, NPY_DOUBLE, 2, 2);
   if (pixcrd == NULL) {
     return NULL;
   }
@@ -306,7 +313,7 @@ Wcs_p4_pix2foc(
     goto exit;
   }
 
-  foccrd = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+  foccrd = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (foccrd == NULL) {
     status = 2;
     goto exit;
@@ -364,7 +371,7 @@ Wcs_det2im(
     return detcrd_obj;
   }
 
-  detcrd = (PyArrayObject*)PyArray_ContiguousFromAny(detcrd_obj, PyArray_DOUBLE, 2, 2);
+  detcrd = (PyArrayObject*)PyArray_ContiguousFromAny(detcrd_obj, NPY_DOUBLE, 2, 2);
   if (detcrd == NULL) {
     return NULL;
   }
@@ -374,7 +381,7 @@ Wcs_det2im(
     goto exit;
   }
 
-  imcrd = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(detcrd), PyArray_DOUBLE);
+  imcrd = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(detcrd), NPY_DOUBLE);
   if (imcrd == NULL) {
     status = 2;
     goto exit;
@@ -427,7 +434,7 @@ Wcs_pix2foc(
     return NULL;
   }
 
-  pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny(pixcrd_obj, PyArray_DOUBLE, 2, 2);
+  pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny(pixcrd_obj, NPY_DOUBLE, 2, 2);
   if (pixcrd == NULL) {
     return NULL;
   }
@@ -437,7 +444,7 @@ Wcs_pix2foc(
     goto _exit;
   }
 
-  foccrd = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+  foccrd = (PyArrayObject*)PyArray_SimpleNew(2, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (foccrd == NULL) {
     goto _exit;
   }
@@ -705,97 +712,6 @@ Wcs_set_sip(
   return 0;
 }
 
-static PyObject*
-Wcs___copy__(
-    Wcs* self,
-    /*@unused@*/ PyObject* args,
-    /*@unused@*/ PyObject* kwds) {
-
-  PyObject* copy = NULL;
-
-  copy = Wcs_new(&WcsType, NULL, NULL);
-  if (copy == NULL) {
-    return NULL;
-  }
-
-  if (self->py_det2im[0]) {
-    Wcs_set_det2im1((Wcs*)copy, self->py_det2im[0], NULL);
-  }
-
-  if (self->py_det2im[1]) {
-    Wcs_set_det2im2((Wcs*)copy, self->py_det2im[1], NULL);
-  }
-
-  if (self->py_sip) {
-    Wcs_set_sip((Wcs*)copy, self->py_sip, NULL);
-  }
-
-  if (self->py_distortion_lookup[0]) {
-    Wcs_set_cpdis1((Wcs*)copy, self->py_distortion_lookup[0], NULL);
-  }
-
-  if (self->py_distortion_lookup[1]) {
-    Wcs_set_cpdis2((Wcs*)copy, self->py_distortion_lookup[1], NULL);
-  }
-
-  if (self->py_wcsprm) {
-    Wcs_set_wcs((Wcs*)copy, self->py_wcsprm, NULL);
-  }
-
-  return copy;
-}
-
-static int
-_deepcopy_helper(
-    Wcs* copy,
-    PyObject* item,
-    int (*function)(Wcs*, PyObject*, void*),
-    PyObject* memo) {
-  PyObject* obj_copy;
-
-  if (item) {
-    obj_copy = get_deepcopy(item, memo);
-    if (obj_copy == NULL) {
-      return 1;
-    }
-
-    if (function(copy, obj_copy, NULL)) {
-      Py_DECREF(obj_copy);
-      return 1;
-    }
-
-    Py_DECREF(obj_copy);
-  }
-
-  return 0;
-}
-
-static PyObject*
-Wcs___deepcopy__(
-    Wcs* self,
-    PyObject* memo,
-    /*@unused@*/ PyObject* kwds) {
-
-  Wcs*    copy;
-
-  copy = (Wcs*)Wcs_new(&WcsType, NULL, NULL);
-  if (copy == NULL) {
-    return NULL;
-  }
-
-  if (_deepcopy_helper(copy, self->py_det2im[0], Wcs_set_det2im1, memo) ||
-      _deepcopy_helper(copy, self->py_det2im[1], Wcs_set_det2im2, memo) ||
-      _deepcopy_helper(copy, self->py_sip, Wcs_set_sip, memo) ||
-      _deepcopy_helper(copy, self->py_distortion_lookup[0], Wcs_set_cpdis1, memo) ||
-      _deepcopy_helper(copy, self->py_distortion_lookup[1], Wcs_set_det2im1, memo) ||
-      _deepcopy_helper(copy, self->py_wcsprm, Wcs_set_wcs, memo)) {
-    Py_DECREF(copy);
-    return NULL;
-  }
-
-  return (PyObject*)copy;
-}
-
 static PyObject*
 _sanity_check(
     PyObject* self,
@@ -827,8 +743,6 @@ static PyGetSetDef Wcs_getset[] = {
 
 static PyMethodDef Wcs_methods[] = {
   {"_all_pix2world", (PyCFunction)Wcs_all_pix2world, METH_VARARGS|METH_KEYWORDS, doc_all_pix2world},
-  {"__copy__", (PyCFunction)Wcs___copy__, METH_NOARGS, NULL},
-  {"__deepcopy__", (PyCFunction)Wcs___deepcopy__, METH_O, NULL},
   {"_det2im", (PyCFunction)Wcs_det2im, METH_VARARGS|METH_KEYWORDS, doc_det2im},
   {"_p4_pix2foc", (PyCFunction)Wcs_p4_pix2foc, METH_VARARGS|METH_KEYWORDS, doc_p4_pix2foc},
   {"_pix2foc", (PyCFunction)Wcs_pix2foc, METH_VARARGS|METH_KEYWORDS, doc_pix2foc},
@@ -964,16 +878,12 @@ struct module_state {
     INITERROR;
 
   import_array();
-#if defined(_MSC_VER)
-  fill_docstrings();
-#endif
 
   if (_setup_api(m)                 ||
       _setup_str_list_proxy_type(m) ||
       _setup_unit_list_proxy_type(m)||
       _setup_wcsprm_type(m)         ||
       _setup_tabprm_type(m)         ||
-      _setup_units_type(m)          ||
       /* _setup_wtbarr_type(m)         || */
       _setup_distortion_type(m)     ||
       _setup_sip_type(m)            ||
@@ -983,6 +893,16 @@ struct module_state {
     INITERROR;
   }
 
+#ifdef HAVE_WCSLIB_VERSION
+  if (PyModule_AddStringConstant(m, "__version__", wcslib_version(NULL))) {
+    INITERROR;
+  }
+#else
+  if (PyModule_AddStringConstant(m, "__version__", "4.x")) {
+    INITERROR;
+  }
+#endif
+
 #if PY3K
   return m;
 #endif
