@@ -482,6 +482,7 @@ PyWcsprm_bounds_check(
       bounds |= 1;
   }
 
+  wcsprm_python2c(&self->x);
   wcsbchk(&self->x, bounds);
 
   Py_RETURN_NONE;
@@ -1775,7 +1776,6 @@ PyWcsprm_sub(
           }
 
           element_str = PyBytes_AsString(element_utf8);
-          Py_DECREF(element_utf8); element_utf8 = NULL;
         } else if (PyBytes_Check(element)) {
           element_str = PyBytes_AsString(element);
         }
@@ -1798,6 +1798,7 @@ PyWcsprm_sub(
             "string values for axis sequence must be one of 'latitude', 'longitude', 'cubeface', 'spectral', 'stokes', or 'celestial'");
           goto exit;
         }
+        Py_CLEAR(element_utf8);
       #if PY3K
       } else if (PyLong_Check(element)) {
         tmp = (Py_ssize_t)PyLong_AsSsize_t(element);
@@ -1934,8 +1935,8 @@ PyWcsprm_to_header(
     goto exit;
   }
 
-  /* Just return the raw header string.  PyFITS on the Python side will help
-     to parse and use this information. */
+  /* Just return the raw header string.  astropy.io.fits on the Python side will
+     help to parse and use this information. */
   #if PY3K
   result = PyUnicode_FromStringAndSize(header, (Py_ssize_t)nkeyrec * 80);
   #else
