@@ -3,9 +3,12 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 import numpy as np
 from .. import units as u
+from ..extern.six.moves import range
+from .wcs import WCSSUB_LONGITUDE, WCSSUB_LATITUDE
 
 __doctest_skip__ = ['wcs_to_celestial_frame']
 
+
 __all__ = ['add_stokes_axis_to_wcs',
            'custom_frame_mappings',
            'wcs_to_celestial_frame', 'proj_plane_pixel_scales',
@@ -44,10 +47,6 @@ def _wcs_to_celestial_frame_builtin(wcs):
 
     from ..coordinates import FK4, FK4NoETerms, FK5, ICRS, Galactic
     from ..time import Time
-    from . import WCSSUB_CELESTIAL
-
-    # Keep only the celestial part of the axes
-    wcs = wcs.sub([WCSSUB_CELESTIAL])
 
     if wcs.wcs.lng == -1 or wcs.wcs.lat == -1:
         return None
@@ -59,8 +58,8 @@ def _wcs_to_celestial_frame_builtin(wcs):
     else:
         equinox = wcs.wcs.equinox
 
-    xcoord = wcs.wcs.ctype[0][:4]
-    ycoord = wcs.wcs.ctype[1][:4]
+    xcoord = wcs.wcs.ctype[wcs.wcs.lng][:4]
+    ycoord = wcs.wcs.ctype[wcs.wcs.lat][:4]
 
     # Apply logic from FITS standard to determine the default radesys
     if radesys == '' and xcoord == 'RA--' and ycoord == 'DEC-':
@@ -254,7 +253,7 @@ def proj_plane_pixel_area(wcs):
 
 
 def is_proj_plane_distorted(wcs, maxerr=1.0e-5):
-    """
+    r"""
     For a WCS returns `False` if square image (detector) pixels stay square
     when projected onto the "plane of intermediate world coordinates"
     as defined in
@@ -277,8 +276,8 @@ def is_proj_plane_distorted(wcs, maxerr=1.0e-5):
     check:
 
     .. math::
-        \\left \| \\frac{C \cdot C^{\mathrm{T}}}\
-        {| det(C)|} - I \\right \|_{\mathrm{max}} < \epsilon .
+        \left \| \frac{C \cdot C^{\mathrm{T}}}
+        {| det(C)|} - I \right \|_{\mathrm{max}} < \epsilon .
 
     Parameters
     ----------
@@ -391,7 +390,7 @@ def skycoord_to_pixel(coords, wcs, origin=0, mode='all'):
         raise ValueError("Can only handle WCS with distortions for 2-dimensional WCS")
 
     # Keep only the celestial part of the axes, also re-orders lon/lat
-    wcs = wcs.sub([WCSSUB_CELESTIAL])
+    wcs = wcs.sub([WCSSUB_LONGITUDE, WCSSUB_LATITUDE])
 
     if wcs.naxis != 2:
         raise ValueError("WCS should contain celestial component")
@@ -474,7 +473,7 @@ def pixel_to_skycoord(xp, yp, wcs, origin=0, mode='all', cls=None):
         raise ValueError("Can only handle WCS with distortions for 2-dimensional WCS")
 
     # Keep only the celestial part of the axes, also re-orders lon/lat
-    wcs = wcs.sub([WCSSUB_CELESTIAL])
+    wcs = wcs.sub([WCSSUB_LONGITUDE, WCSSUB_LATITUDE])
 
     if wcs.naxis != 2:
         raise ValueError("WCS should contain celestial component")
